import java.io.*;
import Z80SystemBus;

class Z80CPU
{
	//	Reference to system bus object
	private Z80SystemBus m_bus;
	
	// Introducing the emulated Z80 registers...
	private char m_AF = 0;
	private char m_BC = 0;
	private char m_DE = 0;
	private char m_HL = 0;
	private char m_AF_PRIME = 0;
	private char m_BC_PRIME = 0;
	private char m_DE_PRIME = 0;
	private char m_HL_PRIME = 0;
	private char m_IX = 0;
	private char m_IY = 0;
	private char m_SP = 0;
	private char m_PC = 0;
	private char m_I = 0;
	private char m_R = 0;
	private boolean m_IFF1 = false;
	private boolean m_IFF2 = false;

	char get_A()
	{
		return	(char)(m_AF>>8);
	}

	void	set_A(char newVal)
	{
		m_AF = (char)((m_AF & 0x00FF) | ((newVal & 0xFF) << 8) );
	}

	char get_F()
	{
		return	(char)(m_AF&0xFF);
	}

	void	set_F(char newVal)
	{
		m_AF = (char)((m_AF & 0xFF00) | (newVal & 0xFF));
	}

	char get_B()
	{
		return	(char)(m_BC>>8);
	}

	void	set_B(char newVal)
	{
		m_BC = (char)((m_BC & 0x00FF) | ((newVal & 0xFF) << 8) );
	}

	char get_C()
	{
		return	(char)(m_BC&0xFF);
	}

	void	set_C(char newVal)
	{
		m_BC = (char)((m_BC & 0xFF00) | (newVal & 0xFF));
	}

	char get_D()
	{
		return	(char)(m_DE>>8);
	}

	void	set_D(char newVal)
	{
		m_DE = (char)((m_DE & 0x00FF) | ((newVal & 0xFF) << 8) );
	}

	char get_E()
	{
		return	(char)(m_DE&0xFF);
	}

	void	set_E(char newVal)
	{
		m_DE = (char)((m_DE & 0xFF00) | (newVal & 0xFF));
	}

	char get_H()
	{
		return	(char)(m_HL>>8);
	}

	void	set_H(char newVal)
	{
		m_HL = (char)((m_HL & 0x00FF) | ((newVal & 0xFF) << 8) );
	}

	char get_L()
	{
		return	(char)(m_HL&0xFF);
	}

	void	set_L(char newVal)
	{
		m_HL = (char)((m_HL & 0xFF00) | (newVal & 0xFF));
	}

	char	set_HIBYTE(char Val, char bByte)
	{
		return	(char)((Val & 0x00FF) | (bByte << 8));
	}

	char	set_LOBYTE(char Val, char bByte)
	{
		return	(char)((Val & 0xFF00) | (bByte & 0xFF));
	}

	char	get_HIBYTE(char Val)
	{
		return	(char)(Val >> 8);
	}

	char	get_LOBYTE(char Val)
	{
		return	(char)(Val & 0x00FF);
	}

	private static final char m_CARRY_MASK			= (char)0x01;
	private static final char m_SUBTRACT_MASK		= (char)0x02;
	private static final char m_PARITY_MASK		= (char)0x04;
	private static final char m_OVERFLOW_MASK		= (char)0x04;
	private static final char m_HALF_CARRY_MASK	= (char)0x10;
	private static final char m_ZERO_MASK			= (char)0x40;
	private static final char m_SIGN_MASK			= (char)0x80;


	boolean SIGN_FLAG_SET()
	{
		return	(get_F() & m_SIGN_MASK)==m_SIGN_MASK;
	}

	boolean ZERO_FLAG_SET()
	{
		return	(get_F() & m_ZERO_MASK)==m_ZERO_MASK;
	}

	boolean HALF_CARRY_FLAG_SET()
	{
		return	(get_F() & m_HALF_CARRY_MASK)==m_HALF_CARRY_MASK;
	}

	boolean OVERFLOW_FLAG_SET()
	{
		return	(get_F() & m_OVERFLOW_MASK)==m_OVERFLOW_MASK;
	}

	boolean PARITY_FLAG_SET()
	{
		return	(get_F() & m_PARITY_MASK)==m_PARITY_MASK;
	}

	boolean SUBTRACT_FLAG_SET()
	{
		return	(get_F() & m_SUBTRACT_MASK)==m_SUBTRACT_MASK;
	}

	boolean CARRY_FLAG_SET()
	{
		return	(get_F() & m_CARRY_MASK)==m_CARRY_MASK;
	}

	// Cycle counter
	private int m_nCycle = 0;
	private boolean m_fHalted = false;
	private boolean m_fNMIFired = false;
	private boolean m_fInterrupt = false;
	private int m_nInterruptMode = 0;
	private char m_bInterruptData = 0;

	// Some static tables to help with parity and cycle counting
	static char m_rgbParityTable[] =
	{
		1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 
		0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 
		0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 
		1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 
		0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 
		1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 
		1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 
		0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 
		0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 
		1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 
		1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 
		0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 
		1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 
		0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 
		0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 
		1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1
	};

	static char m_rgbSingleOpcodeCycles[] =
	{
	  4, 10, 7, 6, 4, 4, 7, 4, 			// 00	- 07
	  4, 11, 7, 6, 4, 4, 7, 4, 			//	08	- 0F
	  8, 10, 7, 6, 4, 4, 7, 4, 			//	10	- 17
	  12, 11, 7, 6, 4, 4, 7, 4, 			//	18	- 1F
	  7, 10, 16, 6, 4, 4, 7, 4, 			//	20	- 27
	  7, 11, 16, 6, 4, 4, 7, 4, 			//	20	- 2F
	  7, 10, 13, 6, 11, 11, 10, 4, 		//	38	- 37
	  7, 11, 13, 6, 4, 4, 7, 4, 			//	30	- 3F
	  4, 4, 4, 4, 4, 4, 7, 4, 			//	40	- 47
	  4, 4, 4, 4, 4, 4, 7, 4, 			//	48	- 4F
	  4, 4, 4, 4, 4, 4, 7, 4, 			//	50	- 57
	  4, 4, 4, 4, 4, 4, 7, 4, 			//	50	- 5F
	  4, 4, 4, 4, 4, 4, 7, 4, 			//	68	- 67
	  4, 4, 4, 4, 4, 4, 7, 4, 			//	60	- 6F
	  7, 7, 7, 7, 7, 7, 4, 7, 			//	70	- 77
	  4, 4, 4, 4, 4, 4, 7, 4, 			//	78	- 7F
	  4, 4, 4, 4, 4, 4, 7, 4, 			//	80	- 87
	  4, 4, 4, 4, 4, 4, 7, 4, 			//	80	- 8F
	  4, 4, 4, 4, 4, 4, 7, 4, 			//	98	- 97
	  4, 4, 4, 4, 4, 4, 7, 4, 			//	90	- 9F
	  4, 4, 4, 4, 4, 4, 7, 4, 			//	A0	- A7
	  4, 4, 4, 4, 4, 4, 7, 4, 			//	A8	- AF
	  4, 4, 4, 4, 4, 4, 7, 4, 			//	B0	- B7
	  4, 4, 4, 4, 4, 4, 7, 4, 			//	B8	- BF
	  5, 10, 10, 10, 10, 11, 7, 11, 	//	C0	- C7
	  5, 10, 7, 0, 10, 17, 7, 11, 		//	C8	- CF
	  5, 10, 10, 11, 10, 11, 7, 11, 	//	D0	- D7
	  5, 4, 10, 11, 10, 0, 7, 11, 		//	D8	- DF
	  5, 10, 10, 19, 10, 11, 7, 11, 	//	E0	- E7
	  5, 4, 10, 4, 10, 0, 7, 11, 		//	E8	- EF
	  5, 10, 10, 4, 10, 11, 7, 11, 		//	F0	- F7
	  5, 6, 10, 4, 10, 0, 7, 11			//	F8	- FF
	};										 	

	static char m_rgb0xCBOpcodeCycles[] =
	{
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 12, 8, 
	  8, 8, 8, 8, 8, 8, 12, 8, 
	  8, 8, 8, 8, 8, 8, 12, 8, 
	  8, 8, 8, 8, 8, 8, 12, 8, 
	  8, 8, 8, 8, 8, 8, 12, 8, 
	  8, 8, 8, 8, 8, 8, 12, 8, 
	  8, 8, 8, 8, 8, 8, 12, 8, 
	  8, 8, 8, 8, 8, 8, 12, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	  8, 8, 8, 8, 8, 8, 15, 8, 
	};

	static char m_rgb_0xDD_and_0xFD_OpcodeCycles[] =
	{
	  0, 0, 0, 0, 0, 0, 0, 0,																	//	00	- 07
	  0, 15, 0, 0, 0, 0, 0, 0, 																//	08	- 0F
	  0, 0, 0, 0, 0, 0, 0, 0, 																	//	10	- 17
	  0, 15, 0, 0, 0, 0, 0, 0, 																//	18	- 1F
	  0, 14, 20, 10, 0, 0, 0, 0, 																//	20	- 27
	  0, 15, 20, 10, 0, 0, 0, 0, 																//	28	- 2F
	  0, 0, 0, 0, 23, 23, 19, 0, 																//	30	- 37
	  0, 15, 0, 0, 0, 0, 0, 0, 																//	38	- 3F
	  0, 0, 0, 0, 0, 0, 19, 0, 																//	40	- 47
	  0, 0, 0, 0, 0, 0, 19, 0, 																//	48	- 4F
	  0, 0, 0, 0, 0, 0, 19, 0, 																//	50	- 57
	  0, 0, 0, 0, 0, 0, 19, 0, 																//	58	- 5F
	  0, 0, 0, 0, 0, 0, 9, 0, 											//	60	- 67
	  0, 0, 0, 0, 0, 0, 9, 0, 											//	68 - 6F
	  19, 19, 19, 19, 19, 19, 19, 19, 														//	70	- 77
	  0, 0, 0, 0, 0, 0, 19, 0, 																//	78	- 7F
	  0, 0, 0, 0, 0, 0, 19, 0, 																//	80	- 87
	  0, 0, 0, 0, 0, 0, 19, 0, 																//	88	- 8F
	  0, 0, 0, 0, 0, 0, 19, 0, 																//	90	- 97
	  0, 0, 0, 0, 0, 0, 19, 0, 																//	98	- 9F
	  0, 0, 0, 0, 0, 0, 19, 0, 																//	A0	- A7
	  0, 0, 0, 0, 0, 0, 19, 0, 																//	A8	- AF
	  0, 0, 0, 0, 0, 0, 19, 0, 																//	B0	- B7
	  0, 0, 0, 0, 0, 0, 19, 0, 																//	B8	- BF
	  0, 0, 0, 0, 0, 0, 0, 0, 																	//	C0	- C7
	  0, 0, 0, 0, 0, 0, 0, 0, 																	//	C8	- CF
	  0, 0, 0, 0, 0, 0, 0, 0, 																	//	D0	- D7
	  0, 0, 0, 0, 0, 0, 0, 0, 																	//	D8	- DF
	  0, 14, 0, 23, 0, 15, 0, 0, 																//	E0	- E7
	  0, 8, 0, 0, 0, 0, 0, 0, 																	//	E8	- EF
	  0, 0, 0, 0, 0, 0, 0, 0, 																	//	F0	- F7
	  0, 10, 0, 0, 0, 0, 0, 0																	//	F8	- FF
	};

	static char m_rgb0xEDOpcodeCycles[] =
	{
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  12, 12, 15, 20, 8, 14, 8, 9, 
	  12, 12, 15, 20, 0, 14, 0, 9, 
	  12, 12, 15, 20, 0, 0, 8, 9, 
	  12, 12, 15, 20, 0, 0, 8, 9, 
	  12, 12, 15, 20, 0, 0, 0, 18, 
	  12, 12, 15, 20, 0, 0, 0, 18, 
	  12, 00, 15, 20, 0, 0, 0, 0, 
	  12, 12, 15, 20, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  16, 16, 16, 16, 0, 0, 0, 0, 
	  16, 16, 16, 16, 0, 0, 0, 0, 
	  16, 16, 16, 16, 0, 0, 0, 0, 
	  16, 16, 16, 16, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	  0, 0, 0, 0, 0, 0, 0, 0, 
	};


	static char m_rgb_0xDDCB_and_0xFDCB_OpcodeCycles[] =
	{
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  20, 20, 20, 20, 20, 20, 20, 20, 
	  20, 20, 20, 20, 20, 20, 20, 20, 
	  20, 20, 20, 20, 20, 20, 20, 20, 
	  20, 20, 20, 20, 20, 20, 20, 20, 
	  20, 20, 20, 20, 20, 20, 20, 20, 
	  20, 20, 20, 20, 20, 20, 20, 20, 
	  20, 20, 20, 20, 20, 20, 20, 20, 
	  20, 20, 20, 20, 20, 20, 20, 20, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0, 
	  0, 0, 0, 0, 0, 0, 23, 0
	};

	public Z80CPU()
	{
	}

	public boolean init(Z80SystemBus bus)
	{
		m_bus = bus;
		return	true;
	}

	//	opcode workers
	char	inline_ld_BYTE()
	{
		return	m_bus.MemReadByte(m_PC++);
	}

	char	inline_ld_WORD()
	{
		char	wData = m_bus.MemReadWord(m_PC);
		m_PC += 2;
		return	wData;
	}

	char inline_ADDW( char Reg1, char Reg2 )
	{
		int	mask = ~(m_SUBTRACT_MASK | m_CARRY_MASK);
		
		m_AF = (char)((m_AF & 0xFF00) | (get_F() & mask) |
					((((((int)Reg1) + ((int)Reg2)) & 0x10000 )==0x10000) ? m_CARRY_MASK : 0));

		return	(char)((Reg1 + Reg2) & 0xFFFF);
	}

	char	inline_INC( char Reg )
	{
		Reg++;
		m_AF =	(char)(
					(m_AF & 0xFF00) |
					(CARRY_FLAG_SET() ? m_CARRY_MASK : 0) |
					(Reg & m_SIGN_MASK) |
					(Reg>0 ? 0 : m_ZERO_MASK) |
					(Reg==0x80 ? m_OVERFLOW_MASK : 0) |
					((Reg & 0x0F)==0x0F ? 0 : m_HALF_CARRY_MASK) );
		return	(char)(Reg & 0xFF);
	}

	char	inline_DEC( char Reg )
	{
		Reg--;
		m_AF =	(char)(
					(m_AF & 0xFF00) |
					m_SUBTRACT_MASK |
					(CARRY_FLAG_SET() ? m_CARRY_MASK : 0) |
					(Reg & m_SIGN_MASK) |
					(0!=Reg ? 0 : m_ZERO_MASK) |
					(Reg==0x7F ? m_OVERFLOW_MASK : 0) |
					((Reg & 0x0F)==0x0F ? m_HALF_CARRY_MASK : 0) );
		return	(char)(Reg & 0xFF);
	}

	char	inline_POP()
	{
		char stack_data = m_bus.MemReadWord( m_SP );
		m_SP += 2;
		return	stack_data;
	}

	void	inline_PUSH(char Reg)
	{
		m_SP -= 2;
		m_bus.MemWriteWord(m_SP, Reg);
	}

	void	inline_CALL()
	{
		m_SP -= 2;
		m_bus.MemWriteWord(m_SP, (char)(m_PC + 2));
		m_PC = m_bus.MemReadWord(m_PC);
	}

	void	inline_JP()
	{
		m_PC = m_bus.MemReadWord(m_PC);
	}

	void	inline_JR()
	{
		m_PC = (char)(((int)m_PC)+ ((byte)m_bus.MemReadByte(m_PC)) + 1);
	}

	void	inline_RET()
	{
		m_PC = m_bus.MemReadWord(m_SP);
		m_SP += 2;
	}

	void	inline_RST(char Addr)
	{
		inline_PUSH(m_PC);
		m_PC = Addr;
	}

	void	inline_ADD(char Reg)
	{
		char	J = (char)(get_A() + Reg);
		set_F	((char)(
					( (((char)~(get_A() ^ Reg)) &
						(Reg ^ get_LOBYTE(J)) &
						0x80)==0x80 ? m_OVERFLOW_MASK : 0 ) |
					get_HIBYTE(J) |
					((get_LOBYTE(J)!=0) ? 0 : m_ZERO_MASK) |
					(get_LOBYTE(J) & m_SIGN_MASK) |
					((get_A() ^ Reg ^ get_LOBYTE(J)) & m_HALF_CARRY_MASK )
				));
		set_A(get_LOBYTE(J));
	}						

	void	inline_SUB(char Reg)
	{
		char	J = (char)(get_A() - Reg);
		set_F	((char)(
					( (((char)~(get_A() ^ Reg)) &
						(Reg ^ get_LOBYTE(J)) &
						0x80)==0x80 ? m_OVERFLOW_MASK : 0 ) |
					m_SUBTRACT_MASK |
					-get_HIBYTE(J) |
					((get_LOBYTE(J)!=0) ? 0 : m_ZERO_MASK) |
					(get_LOBYTE(J) & m_SIGN_MASK) |
					((get_A() ^ Reg ^ get_LOBYTE(J)) & m_HALF_CARRY_MASK )
				));
		set_A(get_LOBYTE(J));
	}

	void	inline_ADC(char Reg)
	{
		char	J = (char)(get_A() + Reg + (CARRY_FLAG_SET() ? m_CARRY_MASK : 0));
		set_F	((char)(
					( (((char)~(get_A() ^ Reg)) &
						(Reg ^ get_LOBYTE(J)) &
						0x80)==0x80 ? m_OVERFLOW_MASK : 0 ) |
					get_HIBYTE(J) |
					((get_LOBYTE(J)!=0) ? 0 : m_ZERO_MASK) |
					(get_LOBYTE(J) & m_SIGN_MASK) |
					((get_A() ^ Reg ^ get_LOBYTE(J)) & m_HALF_CARRY_MASK )
				));
		set_A(get_LOBYTE(J));
	}

	void	inline_SBC(char Reg)
	{
		char	J = (char)(get_A() - Reg - (CARRY_FLAG_SET() ? m_CARRY_MASK : 0));
		set_F	((char)(
					( (((char)~(get_A() ^ Reg)) &
						(Reg ^ get_LOBYTE(J)) &
						0x80)==0x80 ? m_OVERFLOW_MASK : 0 ) |
					m_SUBTRACT_MASK |
					-get_HIBYTE(J) |
					((get_LOBYTE(J)!=0) ? 0 : m_ZERO_MASK) |
					(get_LOBYTE(J) & m_SIGN_MASK) |
					((get_A() ^ Reg ^ get_LOBYTE(J)) & m_HALF_CARRY_MASK )
				));
		set_A(get_LOBYTE(J));
	}

	void	inline_CP(char Reg)
	{
		char	J = (char)(get_A() - Reg);
		set_F	((char)(
					( (((char)~(get_A() ^ Reg)) &
						(Reg ^ get_LOBYTE(J)) &
						0x80)==0x80 ? m_OVERFLOW_MASK : 0 ) |
					m_SUBTRACT_MASK |
					-get_HIBYTE(J) |
					((get_LOBYTE(J)!=0) ? 0 : m_ZERO_MASK) |
					(get_LOBYTE(J) & m_SIGN_MASK) |
					((get_A() ^ Reg ^ get_LOBYTE(J)) & m_HALF_CARRY_MASK )
				));
	}

	void	inline_AND(char Reg)
	{
		set_A((char)(get_A() & Reg));
		set_F((char)(
					m_HALF_CARRY_MASK + (m_rgbParityTable[ get_A() ]!=0 ? m_PARITY_MASK : 0) |
					(get_A() & m_SIGN_MASK) |
					(get_A()!=0 ? 0 : m_ZERO_MASK)
				));
	}

	void	inline_OR(char Reg)
	{
		set_A((char)(get_A() | Reg));
		set_F((char)(
					(m_rgbParityTable[ get_A() ]!=0 ? m_PARITY_MASK : 0) |
					(get_A() & m_SIGN_MASK) |
					(get_A()!=0 ? 0 : m_ZERO_MASK)
				));
	}

	void	inline_XOR(char Reg)
	{
		set_A((char)(get_A() ^ Reg));
		set_F((char)(
					(m_rgbParityTable[ get_A() ]!=0 ? m_PARITY_MASK : 0) |
					(get_A() & m_SIGN_MASK) |
					(get_A()!=0 ? 0 : m_ZERO_MASK)
				));
	}

	char	inline_RLC(char Reg)
	{
		set_F	((char)(
					((Reg & 0x80)==0x80) ? m_CARRY_MASK : 0
				));
		Reg = (char)(((Reg << 1) | get_F()) & 0x00FF);
		set_F	((char)(get_F() |
					(Reg > 0 ? 0 : m_ZERO_MASK) |
					(Reg & m_SIGN_MASK) |
					(m_rgbParityTable[ Reg ]==1 ? m_PARITY_MASK : 0)
				));
		return	Reg;
	}

	char	inline_RRC(char Reg)
	{
		set_F	((char)(
					(Reg & 0x01)==0x01 ? m_CARRY_MASK : 0
				));
		Reg = (char)(((Reg >> 1) | (get_F() > 0 ? 0x80 : 0)) & 0x00FF);
		set_F	((char)(get_F() |
					(Reg > 0 ? 0 : m_ZERO_MASK) |
					(Reg & m_SIGN_MASK) |
					(m_rgbParityTable[ Reg ]==1 ? m_PARITY_MASK : 0)
				));
		return	Reg;
	}

	char	inline_RL(char Reg)
	{
		if((Reg & 0x80)==0x80)
		{
			Reg = (char)(((Reg << 1) | (CARRY_FLAG_SET() ? m_CARRY_MASK : 0)) & 0x00FF);
			set_F	((char)(
						(Reg > 0 ? 0 : m_ZERO_MASK) |
						(Reg & m_SIGN_MASK) |
						(m_rgbParityTable[ Reg ]==1 ? m_PARITY_MASK : 0) |
						m_CARRY_MASK
					));
			return	Reg;
		}
		else
		{
			Reg = (char)(
						((Reg << 1) | (CARRY_FLAG_SET() ? m_CARRY_MASK : 0)) & 0x00FF
					);
			set_F	((char)(
						(Reg > 0 ? 0 : m_ZERO_MASK) |
						(Reg & m_SIGN_MASK) |
						(m_rgbParityTable[ Reg ]==1 ? m_PARITY_MASK : 0)
					));
			return	Reg;
		}
	}

	char	inline_RR(char Reg)
	{
		if((Reg & 0x01)==0x01)
		{
			Reg = (char)(
						((Reg >> 1) | (CARRY_FLAG_SET() ? 0x80 : 0)) & 0x00FF
					);
			set_F	((char)(
						(Reg > 0 ? 0 : m_ZERO_MASK) |
						(Reg & m_SIGN_MASK) |
						(m_rgbParityTable[ Reg ]==1 ? m_PARITY_MASK : 0) |
						m_CARRY_MASK
					));
			return	Reg;
		}
		else
		{
			Reg = (char)(
						((Reg >> 1) | (CARRY_FLAG_SET() ? 0x80 : 0)) & 0x00FF
					);
			set_F	((char)(
						(Reg > 0 ? 0 : m_ZERO_MASK) |
						(Reg & m_SIGN_MASK) |
						(m_rgbParityTable[ Reg ]==1 ? m_PARITY_MASK : 0)
					));
			return	Reg;		}
	}		

	char	inline_SLA(char Reg)
	{
		set_F	((char)(
					(Reg & 0x80)==0x80 ? m_CARRY_MASK : 0
				));
		Reg <<= 1;
		Reg &= (char)0x00FF;
		set_F	((char)(get_F() |
					(Reg > 0 ? 0 : m_ZERO_MASK) | (Reg & m_SIGN_MASK) |
					(m_rgbParityTable[ Reg ]==1 ? m_PARITY_MASK : 0)
				));
		return	Reg;
	}

	char	inline_SRA(char Reg)
	{
		set_F	((char)(
					Reg & m_CARRY_MASK
				));
		Reg = (char)(
					(Reg >> 1) | (Reg & 0x80)
				);
		set_F	((char)(get_F() |
					(Reg > 0 ? 0 : m_ZERO_MASK) |
					(Reg & m_SIGN_MASK) |
					(m_rgbParityTable[ Reg ]==1 ? m_PARITY_MASK : 0)
				));
		return	Reg;
	}

	char	inline_SLL(char Reg)
	{
		set_F	((char)(
					(Reg & 0x80)==0x80 ? m_CARRY_MASK : 0
				));
		Reg = (char)(
					((Reg << 1) | 0x01) & 0x00FF
				);
		set_F	((char)(get_F() |
					(Reg > 0 ? 0 : m_ZERO_MASK) |
					(Reg & m_SIGN_MASK) |
					(m_rgbParityTable[ Reg ]==1 ? m_PARITY_MASK : 0)
				));
		return	Reg;
	}

	char	inline_SRL(char Reg)
	{
		set_F	((char)(
					Reg & 0x01
				));
		Reg >>= 1;
		set_F	((char)(get_F() |
					(Reg > 0 ? 0 : m_ZERO_MASK) |
					(Reg & m_SIGN_MASK) |
					(m_rgbParityTable[ Reg ]==1 ? m_PARITY_MASK : 0)
				));
		return	Reg;
	}

	void	inline_BIT(char Bit, char Reg)
	{
		set_F	((char)(
					(get_F() & ((char)~(m_SUBTRACT_MASK | m_ZERO_MASK))) |
					m_HALF_CARRY_MASK |
					((Reg & (1 << Bit))==(1<<Bit) ? 0 : m_ZERO_MASK)
				));
	}

	char	inline_SET(char Bit, char Reg)
	{
		Reg |= (char)((1 << Bit) & 0x00FF);
		return	Reg;
	}

	char	inline_RES(char Bit, char Reg)
	{
		Reg &= (char)((~(1 << Bit)) & 0x00FF);
		return	Reg;
	}

	char	inline_IN()
	{
		char	Reg = inline_Inport(get_C());
		set_F	((char)(
					(m_rgbParityTable[ Reg ]==1 ? m_PARITY_MASK : 0)	|
					(CARRY_FLAG_SET() ? m_CARRY_MASK : 0) |
					(Reg & m_SIGN_MASK)	|
					(Reg>0 ? 0 : m_ZERO_MASK)
				));
		return	Reg;
	}

	void	inline_OUT(char Reg)
	{
		inline_Outport(get_C(), Reg);
	}

	void	inline_ADCW(char Reg)
	{
		char	I = CARRY_FLAG_SET() ? m_CARRY_MASK : 0;
		char	J = (char)((m_HL + Reg + I) & 0xFFFF);
		set_F	((char)(
					(((m_HL + Reg + I) & 0x1000)==0x1000 ? m_CARRY_MASK : 0)	|
					((~(m_HL ^ Reg) & (Reg ^ J) & 0x8000)==0x8000 ? m_OVERFLOW_MASK : 0) |
					(J>0 ? 0 : m_ZERO_MASK) |
					(J & m_SIGN_MASK)
				));
		m_HL = J;
	}

	void	inline_SBCW(char Reg)
	{
		char	I = CARRY_FLAG_SET() ? m_CARRY_MASK : 0;
		char	J = (char)((m_HL - Reg - I) & 0xFFFF);
		set_F	((char)(
					m_SUBTRACT_MASK	|
					(((m_HL - Reg - I) & 0x1000)==0x1000 ? m_CARRY_MASK : 0)	|
					((~(m_HL ^ Reg) & (Reg ^ J) & 0x8000)==0x8000 ? m_OVERFLOW_MASK : 0) |
					(J>0 ? 0 : m_ZERO_MASK) |
					(J & m_SIGN_MASK)
				));
		m_HL = J;
	}

	char	inline_Inport(char bPort)
	{
		return	m_bus.ReadDevice(bPort);
	}

	void	inline_Outport(char bPort, char bData)
	{
		m_bus.WriteDevice(bPort, bData);
	}

	// 
	// 																									
	//   Start single opcode instructions													
	// 																									
	// 

	//	0x00 = "nop"
	void	do_nop()
	{
	}

	// 0x01 = "ld	bc, WORD"
	void	do_ld_bc_word()
	{
		m_BC = inline_ld_WORD();
	}

	// 0x02 = "ld	(bc), a"
	void	do_ld_memat_bc_a()
	{
		m_bus.MemWriteByte(m_BC, get_A());
	}

	//	0x03 = "inc	bc"
	void	do_inc_bc()
	{
		m_BC++;
	}

	// 0x04 = "inc	b"
	void	do_inc_b()
	{
		set_B((char)(get_B()+1));
	}

	// 0x05 = "dec b"
	void	do_dec_b()
	{
		set_B((char)(get_B()-1));
	}

	//	0x06 = "ld	b, byte"
	void	do_ld_b_byte()
	{
		set_B(inline_ld_BYTE());
	}

	// 0x07 = "rlca"
	void	do_rlca()
	{	
		char	I = (get_A() & 0x80)==0x80 ? m_CARRY_MASK : 0;

		set_A((char)((get_A() << 1) | I));
		set_F((char)(((get_F() & (char)(~(m_CARRY_MASK | m_SUBTRACT_MASK | m_HALF_CARRY_MASK))) | I) & 0xFF));
	}

	//	0x08 = "ex af, af'"
	void	do_ex_af_afprime()
	{
		char	wTemp = m_AF;
			
		m_AF = m_AF_PRIME;
		m_AF_PRIME = wTemp;
	}

	// 0x09 = "add	hl, bc"
	void	do_add_hl_bc()
	{
		m_HL = inline_ADDW( m_HL, m_BC );
	}

	// 0x0A = "ld	a, (bc)"
	void	do_ld_a_memat_bc()
	{
		set_A(m_bus.MemReadByte(m_BC));
	}

	//	0x0B = "dec	bc"
	void	do_dec_bc()
	{
		m_BC--;
	}

	//	0x0C = "inc	c"
	void	do_inc_c()
	{
		set_C(inline_INC(get_C()));
	}

	//	0x0D = "dec	c"
	void	do_dec_c()
	{
		set_C(inline_DEC(get_C()));
	}    

	//	0x0E = "ld	c, BYTE"
	void	do_ld_c_BYTE()
	{
		set_C(inline_ld_BYTE());
	}

	//	0x0F = "rrca"
	void	do_rrca()
	{
		char	I = (char)(get_A() & 0x01);
		m_AF = (char)(((get_F() & ~(m_CARRY_MASK | m_SUBTRACT_MASK | m_HALF_CARRY_MASK)) | I) |
					(((get_A() >> 1) | (0!=I ? 0x80 : 0)) << 8));
	}

	//	0x10 = "djnz	offset"
	void	do_djnz_offset()
	{
		set_B((char)(get_B()-1));
		if (get_B() > 0)
		{
			inline_JR();
			m_nCycle += 5;
		}
		else
			m_PC++;
	}    

	// 0x11 = "ld	de, WORD"
	void	do_ld_de_WORD()
	{
		m_DE = inline_ld_WORD();
	}

	// 0x12 = "ld	(de), a"
	void	do_ld_memat_de_a()
	{
		m_bus.MemWriteByte( m_DE, get_A() );
	}

	//	0x13 = "inc	de"
	void	do_inc_de()
	{
		m_DE++;
	}

	//	0x14 = "inc	d"
	void	do_inc_d()
	{
		set_D(inline_INC(get_D()));
	}

	//	0x15 = "dec	d"
	void	do_dec_d()
	{
		set_D(inline_DEC(get_D()));
	}

	//	0x16 = "ld	d, BYTE"
	void	do_ld_d_BYTE()
	{
		set_D(inline_ld_BYTE());
	}

	//	0x17 = "rla"
	void	do_rla()
	{
		char	bByte = (char)(((get_A() & 0x80)==0x80) ? m_CARRY_MASK : 0);

		set_A((char)((get_A() << 1) | (get_F() & m_CARRY_MASK)));
		set_F((char)((get_F() & ~(m_CARRY_MASK | m_SUBTRACT_MASK | m_HALF_CARRY_MASK)) | bByte));
	}

	//	0x18 = "jr	offset"
	void	do_jr_offset()
	{
		inline_JR();
	}

	//	0x19 = "add	hl, de"
	void	do_add_hl_de()
	{
		m_HL = inline_ADDW(m_HL, m_DE);
	}

	//	0x1A = "ld a, (de)"
	void	do_ld_a_memat_de()
	{
		set_A(m_bus.MemReadByte(m_DE));
	}

	//	0x1B = "dec	de"
	void	do_dec_de()
	{
		m_DE--;
	}

	//	0x1C = "inc	e"
	void	do_inc_e()
	{
		set_E(inline_INC(get_E()));
	}

	//	0x1D = "dec	e"
	void	do_dec_e()
	{
		set_E(inline_DEC(get_E()));
	}

	//	0x1E = "ld	e, BYTE"
	void	do_ld_e_BYTE()
	{
		set_E(inline_ld_BYTE());
	}

	//	0x1F = "rra"
	void	do_rra()
	{
		char	bByte = (char)(get_A() & 0x01);

		set_A((char)(
				(get_A() >> 1) | (((get_F() & m_CARRY_MASK)==m_CARRY_MASK) ? 0x80 : 0)
			));
		set_F((char)(
				(get_F() & ~(m_CARRY_MASK | m_SUBTRACT_MASK | m_HALF_CARRY_MASK)) | bByte
			));
	}

	//	0x20 = "jr	nz, offset"
	void	do_jr_nz_offset()
	{
		if (!ZERO_FLAG_SET())
		{
			inline_JR();
			m_nCycle += 5;
		}
		else
			m_PC++;
	}

	//	0x21 = "ld	hl, WORD"
	void	do_ld_hl_WORD()
	{
		m_HL = inline_ld_WORD();
	}

	//	0x22 = "ld	(address), hl"
	void	do_ld_memataddr_hl ()
	{
		m_bus.MemWriteWord(m_bus.MemReadWord(m_PC), m_HL);
		m_PC += 2;
	}

	//	0x23 = "inc	hl"
	void	do_inc_hl()
	{
		m_HL++;
	}

	//	0x24 = "inc	h"
	void	do_inc_h()
	{
		set_H(inline_INC(get_H()));
	}

	//	0x25 = "dec	h"
	void	do_dec_h()
	{
		set_H(inline_DEC(get_H()));
	}

	//	0x26 = "ld	h, BYTE"
	void	do_ld_h_BYTE()
	{
		set_H(inline_ld_BYTE());
	}

	//	0x27 = "daa"
	void	do_daa()
	{
		char	J = 0;

		J = set_HIBYTE(J, (char)(get_A() & 0x0F));
		J = (char)(
		SUBTRACT_FLAG_SET() ?
		(
			CARRY_FLAG_SET() ? 
				((HALF_CARRY_FLAG_SET() ? 0x9A00 : 0xA000) + m_CARRY_MASK)
			:
				(HALF_CARRY_FLAG_SET() ? 0xFA00 : 0x0000)
		)
		:
		(
			CARRY_FLAG_SET() ?
				((HALF_CARRY_FLAG_SET() ? 0x6600 : ((get_HIBYTE(J) < 0x0A) ? 0x6000 : 0x6600)) + m_CARRY_MASK)
			:
			(
				HALF_CARRY_FLAG_SET() ?
					((get_A() < 0xA0) ? 0x0600 : (0x6600 + m_CARRY_MASK))
				:
				(
					(get_HIBYTE(J) < 0x0A) ?
						((get_A() < 0xA0) ? 0x0000 : (0x6000 + m_CARRY_MASK))
					:
						((get_A() < 0x90) ? 0x0600 : (0x6600 + m_CARRY_MASK))
				)
			)
		));

		set_A((char)(get_A() + get_HIBYTE(J)));
		set_F	((char)(
					(m_rgbParityTable[ get_A() ]==1 ? m_PARITY_MASK : 0) |
					get_LOBYTE(J) | (SUBTRACT_FLAG_SET() ? m_SUBTRACT_MASK : 0) |
					(get_A() > 0 ? 0 : m_ZERO_MASK) | (get_A() & m_SIGN_MASK)
				));
	}


	//	0x28 = "jr	z, offset"
	void	do_jr_z_offset()
	{
		if (ZERO_FLAG_SET())
		{
			inline_JR();
			m_nCycle += 5;
		}
		else
			m_PC++;
	}

	//	0x29 = "add	hl, hl"
	void	do_add_hl_hl()
	{
		m_HL = inline_ADDW(m_HL, m_HL);
	}

	//	0x2A = "ld	hl, (address)"
	void	do_ld_hl_memataddr ()
	{
		m_HL = m_bus.MemReadWord(m_bus.MemReadWord(m_PC));
		m_PC += 2;
	}

	//	0x2B = "dec	hl"
	void	do_dec_hl()
	{
		m_HL--;
	}

	//	0x2C = "inc	l"
	void	do_inc_l()
	{
		set_L(inline_INC(get_L()));
	}

	//	0x2D = "dec	l"
	void	do_dec_l()
	{
		set_L(inline_DEC(get_L()));
	}

	//	0x2E = "ld	l, BYTE"
	void	do_ld_l_BYTE()
	{
		set_L(inline_ld_BYTE());
	}

	//	0x2F = "cpl"
	void	do_cpl()
	{
		set_A((char)~get_A());
		m_AF |= (m_HALF_CARRY_MASK | m_SUBTRACT_MASK);
	}

	//	0x30 = "jr	nc, offset"
	void	do_jr_nc_offset()
	{
		if (!CARRY_FLAG_SET())
		{
			inline_JR();
			m_nCycle += 5;
		}
		else
			m_PC++;
	}

	//	0x31 = "ld	sp, WORD"
	void	do_ld_sp_WORD()
	{
		m_SP = inline_ld_WORD();
	}

	//	0x32 = "ld	(address), a"
	void	do_ld_memataddr_a()
	{
		m_bus.MemWriteByte(m_bus.MemReadWord(m_PC), get_A());
		m_PC += 2;
	}

	//	0x33 = "inc	sp"
	void	do_inc_sp()
	{
		m_SP++;
	}

	//	0x34 = "inc (hl)"
	void	do_inc_memat_hl()
	{
		char	bByte = m_bus.MemReadByte(m_HL);
		bByte = inline_INC(bByte);
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0x35 = "dec	(hl)"
	void	do_dec_memat_hl()
	{
		char	bByte = m_bus.MemReadByte(m_HL);
		bByte = inline_DEC(bByte);
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0x36 = "ld	(hl), BYTE"
	void	do_ld_memat_hl_BYTE()
	{
		m_bus.MemWriteByte(m_HL, m_bus.MemReadByte(m_PC++));
	}

	//	0x37 = "scf"
	void	do_scf()
	{
		set_F((char)((get_F() | m_CARRY_MASK) & ~(m_SUBTRACT_MASK | m_HALF_CARRY_MASK)));
	}

	//	0x38 = "jr	c, offset"
	void	do_jr_c_offset()
	{
		if (CARRY_FLAG_SET())
		{
			inline_JR();
			m_nCycle += 5;	
		}
		else
			m_PC++;
	}

	//	0x39 = "add	hl, sp"
	void	do_add_hl_sp()
	{
		m_HL = inline_ADDW(m_HL, m_SP);
	}

	//	0x3A = "ld	a, (address)"
	void	do_ld_a_memataddr()
	{
		set_A(m_bus.MemReadByte(m_bus.MemReadWord(m_PC)));
		m_PC += 2;
	}

	//	0x3B = "dec	sp"
	void	do_dec_sp()
	{
		m_SP--;
	}

	//	0x3C = "inc	a"
	void	do_inc_a()
	{
		set_A(inline_INC(get_A()));
	}

	//	0x3D = "dec	a"
	void	do_dec_a()
	{
		set_A(inline_DEC(get_A()));
	}

	//	0x3E = "ld	a, BYTE"
	void	do_ld_a_BYTE()
	{
		set_A(inline_ld_BYTE());
	}

	//	0x3F = "ccf"
	void	do_ccf()
	{
		set_F((char)((get_F() ^ m_CARRY_MASK) & ~m_SUBTRACT_MASK));
	}

	//	0x40 = "ld	b, b"
	void	do_ld_b_b()
	{
		//		This is worthless except for that of wasting
		//		CPU cycles.  We'll just ignore it and handle
		//		just like NOP.
	}

	// 0x41 = "ld	b, c"
	void	do_ld_b_c()
	{
		set_B(get_C());
	}

	//	0x42 = "ld	b, d"
	void	do_ld_b_d()
	{
		set_B(get_D());
	}

	//	0x43 = "ld	b, e"
	void	do_ld_b_e()
	{
		set_B(get_E());
	}

	//	0x44 = "ld	b, h"
	void	do_ld_b_h()
	{
		set_B(get_H());
	}

	//	0x45 = "ld	b, l"
	void	do_ld_b_l()
	{
		set_B(get_L());
	}

	//	0x46 = "ld	b, (hl)"
	void	do_ld_b_memat_hl()
	{
		set_B(m_bus.MemReadByte(m_HL));
	}

	// 0x47 = "ld	b, a"
	void	do_ld_b_a()
	{
		set_B(get_A());
	}

	//	0x48 = "ld	c, b"
	void	do_ld_c_b()
	{
		set_C(get_B());
	}

	//	0x49 = "ld	c, c"
	void	do_ld_c_c()
	{
		//		This is worthless except for that of wasting
		//		CPU cycles.  We'll just ignore it and handle
		//		just like NOP.
	}

	//	0x4A = "ld	c, d"
	void	do_ld_c_d()
	{
		set_C(get_D());
	}

	//	0x4B = "ld	c, e"
	void	do_ld_c_e()
	{
		set_C(get_E());
	}

	//	0x4C = "ld	c, h"
	void	do_ld_c_h()
	{
		set_C(get_H());
	}

	//	0x4D = "ld	c, l"
	void	do_ld_c_l()
	{
		set_C(get_L());
	}

	//	0x4E = "ld	c, (hl)"
	void	do_ld_c_memat_hl()
	{
		set_C(m_bus.MemReadByte(m_HL));
	}

	//	0x4F = "ld	c, a"
	void	do_ld_c_a()
	{
		set_C(get_A());
	}

	//	0x50 = "ld	d, b"
	void	do_ld_d_b()
	{
		set_D(get_B());
	}

	// 0x51 = "ld  d, c"
	void	do_ld_d_c()
	{
		set_D(get_C());
	}

	//	0x52 = "ld	d, d"
	void	do_ld_d_d()
	{
		//		This is worthless except for that of wasting
		//		CPU cycles.  We'll just ignore it and handle
		//		just like NOP.
	}

	//	0x53 = "ld	d, e"
	void	do_ld_d_e()
	{    
		set_D(get_E());
	}

	//	0x54 = "ld	d, h"
	void	do_ld_d_h()
	{
		set_D(get_H());
	}

	//	0x55 = "ld	d, l"
	void	do_ld_d_l()
	{
		set_D(get_L());
	}

	//	0x56 = "ld	d, (hl)"
	void	do_ld_d_memat_hl()
	{
		set_D(m_bus.MemReadByte(m_HL));
	}

	//	0x57 = "ld	d, a"
	void	do_ld_d_a()
	{
		set_D(get_A());
	}

	// 0x58 = "ld  e, b"
	void	do_ld_e_b()
	{
		set_E(get_B());
	}

	//	0x59 = "ld	e, c"
	void	do_ld_e_c()
	{
		set_E(get_C());
	}

	// 0x5A = "ld	e, d"
	void	do_ld_e_d()
	{
		set_E(get_D());
	}

	//	0x5B = "ld	e, e"
	void	do_ld_e_e()
	{
		//		This is worthless except for that of wasting
		//		CPU cycles.  We'll just ignore it and handle
		//		just like NOP.
	}

	//	0x5C = "ld	e, h"
	void	do_ld_e_h()
	{
		set_E(get_H());
	}

	//	0x5D = "ld	e, l"
	void	do_ld_e_l()
	{
		set_E(get_L());
	}

	//	0x5E = "ld	e, (hl)"
	void	do_ld_e_memat_hl()
	{
		set_E(m_bus.MemReadByte(m_HL));
	}

	//	0x5F = "ld	e, a"
	void	do_ld_e_a()
	{
		set_E(get_A());
	}

	//	0x60 = "ld	h, b"
	void	do_ld_h_b()
	{
		set_H(get_B());
	}

	// 0x61 = "ld	h, c"
	void	do_ld_h_c()
	{
		set_H(get_C());
	}

	//	0x62 = "ld	h, d"
	void	do_ld_h_d()
	{
		set_H(get_D());
	}

	// 0x63 = "ld	h, e"
	void	do_ld_h_e()
	{
		set_H(get_E());
	}

	// 0x64 = "ld	h, h"
	void	do_ld_h_h()
	{
		//		This is worthless except for that of wasting
		//		CPU cycles.  We'll just ignore it and handle
		//		just like NOP.
	}

	//	0x65 = "ld	h, l"
	void	do_ld_h_l()
	{
		set_H(get_L());
	}

	//	0x66 = "ld h, (hl)"
	void	do_ld_h_memat_hl()
	{
		set_H(m_bus.MemReadByte(m_HL));
	}

	//	0x67 = "ld	h, a"
	void	do_ld_h_a()
	{
		set_H(get_A());
	}

	//	0x68 = "ld	l, b"
	void	do_ld_l_b()
	{
		set_L(get_B());
	}

	// 0x69 = "ld	l, c"
	void	do_ld_l_c()
	{
		set_L(get_C());
	}

	//	0x6A = "ld	l, d"
	void	do_ld_l_d()
	{
		set_L(get_D());
	}

	//	0x6B = "ld	l, e"
	void	do_ld_l_e()
	{
		set_L(get_E());
	}

	//	0x6C = "ld	l, h"
	void	do_ld_l_h()
	{
		set_L(get_H());
	}

	//	0x6D = "ld	l, l"
	void	do_ld_l_l()
	{
		//		This is worthless except for that of wasting
		//		CPU cycles.  We'll just ignore it and handle
		//		just like NOP.
	}

	//	0x6E = "ld	l, (hl)"
	void	do_ld_l_memat_hl()
	{
		set_L(m_bus.MemReadByte(m_HL));
	}

	//	0x6F = "ld	l, a"
	void	do_ld_l_a()
	{
		set_L(get_A());
	}

	//	0x70 = "ld	(hl), b"
	void	do_ld_memat_hl_b()
	{
		m_bus.MemWriteByte(m_HL, get_B());
	}

	//	0x71 = "ld	(hl), c"
	void	do_ld_memat_hl_c()
	{
		m_bus.MemWriteByte(m_HL, get_C());
	}

	// 0x72 = "ld	(hl), d"
	void	do_ld_memat_hl_d()
	{
		m_bus.MemWriteByte(m_HL, get_D());
	}

	// 0x73 = "ld	(hl), e"
	void	do_ld_memat_hl_e()
	{
		m_bus.MemWriteByte(m_HL, get_E());
	}

	//	0x74 = "ld	(hl), h"
	void	do_ld_memat_hl_h()
	{
		m_bus.MemWriteByte(m_HL, get_H());
	}

	//	0x75 = "ld	(hl), l"
	void	do_ld_memat_hl_l()
	{
		m_bus.MemWriteByte(m_HL, get_L());
	}

	//	0x76 = "halt"
	void	do_halt()
	{
		m_PC--;
		m_fHalted = true;	// Flag that CPU is halted.
	}

	//	0x77 = "ld	(hl), a"
	void	do_ld_memat_hl_a()
	{
		m_bus.MemWriteByte(m_HL, get_A());
	}

	//	0x78 = "ld	a, b"
	void	do_ld_a_b()
	{
		set_A(get_B());
	}

	//	0x79 = "ld	a, c"
	void	do_ld_a_c()
	{
		set_A(get_C());
	}

	//	0x7A = "ld	a, d"
	void	do_ld_a_d()
	{
		set_A(get_D());
	}

	//	0x7B = "ld	a, e"
	void	do_ld_a_e()
	{
		set_A(get_E());
	}

	//	0x7C = "ld	a, h"
	void	do_ld_a_h()
	{
		set_A(get_H());
	}

	//	0x7D = "ld	a, l"
	void	do_ld_a_l()
	{
		set_A(get_L());
	}

	//	0x7E = "ld, a, (hl)"
	void	do_ld_a_memat_hl()
	{
		set_A(m_bus.MemReadByte(m_HL));
	}

	//	0x7F = "ld	a, a"
	void	do_ld_a_a()
	{
		//		This is worthless except for that of wasting
		//		CPU cycles.  We'll just ignore it and handle
		//		just like NOP.
	}

	//	0x80 = "add	a, b"
	void	do_add_a_b ()
	{
		inline_ADD(get_B());
	}

	//	0x81 = "add	a, c"
	void	do_add_a_c ()
	{
		inline_ADD(get_C());
	}

	//	0x82 = "add	a, d"
	void	do_add_a_d ()
	{
		inline_ADD(get_D());
	}

	//	0x83 = "add	a, e"
	void	do_add_a_e ()
	{
		inline_ADD(get_E());
	}

	//	0x84 = "add	a, h"
	void	do_add_a_h ()
	{
		inline_ADD(get_H());
	}

	// 0x85 = "add	a, l"
	void	do_add_a_l ()
	{
		inline_ADD(get_L());
	}

	//	0x86 = "add	a, (hl)"
	void	do_add_a_memat_hl()
	{
		char	bByte = m_bus.MemReadByte(m_HL);
		inline_ADD( bByte );
	}

	// 0x87 = "add	a, a"
	void	do_add_a_a ()
	{ 
		inline_ADD(get_A());
	}

	//	0x88 = "adc	a, b"
	void	do_adc_a_b ()
	{
		inline_ADC(get_B());
	}

	// 0x89 = "adc	a, c"
	void	do_adc_a_c ()
	{
		inline_ADC(get_C());
	}

	//	0x8A = "adc	a, d"
	void	do_adc_a_d ()
	{ 
		inline_ADC(get_D());
	}

	//	0x8B = "adc	a, e"
	void	do_adc_a_e ()
	{ 
		inline_ADC(get_E());
	}

	//	0x8C = "adc	a, h"
	void	do_adc_a_h ()
	{ 
		inline_ADC(get_H());
	} 

	//	0x8D = "adc	a, l"
	void	do_adc_a_l ()
	{ 
		inline_ADC(get_L());
	}

	//	0x8E = "adc	a, (hl)"
	void	do_adc_a_memat_hl()
	{ 
		char	bByte = m_bus.MemReadByte(m_HL);
		inline_ADC(bByte);
	}

	//	0x8F = "adc	a, a"
	void	do_adc_a_a ()
	{
		inline_ADC(get_A());
	}

	// 0x90 = "sub	a, b"
	void	do_sub_a_b ()
	{
		inline_SUB(get_B());
	}

	//	0x91 = "sub	a, c"
	void	do_sub_a_c ()
	{
		inline_SUB(get_C());
	}

	//	0x92 = "sub	a, d"
	void	do_sub_a_d ()
	{
		inline_SUB(get_D());
	}

	//	0x93 = "sub	a, e"
	void	do_sub_a_e ()
	{
		inline_SUB(get_E());
	}

	//	0x94 = "sub	a, h"
	void	do_sub_a_h ()
	{
		inline_SUB(get_H());
	}

	//	0x95 = "sub	a, l"
	void	do_sub_a_l ()
	{
		inline_SUB(get_L());
	}

	//	0x96 = "sub	a, (hl)"
	void	do_sub_a_memat_hl()
	{
		char	bByte = m_bus.MemReadByte(m_HL);
		inline_SUB(bByte);
	}

	//	0x97 = "sub	a, a"
	void	do_sub_a_a ()
	{
		inline_SUB(get_A());
	}

	//	0x98 = "sbc	a, b"
	void	do_sbc_a_b ()
	{
		inline_SBC(get_B());
	}

	//	0x99 = "sbc	a, c"
	void	do_sbc_a_c ()
	{
		inline_SBC(get_C());
	}

	//	0x9A = "sbc	a, d"
	void	do_sbc_a_d ()
	{
		inline_SBC(get_D());
	}

	// 0x9B = "sbc	a, e"
	void	do_sbc_a_e ()
	{
		inline_SBC(get_E());
	}

	// 0x9C = "sbc	a, h"
	void	do_sbc_a_h ()
	{
		inline_SBC(get_H());
	}

	//	0x9D = "sbc	a, l"
	void	do_sbc_a_l ()
	{
		inline_SBC(get_L());
	}

	// 0x9E = "sbc	a, (hl)"
	void	do_sbc_a_memat_hl()
	{
		char	bByte = m_bus.MemReadByte(m_HL);
		inline_SBC(bByte);
	}

	//	0x9F = "sbc	a, a"
	void	do_sbc_a_a ()
	{
		inline_SBC(get_A());
	}

	//	0xA0 = "and	b"
	void	do_and_b()
	{
		inline_AND(get_B());
	}

	//	0xA1 = "and	c"
	void	do_and_c()
	{
		inline_AND(get_C());
	}

	//	0xA2 = "and	d"
	void	do_and_d()
	{
		inline_AND(get_D());
	}

	//	0xA3 = "and	e"
	void	do_and_e()
	{
		inline_AND(get_E());
	}

	//	0xA4 = "and	h"
	void	do_and_h()
	{
		inline_AND(get_H());
	}

	//	0xA5 = "and	l"
	void	do_and_l()
	{
		inline_AND(get_L());
	}

	//	0xA6 = "and	(hl)"
	void	do_and_memat_hl()
	{
		char	bByte = m_bus.MemReadByte(m_HL);
		inline_AND(bByte);
	}

	//	0xA7 = "and	a"
	void	do_and_a()
	{
		inline_AND(get_A());
	}

	//	0xA8 = "xor	b"
	void	do_xor_b()
	{
		inline_XOR(get_B());
	}

	//	0xA9 = "xor	c"
	void	do_xor_c()
	{
		inline_XOR(get_C());
	}

	//	0xAA = "xor	d"
	void	do_xor_d()
	{
		inline_XOR(get_D());
	}

	//	0xAB = "xor	e"
	void	do_xor_e()
	{
		inline_XOR(get_E());
	}

	//	0xAC = "xor	h"
	void	do_xor_h()
	{
		inline_XOR(get_H());
	}

	//	0xAD = "xor	l"
	void	do_xor_l()
	{
		inline_XOR(get_L());
	}

	//	0xAE = "xor	(hl)"
	void	do_xor_memat_hl()
	{
		char	bByte = m_bus.MemReadByte(m_HL);
		inline_XOR(bByte);
	}

	//	0xAF = "xor	a"
	void	do_xor_a()
	{
		inline_XOR(get_A());
	}

	//	0xB0 = "or	b"
	void	do_or_b()
	{
		inline_OR(get_B());
	}

	//	0xB1 = "or	c"
	void	do_or_c()
	{
		inline_OR(get_C());
	}

	//	0xB2 = "or	d"
	void	do_or_d()
	{
		inline_OR(get_D());
	}

	//	0xB3 = "or	e"
	void	do_or_e()
	{
		inline_OR(get_E());
	}

	//	0xB4 = "or	h"
	void	do_or_h()
	{
		inline_OR(get_H());
	}

	//	0xB5 = "or	l"
	void	do_or_l()
	{
		inline_OR(get_L());
	}

	//	0xB6 = "or	(hl)"
	void	do_or_memat_hl()
	{
		char	bByte = m_bus.MemReadByte(m_HL);
		inline_OR(bByte);
	}

	// 0xB7 = "or  a"
	void	do_or_a()
	{
		inline_OR(get_A());
	}

	//	0xB8 = "cp	b"
	void	do_cp_b()
	{
		inline_CP(get_B());
	}

	//	0xB9 = "cp	c"
	void	do_cp_c()
	{
		inline_CP(get_C());
	}

	//	0xBA = "cp	d"
	void	do_cp_d()
	{
		inline_CP(get_D());
	}

	//	0xBB = "cp	e"
	void	do_cp_e()
	{
		inline_CP(get_E());
	}

	//	0xBC = "cp	h"
	void	do_cp_h()
	{
		inline_CP(get_H());
	}

	//	0xBD = "cp	l"
	void	do_cp_l()
	{
		inline_CP(get_L());
	}

	//	0xBE = "cp	(hl)"
	void	do_cp_memat_hl()
	{
		char	bByte = m_bus.MemReadByte(m_HL);
		inline_CP(bByte);
	}

	//	0xBF = "cp	a"
	void	do_cp_a()
	{
		inline_CP(get_A());
	}

	//	0xC0 = "ret	nz"
	void	do_ret_nz()
	{
		if (!ZERO_FLAG_SET())
		{
			inline_RET();
			m_nCycle += 6;
		}
	}

	//	0xC1 = "pop	bc"
	void	do_pop_bc()
	{
		m_BC = inline_POP();
	}

	//	0xC2 = "jp	nz, address"
	void	do_jp_nz_addr()
	{
		if (!ZERO_FLAG_SET())
			inline_JP();
		else
			m_PC += 2;
	}

	//	0xC3 = "jp	address"
	void	do_jp_addr ()
	{
		inline_JP();
	}

	//	0xC4 = "call	nz, address"
	void	do_call_nz_addr()
	{
		if (!ZERO_FLAG_SET())
		{
			inline_CALL();
			m_nCycle += 7;
		}
		else
			m_PC += 2;
	}

	//	0xC5 = "push	bc"
	void	do_push_bc ()
	{
		inline_PUSH(m_BC);
	}

	//	0xC6 = "add	a, BYTE"
	void	do_add_a_BYTE()
	{
		inline_ADD(inline_ld_BYTE());
	}

	//	0xC7 = "rst	00h"
	void	do_rst_0x00()
	{
		inline_RST((char)(0x0000));
	}

	//	0xC8 = "ret	z"
	void	do_ret_z()
	{
		if (ZERO_FLAG_SET())
		{
			inline_RET();
			m_nCycle += 6;
		}
	}

	//	0xC9 = "ret"
	void	do_ret()
	{
		inline_RET();
	}

	//	0xCA = "jp	z, address"
	void	do_jp_z_address()
	{
		if (ZERO_FLAG_SET())
		{
			inline_JP();
			m_nCycle += 5;
		}
		else
			m_PC += 2;
	}

	//	0xCB = "CB .. extended instruction"
	void	do_0xCB_instruction()
	{
		Exec0xCBOpcode(m_bus.MemReadByte(m_PC++));
	}

	//	0xCC = "call	z, address"
	void	do_call_z_addr()
	{
		if (ZERO_FLAG_SET())
		{
			inline_CALL();
			m_nCycle += 7;
		}
		else
			m_PC += 2;
	}

	//	0xCD = "call	address"
	void	do_call_addr()
	{
		inline_CALL();
	}

	//	0xCE = "adc	a, BYTE"
	void	do_adc_a_BYTE()
	{ 
		inline_ADC(inline_ld_BYTE());
	} 

	//	0xCF = "rst	08h"
	void	do_rst_0x08()
	{
		inline_RST((char)(0x0008));
	}

	//	0xD0 = "ret	nc"
	void	do_ret_nc()
	{
		if (!CARRY_FLAG_SET())
		{
			inline_RET();
			m_nCycle += 6;
		}
	}

	//	0xD1 = "pop	de"
	void	do_pop_de()
	{
		m_DE = inline_POP();
	}

	//	0xD2 = "jp	nc, address"
	void	do_jp_nc_addr()
	{
		if (!CARRY_FLAG_SET())
			inline_JP();
		else
			m_PC += 2;
	}

	//	0xD3 = "out	port, a"
	void	do_out_port_a()
	{
		inline_Outport(m_bus.MemReadByte(m_PC++), get_A());
	}

	//	0xD4 = "call	nc, address"
	void	do_call_nc_addr()
	{
		if (!CARRY_FLAG_SET())
		{
			inline_CALL();
			m_nCycle += 7;
		}
		else
			m_PC += 2;
	}

	//	0xD5 = "push	de"
	void	do_push_de ()
	{
		inline_PUSH(m_DE);
	}

	//	0xD6 = "sub	a, BYTE"
	void	do_sub_a_BYTE()
	{
		inline_SUB(inline_ld_BYTE());
	}

	//	0xD7 = "rst	10h"
	void	do_rst_0x10()
	{
		inline_RST((char)(0x0010));
	}

	//	0xD8 = "ret	c"
	void	do_ret_c()
	{
		if (CARRY_FLAG_SET())
		{
			inline_RET();
			m_nCycle += 6;
		}
	}

	//	0xD9 = "exx"
	void	do_exx()
	{
		char	wTemp;

		wTemp = m_BC_PRIME;
		m_BC_PRIME = m_BC;
		m_BC = wTemp;
		wTemp = m_DE_PRIME;
		m_DE_PRIME = m_DE;
		m_DE = wTemp;
		wTemp = m_HL_PRIME;
		m_HL_PRIME = m_HL;
		m_HL = wTemp;
	}

	//	0xDA = "jp	c, address"
	void	do_jp_c_addr()
	{
		if (CARRY_FLAG_SET())
			inline_JP();
		else
			m_PC += 2;
	}

	//	0xDB = "in	a, (port)"
	void	do_in_a_port()
	{
		set_A(inline_Inport(m_bus.MemReadByte(m_PC++)));
	}

	//	0xDC = "call	c, address"
	void	do_call_c_addr()
	{
		if (CARRY_FLAG_SET())
		{
			inline_CALL();
			m_nCycle += 7;
		}
		else
			m_PC += 2;
	}

	//	0xDD = "DDh .. extended instruction
	void	do_0xDD_instruction()
	{
		m_IX = Exec0xDD_or_0xFDOpcode(m_bus.MemReadByte(m_PC++), m_IX);
	}

	//	0xDE = "sbc	a, BYTE"
	void	do_sbc_a_BYTE()
	{
		inline_SBC(inline_ld_BYTE());
	}

	//	0xDF = "rst	18h"
	void	do_rst_0x18()
	{
		inline_RST((char)(0x0018));
	}

	//	0xE0 = "ret	po"
	void	do_ret_po()
	{
		if (!PARITY_FLAG_SET())
		{
			inline_RET();
			m_nCycle += 6;
		}
	}

	//	0xE1 = "pop	hl"
	void	do_pop_hl()
	{
		m_HL = inline_POP();
	}

	//	0xE2 = "jp	po, address"
	void	do_jp_po_addr()
	{
		if (!PARITY_FLAG_SET())
			inline_JP();
		else
			m_PC += 2;
	}

	//	0xE3 = "ex	(sp), hl"
	void	do_ex_memat_sp_hl()
	{
		char	wTemp = m_bus.MemReadWord(m_SP);
		m_bus.MemWriteWord(m_SP, m_HL);
		m_HL = wTemp;
	}

	//	0xE4 = "call	po, address"
	void	do_call_po_addr()
	{
		if (!PARITY_FLAG_SET())
		{
			inline_CALL();
			m_nCycle += 7;
		}
		else
			m_PC += 2;
	}

	//	0xE5 = "push	hl"
	void	do_push_hl ()
	{
		inline_PUSH(m_HL);
	}

	//	0xE6 = "and	BYTE"
	void	do_and_BYTE()
	{
		inline_AND(inline_ld_BYTE());
	}

	//	0xE7 = "rst	20h"
	void	do_rst_0x20()
	{
		inline_RST((char)(0x0020));
	}

	//	0xE8 = "ret	pe"
	void	do_ret_pe()
	{
		if (PARITY_FLAG_SET())
		{
			inline_RET();
			m_nCycle += 6;
		}
	}

	//	0xE9 = "jp	(hl)"
	void	do_jp_memat_hl()
	{
		m_PC = m_HL;
	}

	//	0xEA = "jp	pe, address"
	void	do_jp_pe_addr()
	{
		if (PARITY_FLAG_SET())
			inline_JP();
		else
			m_PC += 2;
	}

	//	0xEB = "ex	de, hl"
	void	do_ex_de_hl()
	{
		char wTemp = m_DE;
		m_DE = m_HL;
		m_HL = wTemp;
	}

	//	0xEC = "call	pe, address"
	void	do_call_pe_addr()
	{
		if (PARITY_FLAG_SET())
		{
			inline_CALL();
			m_nCycle += 7;
		}
		else
			m_PC += 2;
	}

	//	0xED = "EDh .. extended instruction"
	void	do_0xED_instruction()
	{
		Exec0xEDOpcode(m_bus.MemReadByte(m_PC++));
	}

	//	0xEE = "xor	BYTE"
	void	do_xor_BYTE()
	{
		inline_XOR(inline_ld_BYTE());
	}

	//	0xEF = "rst	28h"
	void	do_rst_0x28()
	{
		inline_RST((char)(0x0028));
	}

	//	0xF0 = "ret	p"
	void	do_ret_p()
	{
		if (!SIGN_FLAG_SET())
		{
			inline_RET();
			m_nCycle += 6;
		}
	}

	//	0xF1 = "pop	af"
	void	do_pop_af()
	{
		m_AF = inline_POP();
	}

	//	0xF2 = "jp	p, address"
	void	do_jp_p_addr()
	{
		if (!SIGN_FLAG_SET())
			inline_JP();
		else
			m_PC += 2;
	}

	//	0xF3 = "di"
	void	do_di()
	{
		 m_IFF1 = false;
	}

	//	0xF4 = "call	p, address"
	void	do_call_p_addr()
	{
		if (!SIGN_FLAG_SET())
		{
			inline_CALL();
			m_nCycle += 7;
		}
		else
			m_PC += 2;
	}

	//	0xF5 = "push	af"
	void	do_push_af ()
	{
		inline_PUSH(m_AF);
	}

	//	0xF6 = "or	BYTE"
	void	do_or_BYTE()
	{
		inline_OR(inline_ld_BYTE());
	}    

	//	0xF7 = "rst	30h"
	void	do_rst_0x30()
	{
		inline_RST((char)(0x0030));
	}

	//	0xF8 = "ret	m"
	void	do_ret_m()
	{
		if (SIGN_FLAG_SET())
		{
			inline_RET();
			m_nCycle += 6;
		}
	}

	//	0xF9 = "ld	sp, hl"
	void	do_ld_sp_hl()
	{
		m_SP = m_HL;
	}

	//	0xFA = "jp	m, address"
	void	do_jp_m_addr()
	{
		if (SIGN_FLAG_SET())
			inline_JP();
		else
			m_PC += 2;
	}    

	//	0xFB = "ei"     
	void	do_ei()
	{
		//	TODO:	This is supposed to enable interrupts after the 
		//			next instruction instead of right now!!
		m_IFF1 = m_IFF2 = true;
	}

	// 0xFC = "call	m, address"
	void	do_call_m_addr()
	{
		if (SIGN_FLAG_SET())
		{
			inline_CALL();
			m_nCycle += 7;
		}
		else
			m_PC += 2;
	}

	//	0xFD = "FDh .. extended instruction"
	void	do_0xFD_instruction()
	{
		m_IY = Exec0xDD_or_0xFDOpcode(m_bus.MemReadByte(m_PC++), m_IY);
	}

	//	0xFE = "cp	BYTE"
	void	do_cp_BYTE()
	{
		inline_CP(inline_ld_BYTE());
	}

	//	0xFF = "rst	38h"
	void	do_rst_0x38()
	{
		inline_RST((char)(0x0038));
	}

// 
// 																									
//   End single opcode instructions														
// 																									
// 


// 
// 																									
//   Start 0xCB opcode instructions														
// 																									
// 

	//	0x00    "rlc	b"
	void	do_rlc_b()
	{
		set_B(inline_RLC(get_B()));
	}

	//	0x01    "rlc	c"
	void	do_rlc_c()
	{
		set_C(inline_RLC(get_C()));
	}

	//	0x02    "rlc	d"
	void	do_rlc_d()
	{
		set_D(inline_RLC(get_D()));
	}

	//	0x03    "rlc	e"
	void	do_rlc_e()
	{
		set_E(inline_RLC(get_E()));
	}

	//	0x04    "rlc	h"
	void	do_rlc_h()
	{
		set_H(inline_RLC(get_H()));
	}

	//	0x05    "rlc	l"
	void	do_rlc_l()
	{
		set_L(inline_RLC(get_L()));
	}

	//	0x06    "rlc	(hl)"
	void	do_rlc_memat_hl ()
	{
		char	bByte = inline_RLC(m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0x07    "rlc	a"
	void	do_rlc_a()
	{
		set_A(inline_RLC(get_A()));
	}

	//	0x08    "rrc	b"
	void	do_rrc_b()
	{
		set_B(inline_RRC(get_B()));
	}

	//	0x09    "rrc	c"
	void	do_rrc_c()
	{
		set_C(inline_RRC(get_C()));
	}

	//	0x0A    "rrc	d"
	void	do_rrc_d()
	{
		set_D(inline_RRC(get_D()));
	}

	//	0x0B    "rrc	e"
	void	do_rrc_e()
	{
		set_E(inline_RRC(get_E()));
	}

	//	0x0C    "rrc	h"
	void	do_rrc_h()
	{
		set_H(inline_RRC(get_H()));
	}

	//	0x0D    "rrc	l"
	void	do_rrc_l()
	{
		set_L(inline_RRC(get_L()));
	}

	//	0x0E    "rrc	(hl)"
	void	do_rrc_memat_hl ()
	{
		char	bByte = inline_RRC(m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0x0F    "rrc	a"
	void	do_rrc_a()
	{
		set_A(inline_RRC(get_A()));
	}

	//	0x10    "rl	b"
	void	do_rl_b ()
	{
		set_B(inline_RL(get_B()));
	}

	//	0x11    "rl	c"
	void	do_rl_c ()
	{
		set_C(inline_RL(get_C()));
	}

	//	0x12    "rl	d"
	void	do_rl_d ()
	{
		set_D(inline_RL(get_D()));
	}

	//	0x13    "rl	e"
	void	do_rl_e ()
	{
		set_E(inline_RL(get_E()));
	}

	//	0x14    "rl	h"
	void	do_rl_h ()
	{
		set_H(inline_RL(get_H()));
	}

	//	0x15    "rl	l"
	void	do_rl_l ()
	{
		set_L(inline_RL(get_L()));
	}

	//	0x16    "rl	(hl)"
	void	do_rl_memat_hl()
	{
		char	bByte = inline_RL(m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0x17    "rl	a"
	void	do_rl_a ()
	{
		set_A(inline_RL(get_A()));
	}

	//	0x18    "rr	b"
	void	do_rr_b ()
	{
		set_B(inline_RR(get_B()));
	}

	//	0x19    "rr	c"
	void	do_rr_c ()
	{
		set_C(inline_RR(get_C()));
	}

	//	0x1A    "rr	d"
	void	do_rr_d ()
	{
		set_D(inline_RR(get_D()));
	}

	//	0x1B    "rr	e"
	void	do_rr_e ()
	{
		set_E(inline_RR(get_E()));
	}

	//	0x1C    "rr	h"
	void	do_rr_h ()
	{
		set_H(inline_RR(get_H()));
	}

	//	0x1D    "rr	l"
	void	do_rr_l ()
	{
		set_L(inline_RR(get_L()));
	}

	//	0x1E    "rr	(hl)"
	void	do_rr_memat_hl ()
	{
		char	bByte = inline_RR(m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0x1F    "rr	a"
	void	do_rr_a ()
	{
		set_A(inline_RR(get_A()));
	}

	//	0x20    "sla	b"
	void	do_sla_b()
	{
		set_B(inline_SLA(get_B()));
	}

	//	0x21    "sla	c"
	void	do_sla_c()
	{
		set_C(inline_SLA(get_C()));
	}

	//	0x22    "sla	d"
	void	do_sla_d()
	{
		set_D(inline_SLA(get_D()));
	}
			
	//	0x23    "sla	e"
	void	do_sla_e()
	{
		set_E(inline_SLA(get_E()));
	}
			
	//	0x24    "sla	h"
	void	do_sla_h()
	{
		set_H(inline_SLA(get_H()));
	}
			
	//	0x25    "sla	l"
	void	do_sla_l()
	{
		set_L(inline_SLA(get_L()));
	}
			
	//	0x26    "sla	(hl)"
	void	do_sla_memat_hl ()
	{
		char	bByte = inline_SLA(m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0x27    "sla	a"
	void	do_sla_a()
	{
		set_A(inline_SLA(get_A()));
	}

	//	0x28    "sra	b"
	void	do_sra_b()
	{
		set_B(inline_SRA(get_B()));
	}

	//	0x29    "sra	c"
	void	do_sra_c()
	{
		set_C(inline_SRA(get_C()));
	}

	//	0x2A    "sra	d"
	void	do_sra_d()
	{
		set_D(inline_SRA(get_D()));
	}

	//	0x2B    "sra	e"
	void	do_sra_e()
	{
		set_E(inline_SRA(get_E()));
	}

	//	0x2C    "sra	h"
	void	do_sra_h()
	{
		set_H(inline_SRA(get_H()));
	}

	//	0x2D    "sra	l"
	void	do_sra_l()
	{
		set_L(inline_SRA(get_L()));
	}
			
	//	0x2E    "sra	(hl)"
	void	do_sra_memat_hl ()
	{
		char	bByte = inline_SRA(m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0x2F    "sra	a"
	void	do_sra_a()
	{
		set_A(inline_SRA(get_A()));
	}

	//	0x30    "sll	b"
	void	do_sll_b			()
	{
		set_B(inline_SLL(get_B()));
	}

	//	0x31    "sll	c"
	void	do_sll_c			()
	{
		set_C(inline_SLL(get_C()));
	}

	//	0x32    "sll	d"
	void	do_sll_d			()
	{
		set_D(inline_SLL(get_D()));
	}

	//	0x33    "sll	e"
	void	do_sll_e			()
	{
		set_E(inline_SLL(get_E()));
	}

	//	0x34    "sll	h"
	void	do_sll_h			()
	{
		set_H(inline_SLL(get_H()));
	}

	//	0x35    "sll	l"
	void	do_sll_l			()
	{
		set_L(inline_SLL(get_L()));
	}

	//	0x36    "sll	(hl)"
	void	do_sll_memat_hl	()
	{
		char	bByte = inline_SLL(m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0x37    "sll	a"
	void	do_sll_a			()
	{
		set_A(inline_SLL(get_A()));
	}

	//	0x38    "srl	b"
	void	do_srl_b()
	{
		set_B(inline_SRL(get_B()));
	}
			
	//	0x39    "srl	c"
	void	do_srl_c()
	{
		set_C(inline_SRL(get_C()));
	}
			
	//	0x3A    "srl	d"
	void	do_srl_d()
	{
		set_D(inline_SRL(get_D()));
	}
			
	//	0x3B    "srl	e"
	void	do_srl_e()
	{
		set_E(inline_SRL(get_E()));
	}
			
	//	0x3C    "srl	h"
	void	do_srl_h()
	{
		set_H(inline_SRL(get_H()));
	}
			
	//	0x3D    "srl	l"
	void	do_srl_l()
	{
		set_L(inline_SRL(get_L()));
	}

	//	0x3E    "srl	(hl)"
	void	do_srl_memat_hl ()
	{
		char	bByte = inline_SRL(m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0x3F    "srl	a"
	void	do_srl_a()
	{
		set_A(inline_SRL(get_A()));
	}

	//	0x40    "bit	0, b"
	void	do_bit_0_b     ()
	{
		inline_BIT((char)0, get_B());
	} 

	//	0x41    "bit	0, c"
	void	do_bit_0_c     ()
	{
		inline_BIT((char)0, get_C());
	}

	//	0x42    "bit	0, d"
	void	do_bit_0_d     ()
	{
		inline_BIT((char)0, get_D());
	}

	//	0x43    "bit	0, e"
	void	do_bit_0_e     ()
	{
		inline_BIT((char)0, get_E());
	}

	//	0x44    "bit	0, h"
	void	do_bit_0_h     ()
	{
		inline_BIT((char)0, get_H());
	}

	//	0x45    "bit	0, l"
	void	do_bit_0_l     ()
	{
		inline_BIT((char)0, get_L());
	}

	//	0x46    "bit	0, (hl)"
	void	do_bit_0_memat_hl      ()
	{
		inline_BIT((char)0, m_bus.MemReadByte(m_HL));
	}

	//	0x47    "bit	0, a"
	void	do_bit_0_a     ()
	{
		inline_BIT((char)0, get_A());
	}

	//	0x48    "bit	1, b"
	void	do_bit_1_b     ()
	{
		inline_BIT((char)1, get_B());
	}

	//	0x49    "bit	1, c"
	void	do_bit_1_c     ()
	{
		inline_BIT((char)1, get_C());
	}

	//	0x4A    "bit	1, d"
	void	do_bit_1_d     ()
	{
		inline_BIT((char)1, get_D());
	}

	//	0x4B    "bit	1, e"
	void	do_bit_1_e     ()
	{
		inline_BIT((char)1, get_E());
	}

	//	0x4C    "bit	1, h"
	void	do_bit_1_h     ()
	{
		inline_BIT((char)1, get_H());
	}

	//	0x4D    "bit	1, l"
	void	do_bit_1_l     ()
	{
		inline_BIT((char)1, get_L());
	}

	//	0x4E    "bit	1, (hl)"
	void	do_bit_1_memat_hl      ()
	{
		inline_BIT((char)1, m_bus.MemReadByte(m_HL));
	}

	//	0x4F    "bit	1, a"
	void	do_bit_1_a     ()
	{
		inline_BIT((char)1, get_A());
	}

	//	0x50    "bit	2, b"
	void	do_bit_2_b     ()
	{
		inline_BIT((char)2, get_B());
	}

	//	0x51    "bit	2, c"
	void	do_bit_2_c     ()
	{
		inline_BIT((char)2, get_C());
	}

	//	0x52    "bit	2, d"
	void	do_bit_2_d     ()
	{
		inline_BIT((char)2, get_D());
	}

	//	0x53    "bit	2, e"
	void	do_bit_2_e     ()
	{
		inline_BIT((char)2, get_E());
	}

	//	0x54    "bit	2, h"
	void	do_bit_2_h     ()
	{
		inline_BIT((char)2, get_H());
	}

	//	0x55    "bit	2, l"
	void	do_bit_2_l     ()
	{
		inline_BIT((char)2, get_L());
	}

	//	0x56    "bit	2, (hl)"
	void	do_bit_2_memat_hl      ()
	{
		inline_BIT((char)2, m_bus.MemReadByte(m_HL));
	}

	//	0x57    "bit	2, a"
	void	do_bit_2_a     ()
	{
		inline_BIT((char)2, get_A());
	}

	//	0x58    "bit	3, b"
	void	do_bit_3_b     ()
	{
		inline_BIT((char)3, get_B());
	}

	//	0x59    "bit	3, c"
	void	do_bit_3_c     ()
	{
		inline_BIT((char)3, get_C());
	}

	//	0x5A    "bit	3, d"
	void	do_bit_3_d     ()
	{
		inline_BIT((char)3, get_D());
	}

	//	0x5B    "bit	3, e"
	void	do_bit_3_e     ()
	{
		inline_BIT((char)3, get_E());
	}

	//	0x5C    "bit	3, h"
	void	do_bit_3_h     ()
	{
		inline_BIT((char)3, get_H());
	}

	//	0x5D    "bit	3, l"
	void	do_bit_3_l     ()
	{
		inline_BIT((char)3, get_L());
	}

	//	0x5E    "bit	3, (hl)"
	void	do_bit_3_memat_hl      ()
	{
		inline_BIT((char)3, m_bus.MemReadByte(m_HL));
	}

	//	0x5F    "bit	3, a"
	void	do_bit_3_a     ()
	{
		inline_BIT((char)3, get_A());
	}

	//	0x60    "bit	4, b"
	void	do_bit_4_b     ()
	{
		inline_BIT((char)4, get_B());
	}

	//	0x61    "bit	4, c"
	void	do_bit_4_c     ()
	{
		inline_BIT((char)4, get_C());
	}

	//	0x62    "bit	4, d"
	void	do_bit_4_d     ()
	{
		inline_BIT((char)4, get_D());
	}

	//	0x63    "bit	4, e"
	void	do_bit_4_e     ()
	{
		inline_BIT((char)4, get_E());
	}

	//	0x64    "bit	4, h"
	void	do_bit_4_h     ()
	{
		inline_BIT((char)4, get_H());
	}

	//	0x65    "bit	4, l"
	void	do_bit_4_l     ()
	{
		inline_BIT((char)4, get_L());
	}

	//	0x66    "bit	4, (hl)"
	void	do_bit_4_memat_hl      ()
	{
		inline_BIT((char)4, m_bus.MemReadByte(m_HL));
	}

	//	0x67    "bit	4, a"
	void	do_bit_4_a     ()
	{
		inline_BIT((char)4, get_A());
	}

	//	0x68    "bit	5, b"
	void	do_bit_5_b     ()
	{
		inline_BIT((char)5, get_B());
	}

	//	0x69    "bit	5, c"
	void	do_bit_5_c     ()
	{
		inline_BIT((char)5, get_C());
	}

	//	0x6A    "bit	5, d"
	void	do_bit_5_d     ()
	{
		inline_BIT((char)5, get_D());
	}

	//	0x6B    "bit	5, e"
	void	do_bit_5_e     ()
	{
		inline_BIT((char)5, get_E());
	}

	//	0x6C    "bit	5, h"
	void	do_bit_5_h     ()
	{
		inline_BIT((char)5, get_H());
	}

	//	0x6D    "bit	5, l"
	void	do_bit_5_l     ()
	{
		inline_BIT((char)5, get_L());
	}

	//	0x6E    "bit	5, (hl)"
	void	do_bit_5_memat_hl      ()
	{
		inline_BIT((char)5, m_bus.MemReadByte(m_HL));
	}

	//	0x6F    "bit	5, a"
	void	do_bit_5_a     ()
	{
		inline_BIT((char)5, get_A());
	}

	//	0x70    "bit	6, b"
	void	do_bit_6_b     ()
	{
		inline_BIT((char)6, get_B());
	}

	//	0x71    "bit	6, c"
	void	do_bit_6_c     ()
	{
		inline_BIT((char)6, get_C());
	}

	//	0x72    "bit	6, d"
	void	do_bit_6_d     ()
	{
		inline_BIT((char)6, get_D());
	}

	//	0x73    "bit	6, e"
	void	do_bit_6_e     ()
	{
		inline_BIT((char)6, get_E());
	}

	//	0x74    "bit	6, h"
	void	do_bit_6_h     ()
	{
		inline_BIT((char)6, get_H());
	}

	//	0x75    "bit	6, l"
	void	do_bit_6_l     ()
	{
		inline_BIT((char)6, get_L());
	}

	//	0x76    "bit	6, (hl)"
	void	do_bit_6_memat_hl      ()
	{
		inline_BIT((char)6, m_bus.MemReadByte(m_HL));
	}

	//	0x77    "bit	6, a"
	void	do_bit_6_a     ()
	{
		inline_BIT((char)6, get_A());
	}

	//	0x78    "bit	7, b"
	void	do_bit_7_b     ()
	{
		inline_BIT((char)7, get_B());
	}

	//	0x79    "bit	7, c"
	void	do_bit_7_c     ()
	{
		inline_BIT((char)7, get_C());
	}

	//	0x7A    "bit	7, d"
	void	do_bit_7_d     ()
	{
		inline_BIT((char)7, get_D());
	}

	//	0x7B    "bit	7, e"
	void	do_bit_7_e     ()
	{
		inline_BIT((char)7, get_E());
	}

	//	0x7C    "bit	7, h"
	void	do_bit_7_h     ()
	{
		inline_BIT((char)7, get_H());
	}

	//	0x7D    "bit	7, l"
	void	do_bit_7_l     ()
	{
		inline_BIT((char)7, get_L());
	}

	//	0x7E    "bit	7, (hl)"
	void	do_bit_7_memat_hl      ()
	{
		inline_BIT((char)7, m_bus.MemReadByte(m_HL));
	}

	//	0x7F    "bit	7, a"
	void	do_bit_7_a     ()
	{
		inline_BIT((char)7, get_A());
	}

	//	0x80    "res	0, b"
	void	do_res_0_b     ()
	{
		set_B(inline_RES((char)0, get_B()));
	} 

	//	0x81    "res	0, c"
	void	do_res_0_c     ()
	{
		set_C(inline_RES((char)0, get_C()));
	}

	//	0x82    "res	0, d"
	void	do_res_0_d     ()
	{
		set_D(inline_RES((char)0, get_D()));
	}

	//	0x83    "res	0, e"
	void	do_res_0_e     ()
	{
		set_E(inline_RES((char)0, get_E()));
	}

	//	0x84    "res	0, h"
	void	do_res_0_h     ()
	{
		set_H(inline_RES((char)0, get_H()));
	}

	//	0x85    "res	0, l"
	void	do_res_0_l     ()
	{
		set_L(inline_RES((char)0, get_L()));
	}

	//	0x86    "res	0, (hl)"
	void	do_res_0_memat_hl      ()
	{
		char bByte = inline_RES((char)0, m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0x87    "res	0, a"
	void	do_res_0_a     ()
	{
		set_A(inline_RES((char)0, get_A()));
	}

	//	0x88    "res	1, b"
	void	do_res_1_b     ()
	{
		set_B(inline_RES((char)1, get_B()));
	}

	//	0x89    "res	1, c"
	void	do_res_1_c     ()
	{
		set_C(inline_RES((char)1, get_C()));
	}

	//	0x8A    "res	1, d"
	void	do_res_1_d     ()
	{
		set_D(inline_RES((char)1, get_D()));
	}

	//	0x8B    "res	1, e"
	void	do_res_1_e     ()
	{
		set_E(inline_RES((char)1, get_E()));
	}

	//	0x8C    "res	1, h"
	void	do_res_1_h     ()
	{
		set_H(inline_RES((char)1, get_H()));
	}

	//	0x8D    "res	1, l"
	void	do_res_1_l     ()
	{
		set_L(inline_RES((char)1, get_L()));
	}

	//	0x8E    "res	1, (hl)"
	void	do_res_1_memat_hl      ()
	{
		char	bByte = inline_RES((char)1, m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0x8F    "res	1, a"
	void	do_res_1_a     ()
	{
		set_A(inline_RES((char)1, get_A()));
	}

	//	0x90    "res	2, b"
	void	do_res_2_b     ()
	{
		set_B(inline_RES((char)2, get_B()));
	}

	//	0x91    "res	2, c"
	void	do_res_2_c     ()
	{
		set_C(inline_RES((char)2, get_C()));
	}

	//	0x92    "res	2, d"
	void	do_res_2_d     ()
	{
		set_D(inline_RES((char)2, get_D()));
	}

	//	0x93    "res	2, e"
	void	do_res_2_e     ()
	{
		set_E(inline_RES((char)2, get_E()));
	}

	//	0x94    "res	2, h"
	void	do_res_2_h     ()
	{
		set_H(inline_RES((char)2, get_H()));
	} 

	//	0x95    "res	2, l"
	void	do_res_2_l     ()
	{
		set_L(inline_RES((char)2, get_L()));
	} 

	//	0x96    "res	2, (hl)"
	void	do_res_2_memat_hl      ()
	{
		char	bByte = inline_RES((char)2, m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0x97    "res	2, a"
	void	do_res_2_a     ()
	{
		set_A(inline_RES((char)2, get_A()));
	}

	//	0x98    "res	3, b"
	void	do_res_3_b     ()
	{
		set_B(inline_RES((char)3, get_B()));
	}

	//	0x99    "res	3, c"
	void	do_res_3_c     ()
	{
		set_C(inline_RES((char)3, get_C()));
	}

	//	0x9A    "res	3, d"
	void	do_res_3_d     ()
	{
		set_D(inline_RES((char)3, get_D()));
	}

	//	0x9B    "res	3, e"
	void	do_res_3_e     ()
	{
		set_E(inline_RES((char)3, get_E()));
	}

	//	0x9C    "res	3, h"
	void	do_res_3_h     ()
	{
		set_H(inline_RES((char)3, get_H()));
	}

	//	0x9D    "res	3, l"
	void	do_res_3_l     ()
	{
		set_L(inline_RES((char)3, get_L()));
	}

	//	0x9E    "res	3, (hl)"
	void	do_res_3_memat_hl      ()
	{
		char	bByte = inline_RES((char)3, m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0x9F    "res	3, a"
	void	do_res_3_a     ()
	{
		set_A(inline_RES((char)3, get_A()));
	}

	//	0xA0    "res	4, b"
	void	do_res_4_b     ()
	{
		set_B(inline_RES((char)4, get_B()));
	}

	//	0xA1    "res	4, c"
	void	do_res_4_c     ()
	{
		set_C(inline_RES((char)4, get_C()));
	}

	//	0xA2    "res	4, d"
	void	do_res_4_d     ()
	{
		set_D(inline_RES((char)4, get_D()));
	}

	//	0xA3    "res	4, e"
	void	do_res_4_e     ()
	{
		set_E(inline_RES((char)4, get_E()));
	}

	//	0xA4    "res	4, h"
	void	do_res_4_h     ()
	{
		set_H(inline_RES((char)4, get_H()));
	}

	//	0xA5    "res	4, l"
	void	do_res_4_l     ()
	{
		set_L(inline_RES((char)4, get_L()));
	}

	//	0xA6    "res	4, (hl)"
	void	do_res_4_memat_hl      ()
	{
		char	bByte = inline_RES((char)4, m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0xA7    "res	4, a"
	void	do_res_4_a     ()
	{
		set_A(inline_RES((char)4, get_A()));
	}

	//	0xA8    "res	5, b"
	void	do_res_5_b     ()
	{
		set_B(inline_RES((char)5, get_B()));
	}

	//	0xA9    "res	5, c"
	void	do_res_5_c     ()
	{
		set_C(inline_RES((char)5, get_C()));
	}

	//	0xAA    "res	5, d"
	void	do_res_5_d     ()
	{
		set_D(inline_RES((char)5, get_D()));
	}

	//	0xAB    "res	5, e"
	void	do_res_5_e     ()
	{
		set_E(inline_RES((char)5, get_E()));
	}

	//	0xAC    "res	5, h"
	void	do_res_5_h     ()
	{
		set_H(inline_RES((char)5, get_H()));
	}

	//	0xAD    "res	5, l"
	void	do_res_5_l     ()
	{
		set_L(inline_RES((char)5, get_L()));
	}

	//	0xAE    "res	5, (hl)"
	void	do_res_5_memat_hl      ()
	{
		char	bByte = inline_RES((char)5, m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0xAF    "res	5, a"
	void	do_res_5_a     ()
	{
		set_A(inline_RES((char)5, get_A()));
	}

	//	0xB0    "res	6, b"
	void	do_res_6_b     ()
	{
		set_B(inline_RES((char)6, get_B()));
	}

	//	0xB1    "res	6, c"
	void	do_res_6_c     ()
	{
		set_C(inline_RES((char)6, get_C()));
	}

	//	0xB2    "res	6, d"
	void	do_res_6_d     ()
	{
		set_D(inline_RES((char)6, get_D()));
	}

	//	0xB3    "res	6, e"
	void	do_res_6_e     ()
	{ 
		set_E(inline_RES((char)6, get_E()));
	}

	//	0xB4    "res	6, h"
	void	do_res_6_h     ()
	{
		set_H(inline_RES((char)6, get_H()));
	}

	//	0xB5    "res	6, l"
	void	do_res_6_l     ()
	{
		set_L(inline_RES((char)6, get_L()));
	}

	//	0xB6    "res	6, (hl)"
	void	do_res_6_memat_hl      ()
	{
		char	bByte = inline_RES((char)6, m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0xB7    "res	6, a"
	void	do_res_6_a     ()
	{
		set_A(inline_RES((char)6, get_A()));
	}

	//	0xB8    "res	7, b"
	void	do_res_7_b     ()
	{
		set_B(inline_RES((char)7, get_B()));
	}

	//	0xB9    "res	7, c"
	void	do_res_7_c     ()
	{
		set_C(inline_RES((char)7, get_C()));
	}

	//	0xBA    "res	7, d"
	void	do_res_7_d     ()
	{
		set_D(inline_RES((char)7, get_D()));
	}

	//	0xBB    "res	7, e"
	void	do_res_7_e     ()
	{
		set_E(inline_RES((char)7, get_E()));
	}

	//	0xBC    "res	7, h"
	void	do_res_7_h     ()
	{
		set_H(inline_RES((char)7, get_H()));
	}

	//	0xBD    "res	7, l"
	void	do_res_7_l     ()
	{
		set_L(inline_RES((char)7, get_L()));
	}

	//	0xBE    "res	7, (hl)"
	void	do_res_7_memat_hl      ()
	{
		char	bByte = inline_RES((char)7, m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0xBF    "res	7, a"
	void	do_res_7_a     ()
	{
		set_A(inline_RES((char)7, get_A()));
	}

	//	0xC0    "set	0, b"
	void	do_set_0_b     ()
	{
		set_B(inline_SET((char)0, get_B()));
	}

	//	0xC1    "set	0, c"
	void	do_set_0_c     ()
	{
		set_C(inline_SET((char)0, get_C()));
	}

	//	0xC2    "set	0, d"
	void	do_set_0_d     ()
	{
		set_D(inline_SET((char)0, get_D()));
	}

	//	0xC3    "set	0, e"
	void	do_set_0_e     ()
	{
		set_E(inline_SET((char)0, get_E()));
	}

	//	0xC4    "set	0, h"
	void	do_set_0_h     ()
	{
		set_H(inline_SET((char)0, get_H()));
	}

	//	0xC5    "set	0, l"
	void	do_set_0_l     ()
	{
		set_L(inline_SET((char)0, get_L()));
	}

	//	0xC6    "set	0, (hl)"
	void	do_set_0_memat_hl      ()
	{
		char	bByte = inline_SET((char)0, m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0xC7    "set	0, a"
	void	do_set_0_a     ()
	{
		set_A(inline_SET((char)0, get_A()));
	}

	//	0xC8    "set	1, b"
	void	do_set_1_b     ()
	{
		set_B(inline_SET((char)1, get_B()));
	}

	//	0xC9    "set	1, c"
	void	do_set_1_c     ()
	{
		set_C(inline_SET((char)1, get_C()));
	}

	//	0xCA    "set	1, d"
	void	do_set_1_d     ()
	{
		set_D(inline_SET((char)1, get_D()));
	}

	//	0xCB    "set	1, e"
	void	do_set_1_e     ()
	{
		set_E(inline_SET((char)1, get_E()));
	}

	//	0xCC    "set	1, h"
	void	do_set_1_h     ()
	{
		set_H(inline_SET((char)1, get_H()));
	}

	//	0xCD    "set	1, l"
	void	do_set_1_l     ()
	{
		set_L(inline_SET((char)1, get_L()));
	}

	//	0xCE    "set	1, (hl)"
	void	do_set_1_memat_hl      ()
	{
		char	bByte = inline_SET((char)1, m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0xCF    "set	1, a"
	void	do_set_1_a     ()
	{
		set_A(inline_SET((char)1, get_A()));
	}

	//	0xD0    "set	2, b"
	void	do_set_2_b     ()
	{
		set_B(inline_SET((char)2, get_B()));
	}

	//	0xD1    "set	2, c"
	void	do_set_2_c     ()
	{
		set_C(inline_SET((char)2, get_C()));
	}

	//	0xD2    "set	2, d"
	void	do_set_2_d     ()
	{
		set_D(inline_SET((char)2, get_D()));
	}

	//	0xD3    "set	2, e"
	void	do_set_2_e     ()
	{
		set_E(inline_SET((char)2, get_E()));
	}

	//	0xD4    "set	2, h"
	void	do_set_2_h     ()
	{
		set_H(inline_SET((char)2, get_H()));
	}

	//	0xD5    "set	2, l"
	void	do_set_2_l     ()
	{
		set_L(inline_SET((char)2, get_L()));
	}

	//	0xD6    "set	2, (hl)"
	void	do_set_2_memat_hl      ()
	{
		char	bByte = inline_SET((char)2, m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0xD7    "set	2, a"
	void	do_set_2_a     ()
	{
		set_A(inline_SET((char)2, get_A()));
	}

	//	0xD8    "set	3, b"
	void	do_set_3_b     ()
	{
		set_B(inline_SET((char)3, get_B()));
	}

	//	0xD9    "set	3, c"
	void	do_set_3_c     ()
	{
		set_C(inline_SET((char)3, get_C()));
	}

	//	0xDA    "set	3, d"
	void	do_set_3_d     ()
	{
		set_D(inline_SET((char)3, get_D()));
	}

	//	0xDB    "set	3, e"
	void	do_set_3_e     ()
	{
		set_E(inline_SET((char)3, get_E()));
	}

	//	0xDC    "set	3, h"
	void	do_set_3_h     ()
	{
		set_H(inline_SET((char)3, get_H()));
	}

	//	0xDD    "set	3, l"
	void	do_set_3_l     ()
	{
		set_L(inline_SET((char)3, get_L()));
	}
		
	//	0xDE    "set	3, (hl)"
	void	do_set_3_memat_hl      ()
	{
		char	bByte = inline_SET((char)3, m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0xDF    "set	3, a"
	void	do_set_3_a     ()
	{
		set_A(inline_SET((char)3, get_A()));
	}

	//	0xE0    "set	4, b"
	void	do_set_4_b     ()
	{
		set_B(inline_SET((char)4, get_B()));
	}

	//	0xE1    "set	4, c"
	void	do_set_4_c     ()
	{
		set_C(inline_SET((char)4, get_C()));
	}

	//	0xE2    "set	4, d"
	void	do_set_4_d     ()
	{
		set_D(inline_SET((char)4, get_D()));
	}

	//	0xE3    "set	4, e"
	void	do_set_4_e     ()
	{
		set_E(inline_SET((char)4, get_E()));
	}

	//	0xE4    "set	4, h"
	void	do_set_4_h     ()
	{
		set_H(inline_SET((char)4, get_H()));
	}

	//	0xE5    "set	4, l"
	void	do_set_4_l     ()
	{
		set_L(inline_SET((char)4, get_L()));
	}

	//	0xE6    "set	4, (hl)"
	void	do_set_4_memat_hl      ()
	{
		char	bByte = inline_SET((char)4, m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0xE7    "set	4, a"
	void	do_set_4_a     ()
	{
		set_A(inline_SET((char)4, get_A()));
	}

	//	0xE8    "set	5, b"
	void	do_set_5_b     ()
	{
		set_B(inline_SET((char)5, get_B()));
	}

	//	0xE9    "set	5, c"
	void	do_set_5_c     ()
	{
		set_C(inline_SET((char)5, get_C()));
	}

	//	0xEA    "set	5, d"
	void	do_set_5_d     ()
	{
		set_D(inline_SET((char)5, get_D()));
	}

	//	0xEB    "set	5, e"
	void	do_set_5_e     ()
	{
		set_E(inline_SET((char)5, get_E()));
	}

	//	0xEC    "set	5, h"
	void	do_set_5_h     ()
	{
		set_H(inline_SET((char)5, get_H()));
	}

	//	0xED    "set	5, l"
	void	do_set_5_l     ()
	{
		set_L(inline_SET((char)5, get_L()));
	}
		
	//	0xEE    "set	5, (hl)"
	void	do_set_5_memat_hl      ()
	{
		char	bByte = inline_SET((char)5, m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0xEF    "set	5, a"
	void	do_set_5_a     ()
	{
		set_A(inline_SET((char)5, get_A()));
	}

	//	0xF0    "set	6, b"
	void	do_set_6_b     ()
	{
		set_B(inline_SET((char)6, get_B()));
	}

	//	0xF1    "set	6, c"
	void	do_set_6_c     ()
	{
		set_C(inline_SET((char)6, get_C()));
	}

	//	0xF2    "set	6, d"
	void	do_set_6_d     ()
	{
		set_D(inline_SET((char)6, get_D()));
	}

	//	0xF3    "set	6, e"
	void	do_set_6_e     ()
	{
		set_E(inline_SET((char)6, get_E()));
	}

	//	0xF4    "set	6, h"
	void	do_set_6_h     ()
	{
		set_H(inline_SET((char)6, get_H()));
	}

	//	0xF5    "set	6, l"
	void	do_set_6_l     ()
	{
		set_L(inline_SET((char)6, get_L()));
	}

	//	0xF6    "set	6, (hl)"
	void	do_set_6_memat_hl      ()
	{
		char	bByte = inline_SET((char)6, m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}
		
	//	0xF7    "set	6, a"
	void	do_set_6_a     ()
	{
		set_A(inline_SET((char)6, get_A()));
	}

	//	0xF8    "set	7, b"
	void	do_set_7_b     ()
	{
		set_B(inline_SET((char)7, get_B()));
	}

	//	0xF9    "set	7, c"
	void	do_set_7_c     ()
	{
		set_C(inline_SET((char)7, get_C()));
	}

	//	0xFA    "set	7, d"
	void	do_set_7_d     ()
	{
		set_D(inline_SET((char)7, get_D()));
	}

	//	0xFB    "set	7, e"
	void	do_set_7_e     ()
	{
		set_E(inline_SET((char)7, get_E()));
	}

	//	0xFC    "set	7, h"
	void	do_set_7_h     ()
	{
		set_H(inline_SET((char)7, get_H()));
	}

	//	0xFD    "set	7, l"
	void	do_set_7_l     ()
	{
		set_L(inline_SET((char)7, get_L()));
	}

	//	0xFE    "set	7, (hl)"
	void	do_set_7_memat_hl      ()
	{
		char	bByte = inline_SET((char)7, m_bus.MemReadByte(m_HL));
		m_bus.MemWriteByte(m_HL, bByte);
	}

	//	0xFF    "set	7, a"
	void	do_set_7_a     ()
	{
		set_A(inline_SET((char)7, get_A()));
	}

	// 
	// 																									
	//   End 0xCB opcode instructions															
	// 																									
	// 

	// 
	//                                                                          
	//   Start 0xED opcode instructions														
	//                                                                          
	// 

	//      0x00    "invalid opcode"
	//      0x01    "invalid opcode"
	//      0x02    "invalid opcode"
	//      0x03    "invalid opcode"
	//      0x04    "invalid opcode"
	//      0x05    "invalid opcode"
	//      0x06    "invalid opcode"
	//      0x07    "invalid opcode"
	//      0x08    "invalid opcode"
	//      0x09    "invalid opcode"
	//      0x0A    "invalid opcode"
	//      0x0B    "invalid opcode"
	//      0x0C    "invalid opcode"
	//      0x0D    "invalid opcode"
	//      0x0E    "invalid opcode"
	//      0x0F    "invalid opcode"
	//      0x10    "invalid opcode"
	//      0x11    "invalid opcode"
	//      0x12    "invalid opcode"
	//      0x13    "invalid opcode"
	//      0x14    "invalid opcode"
	//      0x15    "invalid opcode"
	//      0x16    "invalid opcode"
	//      0x17    "invalid opcode"
	//      0x18    "invalid opcode"
	//      0x19    "invalid opcode"
	//      0x1A    "invalid opcode"
	//      0x1B    "invalid opcode"
	//      0x1C    "invalid opcode"
	//      0x1D    "invalid opcode"
	//      0x1E    "invalid opcode"
	//      0x1F    "invalid opcode"
	//      0x20    "invalid opcode"
	//      0x21    "invalid opcode"
	//      0x22    "invalid opcode"
	//      0x23    "invalid opcode"
	//      0x24    "invalid opcode"
	//      0x25    "invalid opcode"
	//      0x26    "invalid opcode"
	//      0x27    "invalid opcode"
	//      0x28    "invalid opcode"
	//      0x29    "invalid opcode"
	//      0x2A    "invalid opcode"
	//      0x2B    "invalid opcode"
	//      0x2C    "invalid opcode"
	//      0x2D    "invalid opcode"
	//      0x2E    "invalid opcode"
	//      0x2F    "invalid opcode"
	//      0x30    "invalid opcode"
	//      0x31    "invalid opcode"
	//      0x32    "invalid opcode"
	//      0x33    "invalid opcode"
	//      0x34    "invalid opcode"
	//      0x35    "invalid opcode"
	//      0x36    "invalid opcode"
	//      0x37    "invalid opcode"
	//      0x38    "invalid opcode"
	//      0x39    "invalid opcode"
	//      0x3A    "invalid opcode"
	//      0x3B    "invalid opcode"
	//      0x3C    "invalid opcode"
	//      0x3D    "invalid opcode"
	//      0x3E    "invalid opcode"
	//      0x3F    "invalid opcode"

//      0x40    "in	b, (c)"
void	do_in_b_portin_c()
{
	set_B(inline_IN());
}       

//      0x41    "out	(c), b"
void	do_out_portin_c_b()
{
	inline_OUT(get_B());
}

//      0x42    "sbc	hl, bc"
void	do_sbc_hl_bc()
{
	inline_SBCW(m_BC);
}       

//      0x43    "ld	(address), bc"
void	do_ld_memataddr_bc  ()
{
	m_bus.MemWriteWord(m_bus.MemReadWord(m_PC), m_BC);
	m_PC += 2;
}

//      0x44    "neg"
void	do_negate()
{
	char	bOldA = get_A();
	set_A((char)0);
	inline_SUB(bOldA);
}

//      0x45    "retn"
void	do_retn ()
{
	inline_RET();
	m_IFF1 = m_IFF2;  // restore the iff state
}

//      0x46    "im	0"
void	do_im_0 ()
{
	m_nInterruptMode = 0;
}

//      0x47    "ld	i, a"
void	do_ld_i_a()
{
	m_I = get_A();
}       

//      0x48    "in	c, (c)"
void	do_in_c_portin_c()
{
	set_C(inline_IN());
}       

//      0x49    "out	(c), c"
void	do_out_portin_c_c()
{
	inline_OUT(get_C());
}       

//      0x4A    "adc	hl, bc"
void	do_adc_hl_bc()
{
	inline_ADCW(m_BC);
}       

//      0x4B    "ld	bc, (address)"
void	do_ld_bc_memataddr  ()
{
	m_BC = m_bus.MemReadWord(m_bus.MemReadWord(m_PC));
	m_PC += 2;
}               

//      0x4C    "invalid opcode"

//      0x4D    "reti"
void	do_reti ()
{
	// no support for alerting peripherals, just like ret
	m_PC = m_bus.MemReadWord(m_SP);
	m_SP += 2;
}

//      0x4E    "invalid opcode"

//      0x4F    "ld	r, a"
void	do_ld_r_a()
{
	m_R = get_A();
}

//      0x50    "in	d, (c)"
void	do_in_d_portin_c()
{
	set_D(inline_IN());
}       

//      0x51    "out	(c), d"
void	do_out_portin_c_d()
{
	inline_OUT(get_D());
}

//      0x52    "sbc	hl, de"
void	do_sbc_hl_de()
{
	inline_SBCW(m_DE);
}

//      0x53    "ld	(address), de"
void	do_ld_memataddr_de  ()
{
	m_bus.MemWriteWord(m_bus.MemReadWord(m_PC), m_DE);
	m_PC += 2;
}

//      0x54    "invalid opcode"
//      0x55    "invalid opcode"

//      0x56    "im	1"
void	do_im_1 ()
{
	m_nInterruptMode = 1;
}

//      0x57    "ld	a, i"
void	do_ld_a_i()
{
	set_A(m_I);
	set_F	((char)(
				(CARRY_FLAG_SET() ? m_CARRY_MASK : 0) |
				(m_IFF1 ? m_PARITY_MASK : 0) |
				(get_A()>0 ? 0 : m_ZERO_MASK) |
				(get_A() & m_SIGN_MASK)
			));
}
			 
//      0x58    "in	e, (c)"
void	do_in_e_portin_c()
{
	set_E(inline_IN());
}       

//      0x59    "out	(c), e"
void	do_out_portin_c_e()
{
	inline_OUT(get_E());
}

//      0x5A    "adc	hl, de"
void	do_adc_hl_de()
{
	inline_ADCW(m_DE);
}       

//      0x5B    "ld	de, (address)"
void	do_ld_de_memataddr()
{
	m_DE = m_bus.MemReadWord(m_bus.MemReadWord(m_PC));
	m_PC += 2;
}

//      0x5C    "invalid opcode"
//      0x5D    "invalid opcode"

//      0x5E    "im	2"
void	do_im_2 ()
{
	m_nInterruptMode = 2;
}

//      0x5F    "ld	a, r"
void	do_ld_a_r()
{
	set_A(m_R);
	set_F	((char)(
				(CARRY_FLAG_SET() ? m_CARRY_MASK : 0) |
				(m_IFF1 ? m_PARITY_MASK : 0) |
				(get_A()>0 ? 0 : m_ZERO_MASK) |
				(get_A() & m_SIGN_MASK)
			));
}

//      0x60    "in	h, (c)"
void	do_in_h_portin_c()
{
	set_H(inline_IN());
}       

//      0x61    "out	(c), h"
void	do_out_portin_c_h()
{
	inline_OUT(get_H());
}

//      0x62    "sbc	hl, hl"
void	do_sbc_hl_hl()
{       
	inline_SBCW(m_HL);
}

//      0x63    "ld	(address), hl"      -       this instruction is redundant with the 2A instruction
//              see CZ80System::do_ld_memataddr_hl().

//      0x64    "invalid opcode"
//      0x65    "invalid opcode"
//      0x66    "invalid opcode"

//      0x67    "rrd"
void	do_rrd()
{
	char	bByte = m_bus.MemReadByte(m_HL);
	char	bNewByte = (char)((bByte >> 4) | (get_A() << 4));

	m_bus.MemWriteByte(m_HL, bNewByte);
	set_A((char)((bByte & 0x0F) | (get_A() & 0xF0)));
	set_F	((char)(
				(m_rgbParityTable[ get_A() ]==1 ? m_PARITY_MASK : 0) |
				(CARRY_FLAG_SET() ? m_CARRY_MASK : 0) |
				(get_A()>0 ? 0 : m_ZERO_MASK) |
				(get_A() & m_SIGN_MASK)
			));
}

//      0x68    "in	l, (c)"
void	do_in_l_portin_c()
{
	set_L(inline_IN());
}       

//      0x69    "out	(c), l"
void	do_out_portin_c_l()
{
	inline_OUT(get_L());
}

//      0x6A    "adc	hl, hl"
void	do_adc_hl_hl()
{
	inline_ADCW(m_HL);
}       

//      0x6B    "ld	hl, (address)"      -       this instruction is redundant with the 2A instruction
//              see CZ80System::do_ld_hl_memataddr().

//      0x6C    "invalid opcode"
//      0x6D    "invalid opcode"
//      0x6E    "invalid opcode"

//      0x6F    "rld"
void	do_rld()
{
	char	bByte = m_bus.MemReadByte(m_HL);
	char	bNewByte = (char)((bByte << 4) | (get_A() & 0x0F));

	m_bus.MemWriteByte(m_HL, bNewByte);
	set_A((char)((bByte >> 4) | (get_A() & 0xF0)));
	set_F	((char)(
				(m_rgbParityTable[ get_A() ]==1 ? m_PARITY_MASK : 0) |
				(CARRY_FLAG_SET() ? m_CARRY_MASK : 0) |
				(get_A()>0 ? 0 : m_ZERO_MASK) |
				(get_A() & m_SIGN_MASK)
			));
}

//      0x70    "invalid opcode"
//      0x71    "invalid opcode"

//      0x72    "sbc	hl, sp"
void	do_sbc_hl_sp()
{
	inline_SBCW(m_SP);
}       

//      0x73    "ld	(address), sp"
void	do_ld_memataddr_sp  ()
{
	m_bus.MemWriteWord(m_bus.MemReadWord(m_PC), m_SP);
	m_PC += 2;
}

//      0x74    "invalid opcode"
//      0x75    "invalid opcode"
//      0x76    "invalid opcode"
//      0x77    "invalid opcode"

//      0x78    "in	a, (c)"
void	do_in_a_portin_c()
{
	set_A(inline_IN());
}       

//      0x79    "out	(c), a"
void	do_out_portin_c_a()
{
	inline_OUT(get_A());
}       

//      0x7A    "adc	hl, sp"
void	do_adc_hl_sp()
{
	inline_ADCW(m_SP);
}       

//      0x7B    "ld	sp, (address)"
void	do_ld_sp_memataddr  ()
{
	m_SP = m_bus.MemReadWord(m_bus.MemReadWord(m_PC));
	m_PC += 2;
}       

//      0x7C    "invalid opcode"
//      0x7D    "invalid opcode"
//      0x7E    "invalid opcode"
//      0x7F    "invalid opcode"
//      0x80    "invalid opcode"
//      0x81    "invalid opcode"
//      0x82    "invalid opcode"
//      0x83    "invalid opcode"
//      0x84    "invalid opcode"
//      0x85    "invalid opcode"
//      0x86    "invalid opcode"
//      0x87    "invalid opcode"
//      0x88    "invalid opcode"
//      0x89    "invalid opcode"
//      0x8A    "invalid opcode"
//      0x8B    "invalid opcode"
//      0x8C    "invalid opcode"
//      0x8D    "invalid opcode"
//      0x8E    "invalid opcode"
//      0x8F    "invalid opcode"
//      0x90    "invalid opcode"
//      0x91    "invalid opcode"
//      0x92    "invalid opcode"
//      0x93    "invalid opcode"
//      0x94    "invalid opcode"
//      0x95    "invalid opcode"
//      0x96    "invalid opcode"
//      0x97    "invalid opcode"
//      0x98    "invalid opcode"
//      0x99    "invalid opcode"
//      0x9A    "invalid opcode"
//      0x9B    "invalid opcode"
//      0x9C    "invalid opcode"
//      0x9D    "invalid opcode"
//      0x9E    "invalid opcode"
//      0x9F    "invalid opcode"

//      0xA0    "ldi"
void	do_ldi()
{
	m_bus.MemWriteByte(m_DE, m_bus.MemReadByte(m_HL++));
	m_DE++;
	m_BC--;
	set_F	((char)(
				(get_F() & ~(m_SUBTRACT_MASK | m_HALF_CARRY_MASK | m_PARITY_MASK)) |
				(m_BC>0 ? m_PARITY_MASK : 0)
			));
}

//      0xA1    "cpi"
void	do_cpi()
{
	char	bByte = m_bus.MemReadByte(m_HL++);
	char	bNewByte = (char)(get_A() - bByte);
	m_BC--;
	set_F	((char)(
				m_SUBTRACT_MASK |
				(CARRY_FLAG_SET() ? m_CARRY_MASK : 0) |
				(bNewByte>0 ? 0 : m_ZERO_MASK) |
				(bNewByte & m_SIGN_MASK) |
				((get_A() ^ bByte ^ bNewByte) & m_HALF_CARRY_MASK) |
				(m_BC>0 ? m_PARITY_MASK : 0)
			));
}

//      0xA2    "ini"
void	do_ini()
{
	m_bus.MemWriteByte(m_HL++, inline_Inport(get_C()));
	set_B((char)(get_B()-1));
	set_F	((char)(
				m_SUBTRACT_MASK |
				(get_B()>0 ? 0 : m_ZERO_MASK)
			));
}

//      0xA3    "outi"
void	do_outi ()
{
	inline_Outport(get_C(), m_bus.MemReadByte(m_HL++));
	set_B((char)(get_B()-1));
	set_F	((char)(
				m_SUBTRACT_MASK |
				(get_B()>0 ? 0 : m_ZERO_MASK)
			));
}

//      0xA4    "invalid opcode"
//      0xA5    "invalid opcode"
//      0xA6    "invalid opcode"
//      0xA7    "invalid opcode"

//      0xA8    "ldd"
void	do_ldd()
{
	m_bus.MemWriteByte(m_DE, m_bus.MemReadByte(m_HL));
	m_HL--;
	m_DE--;
	m_BC--;
	set_F	((char)(
				(get_F() & ~(m_SUBTRACT_MASK | m_HALF_CARRY_MASK | m_PARITY_MASK)) |
				(m_BC>0 ? m_PARITY_MASK : 0)
			));
}

//      0xA9    "cpd"
void	do_cpd()
{
	char	bByte = m_bus.MemReadByte(m_HL--);
	char	bNewByte = (char)(get_A() - bByte);
	m_BC--;
	set_F	((char)(
				m_SUBTRACT_MASK |
				(CARRY_FLAG_SET() ? m_CARRY_MASK : 0) |
				(bNewByte>0 ? 0 : m_ZERO_MASK) |
				(bNewByte & m_SIGN_MASK) |
				((get_A() ^ bByte ^ bNewByte) & m_HALF_CARRY_MASK) |
				(m_BC>0 ? m_PARITY_MASK : 0)
			));
}

//      0xAA    "ind"
void	do_ind()
{
	m_bus.MemWriteByte(m_HL, inline_Inport(get_C()));
	m_HL--;
	set_B((char)(get_B()-1));
	set_F	((char)(
				m_SUBTRACT_MASK |
				(get_B()>0 ? 0 : m_ZERO_MASK)
			));
}

//      0xAB    "outd"
void	do_outd ()
{
	inline_Outport(get_C(), m_bus.MemReadByte(m_HL--));
	set_B((char)(get_B()-1));
	set_F	((char)(
				m_SUBTRACT_MASK |
				(get_B()>0 ? 0 : m_ZERO_MASK)
			));
}

//      0xAC    "invalid opcode"
//      0xAD    "invalid opcode"
//      0xAE    "invalid opcode"
//      0xAF    "invalid opcode"

//      0xB0    "ldir"
void	do_ldir ()
{
	// Take care of cycle counter...
	m_nCycle += (m_BC>0 ? (5 * m_BC) : (5 * 65536));

	do
	{
		m_bus.MemWriteByte(m_DE++, m_bus.MemReadByte(m_HL++));
		m_BC--;
	} while (m_BC!=0);

	set_F	((char)(
				get_F() & ~(m_SUBTRACT_MASK | m_HALF_CARRY_MASK | m_PARITY_MASK)
			));
}

//      0xB1    "cpir"
void	do_cpir ()
{
	char	bByte;
	char	bNewByte;

	do
	{
		m_nCycle += 5;
		bByte = m_bus.MemReadByte(m_HL++);
		bNewByte = (char)(get_A() - bByte);
		m_BC--;
	} while (m_BC!=0 && bNewByte!=0);
	set_F	((char)(
				m_SUBTRACT_MASK |
				(CARRY_FLAG_SET() ? m_CARRY_MASK : 0) |
				(bNewByte>0 ? 0 : m_ZERO_MASK) |
				(bNewByte & m_SIGN_MASK) |
				((get_A() ^ bByte ^ bNewByte) & m_HALF_CARRY_MASK) |
				(m_BC>0 ? m_PARITY_MASK : 0)
			));
}

//      0xB2    "inir"
void	do_inir ()
{
	m_nCycle += (get_B()>0 ? (5 * get_B()) : (5 * 256));

	do
	{
		m_bus.MemWriteByte(m_HL, inline_Inport(get_C()));
		m_HL++;
		set_B((char)(get_B()-1));
	} while (get_B()!=0);
	set_F	((char)(
				m_ZERO_MASK | m_SUBTRACT_MASK
			));
}

//      0xB3    "otir"
void	do_otir()
{
	m_nCycle += (get_B()>0 ? (5 * get_B()) : (5 * 256));

	do
	{
		inline_Outport(get_C(), m_bus.MemReadByte(m_HL++));
		set_B((char)(get_B()-1));
	} while (get_B()!=0);
	set_F	((char)(
				m_ZERO_MASK | m_SUBTRACT_MASK
			));
}

//      0xB4    "invalid opcode"
//      0xB5    "invalid opcode"
//      0xB6    "invalid opcode"
//      0xB7    "invalid opcode"

//      0xB8    "lddr"
void	do_lddr ()
{
	m_nCycle += (m_BC>0 ? (5 * m_BC) : (5 * 65536));

	do
	{
		m_bus.MemWriteByte(m_DE, m_bus.MemReadByte(m_HL--));
		m_DE--;
		m_BC--;
	} while (m_BC!=0);
	set_F	((char)(
				get_F() & ~(m_SUBTRACT_MASK | m_HALF_CARRY_MASK | m_PARITY_MASK)
			));
}

//      0xB9    "cpdr"
void	do_cpdr ()
{
	char	bByte;
	char	bNewByte;

	do
	{
		m_nCycle += 5;

		bByte = m_bus.MemReadByte(m_HL--);
		bNewByte = (char)(get_A() - bByte);
		m_BC--;
	} while (m_BC!=0 && bNewByte!=0);
	set_F	((char)(
				m_SUBTRACT_MASK |
				(CARRY_FLAG_SET() ? m_CARRY_MASK : 0) |
				(bNewByte>0 ? 0 : m_ZERO_MASK) |
				(bNewByte & m_SIGN_MASK) |
				((get_A() ^ bByte ^ bNewByte) & m_HALF_CARRY_MASK) |
				(m_BC>0 ? m_PARITY_MASK : 0)
			));
}

//      0xBA    "indr"
void	do_indr ()
{
	m_nCycle += (get_B()>0 ? (5 * get_B()) : (5 * 256));

	do
	{
		m_bus.MemWriteByte(m_HL, inline_Inport(get_C()));
		m_HL--;
		set_B((char)(get_B()-1));
	} while (get_B()!=0);
	set_F	((char)(
				m_ZERO_MASK | m_SUBTRACT_MASK
			));
}

//      0xBB    "otdr"
void	do_otdr()
{
	m_nCycle += (get_B()>0 ? (5 * get_B()) : (5 * 256));

	do
	{
		inline_Outport(get_C(), m_bus.MemReadByte(m_HL--));
		set_B((char)(get_B()-1));
	} while (get_B()!=0);
	set_F	((char)(
				m_ZERO_MASK | m_SUBTRACT_MASK
			));
}

	//      0xBC    "invalid opcode"
	//      0xBD    "invalid opcode"
	//      0xBE    "invalid opcode"
	//      0xBF    "invalid opcode"
	//      0xC0    "invalid opcode"
	//      0xC1    "invalid opcode"
	//      0xC2    "invalid opcode"
	//      0xC3    "invalid opcode"
	//      0xC4    "invalid opcode"
	//      0xC5    "invalid opcode"
	//      0xC6    "invalid opcode"
	//      0xC7    "invalid opcode"
	//      0xC8    "invalid opcode"
	//      0xC9    "invalid opcode"
	//      0xCA    "invalid opcode"
	//      0xCB    "invalid opcode"
	//      0xCC    "invalid opcode"
	//      0xCD    "invalid opcode"
	//      0xCE    "invalid opcode"
	//      0xCF    "invalid opcode"
	//      0xD0    "invalid opcode"
	//      0xD1    "invalid opcode"
	//      0xD2    "invalid opcode"
	//      0xD3    "invalid opcode"
	//      0xD4    "invalid opcode"
	//      0xD5    "invalid opcode"
	//      0xD6    "invalid opcode"
	//      0xD7    "invalid opcode"
	//      0xD8    "invalid opcode"
	//      0xD9    "invalid opcode"
	//      0xDA    "invalid opcode"
	//      0xDB    "invalid opcode"
	//      0xDC    "invalid opcode"
	//      0xDD    "invalid opcode"
	//      0xDE    "invalid opcode"
	//      0xDF    "invalid opcode"
	//      0xE0    "invalid opcode"
	//      0xE1    "invalid opcode"
	//      0xE2    "invalid opcode"
	//      0xE3    "invalid opcode"
	//      0xE4    "invalid opcode"
	//      0xE5    "invalid opcode"
	//      0xE6    "invalid opcode"
	//      0xE7    "invalid opcode"
	//      0xE8    "invalid opcode"
	//      0xE9    "invalid opcode"
	//      0xEA    "invalid opcode"
	//      0xEB    "invalid opcode"
	//      0xEC    "invalid opcode"
	//      0xED    "invalid opcode"
	//      0xEE    "invalid opcode"
	//      0xEF    "invalid opcode"
	//      0xF0    "invalid opcode"
	//      0xF1    "invalid opcode"
	//      0xF2    "invalid opcode"
	//      0xF3    "invalid opcode"
	//      0xF4    "invalid opcode"
	//      0xF5    "invalid opcode"
	//      0xF6    "invalid opcode"
	//      0xF7    "invalid opcode"
	//      0xF8    "invalid opcode"
	//      0xF9    "invalid opcode"
	//      0xFA    "invalid opcode"
	//      0xFB    "invalid opcode"
	//      0xFC    "invalid opcode"
	//      0xFD    "invalid opcode"
	//      0xFE    "invalid opcode"
	//      0xFF    "invalid opcode"

	// 
	//                                                                          
	//   End 0xED opcode instructions                                           
	//                                                                          
	// 

	// 
	//                                                                          
	//   Start of 0xDD 0x?? and 0xFD 0x?? opcode instructions							
	//                                                                          
	// 

	//      0x00    "invalid opcode"
	//      0x01    "invalid opcode"
	//      0x02    "invalid opcode"
	//      0x03    "invalid opcode"
	//      0x04    "invalid opcode"
	//      0x05    "invalid opcode"
	//      0x06    "invalid opcode"
	//      0x07    "invalid opcode"
	//      0x08    "invalid opcode"

	//      0x09    "add	ix/iy, bc"
	char	do_add_idx_bc  (char wIndex)
	{
		return	inline_ADDW(wIndex, m_BC);
	}

	//      0x0A    "invalid opcode"
	//      0x0B    "invalid opcode"
	//      0x0C    "invalid opcode"
	//      0x0D    "invalid opcode"
	//      0x0E    "invalid opcode"
	//      0x0F    "invalid opcode"
	//      0x10    "invalid opcode"
	//      0x11    "invalid opcode"
	//      0x12    "invalid opcode"
	//      0x13    "invalid opcode"
	//      0x14    "invalid opcode"
	//      0x15    "invalid opcode"
	//      0x16    "invalid opcode"
	//      0x17    "invalid opcode"
	//      0x18    "invalid opcode"

	//      0x19    "add	ix/iy, de"
	char	do_add_idx_de  (char wIndex)
	{
		return	inline_ADDW(wIndex, m_DE);
	}

	//      0x1A    "invalid opcode"
	//      0x1B    "invalid opcode"
	//      0x1C    "invalid opcode"
	//      0x1D    "invalid opcode"
	//      0x1E    "invalid opcode"
	//      0x1F    "invalid opcode"
	//      0x20    "invalid opcode"

	//      0x21    "ld	ix/iy, WORD"
	char	do_ld_idx_WORD()
	{
		return	inline_ld_WORD();
	}       

	//      0x22    "ld	(address), ix/iy"
	void	do_ld_memataddr_idx(char wIndex)
	{
		m_bus.MemWriteWord(m_bus.MemReadWord(m_PC), wIndex);
		m_PC += 2;
	}

	//      0x23    "inc	ix/iy"
	char	do_inc_idx     (char wIndex)
	{
		return	++wIndex;
	}

	//      0x24    "invalid opcode"

	//      0x25    "invalid opcode"

	//      0x26    "invalid opcode"

	//      0x27    "invalid opcode"
	//      0x28    "invalid opcode"

	//      0x29    "add	ix/iy, ix/iy"      - Both must be same index register (ex "add	ix, ix")
	char	do_add_idx_idx (char wIndex)
	{
		return	inline_ADDW(wIndex, wIndex);
	}

	//      0x2A    "ld	ix/iy, (address)"
	char	do_ld_idx_memataddr()
	{
		char	wIndex = m_bus.MemReadWord(m_bus.MemReadWord(m_PC));
		m_PC += 2;
		return	wIndex;
	}

	//      0x2B    "dec	ix/iy"
	char	do_dec_idx(char wIndex)
	{
		return	--wIndex;
	}

	//      0x2C    "invalid opcode"
	//      0x2D    "invalid opcode"
	//      0x2E    "invalid opcode"
	//      0x2F    "invalid opcode"
	//      0x30    "invalid opcode"
	//      0x31    "invalid opcode"
	//      0x32    "invalid opcode"
	//      0x33    "invalid opcode"

	//      0x34    "inc	(ix/iy + offset)"
	void	do_inc_memat_idx_plus_offset(char wIndex)
	{
		char	wAddress = (char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++)));
		char	bByte = m_bus.MemReadByte(wAddress);

		bByte = inline_INC(bByte);
		m_bus.MemWriteByte(wAddress, bByte);
	}

	//      0x35    "dec	(ix/iy + offset)"
	void	do_dec_memat_idx_plus_offset(char wIndex)
	{
		char	wAddress = (char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++)));
		char	bByte = m_bus.MemReadByte(wAddress);

		bByte = inline_DEC(bByte);
		m_bus.MemWriteByte(wAddress, bByte);
	}

	//      0x36    "ld	(ix/iy + offset), BYTE"
	void	do_ld_memat_idx_plus_offset_BYTE(char wIndex)
	{
		m_bus.MemWriteByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC))),
			m_bus.MemReadByte((char)(m_PC+1)));
		m_PC += 2;
	}

	//      0x37    "invalid opcode"
	//      0x38    "invalid opcode"

	//      0x39    "add	ix/iy, sp"
	char	do_add_idx_sp  (char wIndex)
	{
		return	inline_ADDW(wIndex, m_SP);
	}

	//      0x3A    "invalid opcode"
	//      0x3B    "invalid opcode"
	//      0x3C    "invalid opcode"
	//      0x3D    "invalid opcode"
	//      0x3E    "invalid opcode"
	//      0x3F    "invalid opcode"
	//      0x40    "invalid opcode"
	//      0x41    "invalid opcode"
	//      0x42    "invalid opcode"
	//      0x43    "invalid opcode"
	//      0x44    "invalid opcode"
	//      0x45    "invalid opcode"

	//      0x46    "ld	b, (ix/iy + offset)"
	void	do_ld_b_memat_idx_plus_offset  (char wIndex)
	{
		set_B(m_bus.MemReadByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++)))));
	}

	//      0x47    "invalid opcode"
	//      0x48    "invalid opcode"
	//      0x49    "invalid opcode"
	//      0x4A    "invalid opcode"
	//      0x4B    "invalid opcode"
	//      0x4C    "invalid opcode"
	//      0x4D    "invalid opcode"

	//      0x4E    "ld	c, (ix/iy + offset)"
	void	do_ld_c_memat_idx_plus_offset  (char wIndex)
	{
		set_C(m_bus.MemReadByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++)))));
	}

	//      0x4F    "invalid opcode"
	//      0x50    "invalid opcode"
	//      0x51    "invalid opcode"
	//      0x52    "invalid opcode"
	//      0x53    "invalid opcode"
	//      0x54    "invalid opcode"
	//      0x55    "invalid opcode"

	//      0x56    "ld	d, (ix/iy + offset)"
	void	do_ld_d_memat_idx_plus_offset  (char wIndex)
	{
		set_D(m_bus.MemReadByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++)))));
	}

	//      0x57    "invalid opcode"
	//      0x58    "invalid opcode"
	//      0x59    "invalid opcode"
	//      0x5A    "invalid opcode"
	//      0x5B    "invalid opcode"
	//      0x5C    "invalid opcode"
	//      0x5D    "invalid opcode"

	//      0x5E    "ld	e, (ix/iy + offset)"
	void	do_ld_e_memat_idx_plus_offset  (char wIndex)
	{
		set_E(m_bus.MemReadByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++)))));
	}       

	//      0x5F    "invalid opcode"
	//      0x60    "invalid opcode"
	//      0x61    "invalid opcode"
	//      0x62    "invalid opcode"
	//      0x63    "invalid opcode"
	//      0x64    "invalid opcode"
	//      0x65    "invalid opcode"

	//      0x66    "ld	h, (ix/iy + offset)"
	void	do_ld_h_memat_idx_plus_offset  (char wIndex)
	{
		set_H(m_bus.MemReadByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++)))));
	}

	//      0x67    "invalid opcode"
	//      0x68    "invalid opcode"
	//      0x69    "invalid opcode"
	//      0x6A    "invalid opcode"
	//      0x6B    "invalid opcode"
	//      0x6C    "invalid opcode"
	//      0x6D    "invalid opcode"

	//      0x6E    "ld	l, (ix/iy + offset)"
	void	do_ld_l_memat_idx_plus_offset  (char wIndex)
	{
		set_L(m_bus.MemReadByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++)))));
	}

	//      0x6F    "invalid opcode"

	//      0x70    "ld	(ix/iy + offset), b"
	void	do_ld_memat_idx_plus_offset_b  (char wIndex)
	{
		m_bus.MemWriteByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++))), get_B());
	}

	//      0x71    "ld	(ix/iy + offset), c"
	void	do_ld_memat_idx_plus_offset_c  (char wIndex)
	{
		m_bus.MemWriteByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++))), get_C());
	}

	//      0x72    "ld	(ix/iy + offset), d"
	void	do_ld_memat_idx_plus_offset_d  (char wIndex)
	{
		m_bus.MemWriteByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++))), get_D());
	}

	//      0x73    "ld	(ix/iy + offset), e"
	void	do_ld_memat_idx_plus_offset_e  (char wIndex)
	{
		m_bus.MemWriteByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++))), get_E());
	}

	//      0x74    "ld	(ix/iy + offset), h"
	void	do_ld_memat_idx_plus_offset_h  (char wIndex)
	{
		m_bus.MemWriteByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++))), get_H());
	}

	//      0x75    "ld	(ix/iy + offset), l"
	void	do_ld_memat_idx_plus_offset_l  (char wIndex)
	{
		m_bus.MemWriteByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++))), get_L());
	}

	//      0x76    "invalid opcode"

	//      0x77    "ld	(ix/iy + offset), a"
	void	do_ld_memat_idx_plus_offset_a  (char wIndex)
	{
		m_bus.MemWriteByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++))), get_A());
	}

	//      0x78    "invalid opcode"
	//      0x79    "invalid opcode"
	//      0x7A    "invalid opcode"
	//      0x7B    "invalid opcode"
	//      0x7C    "invalid opcode"
	//      0x7D    "invalid opcode"

	//      0x7E    "ld	a, (ix/iy + offset)"
	void	do_ld_a_memat_idx_plus_offset  (char wIndex)
	{
		set_A(m_bus.MemReadByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++)))));
	}

	//      0x7F    "invalid opcode"
	//      0x80    "invalid opcode"
	//      0x81    "invalid opcode"
	//      0x82    "invalid opcode"
	//      0x83    "invalid opcode"
	//      0x84    "invalid opcode"
	//      0x85    "invalid opcode"

	//      0x86    "add	a, (ix/iy + offset)"
	void	do_add_a_memat_idx_plus_offset (char wIndex)
	{
		inline_ADD(m_bus.MemReadByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++)))));
	}

	//      0x87    "invalid opcode"
	//      0x88    "invalid opcode"
	//      0x89    "invalid opcode"
	//      0x8A    "invalid opcode"
	//      0x8B    "invalid opcode"
	//      0x8C    "invalid opcode"
	//      0x8D    "invalid opcode"

	//      0x8E    "adc	a, (ix/iy + offset)"
	void	do_adc_a_memat_idx_plus_offset (char wIndex)
	{
		inline_ADC(m_bus.MemReadByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++)))));
	}

	//      0x8F    "invalid opcode"
	//      0x90    "invalid opcode"
	//      0x91    "invalid opcode"
	//      0x92    "invalid opcode"
	//      0x93    "invalid opcode"
	//      0x94    "invalid opcode"
	//      0x95    "invalid opcode"

	//      0x96    "sub	a, (ix/iy + offset)"
	void	do_sub_a_memat_idx_plus_offset (char wIndex)
	{
		inline_SUB(m_bus.MemReadByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++)))));
	}

	//      0x97    "invalid opcode"
	//      0x98    "invalid opcode"
	//      0x99    "invalid opcode"
	//      0x9A    "invalid opcode"
	//      0x9B    "invalid opcode"
	//      0x9C    "invalid opcode"
	//      0x9D    "invalid opcode"

	//      0x9E    "sbc	a, (ix/iy + offset)"
	void	do_sbc_a_memat_idx_plus_offset (char wIndex)
	{
		inline_SBC(m_bus.MemReadByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++)))));
	}

	//      0x9F    "invalid opcode"
	//      0xA0    "invalid opcode"
	//      0xA1    "invalid opcode"
	//      0xA2    "invalid opcode"
	//      0xA3    "invalid opcode"
	//      0xA4    "invalid opcode"
	//      0xA5    "invalid opcode"

	//      0xA6    "and	(ix/iy + offset)"
	void	do_and_memat_idx_plus_offset   (char wIndex)
	{
		inline_AND(m_bus.MemReadByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++)))));
	}

	//      0xA7    "invalid opcode"
	//      0xA8    "invalid opcode"
	//      0xA9    "invalid opcode"
	//      0xAA    "invalid opcode"
	//      0xAB    "invalid opcode"
	//      0xAC    "invalid opcode"
	//      0xAD    "invalid opcode"

	//      0xAE    "xor	(ix/iy + offset)"
	void	do_xor_memat_idx_plus_offset   (char wIndex)
	{
		inline_XOR(m_bus.MemReadByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++)))));
	}

	//      0xAF    "invalid opcode"
	//      0xB0    "invalid opcode"
	//      0xB1    "invalid opcode"
	//      0xB2    "invalid opcode"
	//      0xB3    "invalid opcode"
	//      0xB4    "invalid opcode"
	//      0xB5    "invalid opcode"

	//      0xB6    "or	(ix/iy + offset)"
	void	do_or_memat_idx_plus_offset    (char wIndex)
	{
		inline_OR(m_bus.MemReadByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++)))));
	}

	//      0xB7    "invalid opcode"
	//      0xB8    "invalid opcode"
	//      0xB9    "invalid opcode"
	//      0xBA    "invalid opcode"
	//      0xBB    "invalid opcode"
	//      0xBC    "invalid opcode"
	//      0xBD    "invalid opcode"

	//      0xBE    "cp	(ix/iy + offset)"
	void	do_cp_memat_idx_plus_offset    (char wIndex)
	{
		inline_CP(m_bus.MemReadByte((char)(((int)wIndex) + ((byte)m_bus.MemReadByte(m_PC++)))));
	}

	//      0xBF    "invalid opcode"
	//      0xC0    "invalid opcode"
	//      0xC1    "invalid opcode"
	//      0xC2    "invalid opcode"
	//      0xC3    "invalid opcode"
	//      0xC4    "invalid opcode"
	//      0xC5    "invalid opcode"
	//      0xC6    "invalid opcode"
	//      0xC7    "invalid opcode"
	//      0xC8    "invalid opcode"
	//      0xC9    "invalid opcode"
	//      0xCA    "invalid opcode"

	//      0xCB = "DDh CBh or FDh CBh.. extended instruction"
	void	do_0xDDCB_or_0xFDCB_instruction	(char wIndex)
	{
		byte	offset = (byte) m_bus.MemReadByte(m_PC++);
		Exec0xDDCB_or_0xFDCBOpcode( m_bus.MemReadByte(m_PC++), wIndex, offset );
	}

	//      0xCC    "invalid opcode"
	//      0xCD    "invalid opcode"
	//      0xCE    "invalid opcode"
	//      0xCF    "invalid opcode"
	//      0xD0    "invalid opcode"
	//      0xD1    "invalid opcode"
	//      0xD2    "invalid opcode"
	//      0xD3    "invalid opcode"
	//      0xD4    "invalid opcode"
	//      0xD5    "invalid opcode"
	//      0xD6    "invalid opcode"
	//      0xD7    "invalid opcode"
	//      0xD8    "invalid opcode"
	//      0xD9    "invalid opcode"
	//      0xDA    "invalid opcode"
	//      0xDB    "invalid opcode"
	//      0xDC    "invalid opcode"
	//      0xDD    "invalid opcode"
	//      0xDE    "invalid opcode"
	//      0xDF    "invalid opcode"
	//      0xE0    "invalid opcode"

	//      0xE1    "pop	ix/iy"
	char	do_pop_idx()
	{
		return	inline_POP();
	}

	//      0xE2    "invalid opcode"

	//      0xE3    "ex	(sp), ix/iy"
	void	do_ex_memat_sp_idx     (char wIndex)
	{
		char	wTemp = m_bus.MemReadWord(m_SP);
		m_bus.MemWriteWord(m_SP, wIndex);
		wIndex = wTemp;
	}

	//      0xE4    "invalid opcode"

	//      0xE5    "push	ix/iy
	void	do_push_idx    (char wIndex)
	{
		inline_PUSH(wIndex);
	}       

	//      0xE6    "invalid opcode"
	//      0xE7    "invalid opcode"
	//      0xE8    "invalid opcode"

	//      0xE9    "jp	(ix/iy)"
	void	do_jp_memat_idx(char wIndex)
	{
		m_PC = wIndex;
	}       

	//      0xEA    "invalid opcode"
	//      0xEB    "invalid opcode"
	//      0xEC    "invalid opcode"
	//      0xED    "invalid opcode"
	//      0xEE    "invalid opcode"
	//      0xEF    "invalid opcode"
	//      0xF0    "invalid opcode"
	//      0xF1    "invalid opcode"
	//      0xF2    "invalid opcode"
	//      0xF3    "invalid opcode"
	//      0xF4    "invalid opcode"
	//      0xF5    "invalid opcode"
	//      0xF6    "invalid opcode"
	//      0xF7    "invalid opcode"
	//      0xF8    "invalid opcode"

	//      0xF9    "ld	sp, ix/iy"
	void	do_ld_sp_idx   (char wIndex)
	{
		m_SP = wIndex;
	}

	//      0xFA    "invalid opcode"
	//      0xFB    "invalid opcode"
	//      0xFC    "invalid opcode"
	//      0xFD    "invalid opcode"
	//      0xFE    "invalid opcode"
	//      0xFF    "invalid opcode"

	// 
	//                                                                          
	//   End of 0xDD 0x?? and 0xFD 0x?? opcode instructions							
	//                                                                          
	// 


	// 
	//                                                                          
	//   Start of 0xDD 0xCB 0x?? and 0xFD 0xCB 0x?? opcode instructions			
	//                                                                          
	// 

	//      0x00    "invalid opcode"
	//      0x01    "invalid opcode"
	//      0x02    "invalid opcode"
	//      0x03    "invalid opcode"
	//      0x04    "invalid opcode"
	//      0x05    "invalid opcode"

	//      0x06    "rlc	(ix/iy + offset)"
	void	do_rlc_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char	wAddr = (char)(((int)wIndex) + offset);
		char	bByte = m_bus.MemReadByte(wAddr);
		
		bByte = inline_RLC(bByte);
		m_bus.MemWriteByte(wAddr, bByte);
	}

	//      0x07    "invalid opcode"
	//      0x08    "invalid opcode"
	//      0x09    "invalid opcode"
	//      0x0A    "invalid opcode"
	//      0x0B    "invalid opcode"
	//      0x0C    "invalid opcode"
	//      0x0D    "invalid opcode"

	//      0x0E    "rrc	(ix/iy + offset)"
	void	do_rrc_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);
		
		bByte = inline_RRC(bByte);
		m_bus.MemWriteByte(wAddr, bByte);
	}

	//      0x0F    "invalid opcode"
	//      0x10    "invalid opcode"
	//      0x11    "invalid opcode"
	//      0x12    "invalid opcode"
	//      0x13    "invalid opcode"
	//      0x14    "invalid opcode"
	//      0x15    "invalid opcode"

	//      0x16    "rl	(ix/iy + offset)"
	void	do_rl_memat_idx_plus_offset (char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);
		
		bByte = inline_RL(bByte);
		m_bus.MemWriteByte(wAddr, bByte);
	}

	//      0x17    "invalid opcode"
	//      0x18    "invalid opcode"
	//      0x19    "invalid opcode"
	//      0x1A    "invalid opcode"
	//      0x1B    "invalid opcode"
	//      0x1C    "invalid opcode"
	//      0x1D    "invalid opcode"

	//      0x1E    "rr	(ix/iy + offset)"
	void	do_rr_memat_idx_plus_offset (char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);
		
		bByte = inline_RR(bByte);
		m_bus.MemWriteByte(wAddr, bByte);
	}

	//      0x1F    "invalid opcode"
	//      0x20    "invalid opcode"
	//      0x21    "invalid opcode"
	//      0x22    "invalid opcode"
	//      0x23    "invalid opcode"
	//      0x24    "invalid opcode"
	//      0x25    "invalid opcode"

	//      0x26    "sla	(ix/iy + offset)"
	void	do_sla_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);
		
		bByte = inline_SLA(bByte);
		m_bus.MemWriteByte(wAddr, bByte);
	}       

	//      0x27    "invalid opcode"
	//      0x28    "invalid opcode"
	//      0x29    "invalid opcode"
	//      0x2A    "invalid opcode"
	//      0x2B    "invalid opcode"
	//      0x2C    "invalid opcode"
	//      0x2D    "invalid opcode"

	//      0x2E    "sra	(ix/iy + offset)
	void	do_sra_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);
		
		bByte = inline_SRA(bByte);
		m_bus.MemWriteByte(wAddr, bByte);
	}

	//      0x2F    "invalid opcode"
	//      0x30    "invalid opcode"
	//      0x31    "invalid opcode"
	//      0x32    "invalid opcode"
	//      0x33    "invalid opcode"
	//      0x34    "invalid opcode"
	//      0x35    "invalid opcode"

	//      0x36    "sll	(ix/iy + offset)"
	void	do_sll_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);
		
		bByte = inline_SLL(bByte);
		m_bus.MemWriteByte(wAddr, bByte);
	}

	//      0x37    "invalid opcode"
	//      0x38    "invalid opcode"
	//      0x39    "invalid opcode"
	//      0x3A    "invalid opcode"
	//      0x3B    "invalid opcode"
	//      0x3C    "invalid opcode"
	//      0x3D    "invalid opcode"

	//      0x3E    "srl	(ix/iy + offset)"
	void	do_srl_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);
		
		bByte = inline_SRL(bByte);
		m_bus.MemWriteByte(wAddr, bByte);
	}

	//      0x3F    "invalid opcode"
	//      0x40    "invalid opcode"
	//      0x41    "invalid opcode"
	//      0x42    "invalid opcode"
	//      0x43    "invalid opcode"
	//      0x44    "invalid opcode"
	//      0x45    "invalid opcode"

	//      0x46    "bit	0, (ix/iy + offset)"
	void	do_bit_0_memat_idx_plus_offset(char wIndex, byte offset)
	{
		inline_BIT((char) 0, m_bus.MemReadByte((char)(((int)wIndex) + offset)) );
	}       

	//      0x47    "invalid opcode"
	//      0x48    "invalid opcode"
	//      0x49    "invalid opcode"
	//      0x4A    "invalid opcode"
	//      0x4B    "invalid opcode"
	//      0x4C    "invalid opcode"
	//      0x4D    "invalid opcode"

	//      0x4E    "bit	1, (ix/iy + offset)"
	void	do_bit_1_memat_idx_plus_offset(char wIndex, byte offset)
	{
		inline_BIT((char) 1, m_bus.MemReadByte((char)(((int)wIndex) + offset)) );
	}       

	//      0x4F    "invalid opcode"
	//      0x50    "invalid opcode"
	//      0x51    "invalid opcode"
	//      0x52    "invalid opcode"
	//      0x53    "invalid opcode"
	//      0x54    "invalid opcode"
	//      0x55    "invalid opcode"

	//      0x56    "bit	2, (ix/iy + offset)"
	void	do_bit_2_memat_idx_plus_offset(char wIndex, byte offset)
	{
		inline_BIT((char) 2, m_bus.MemReadByte((char)(((int)wIndex) + offset)) );
	}       

	//      0x57    "invalid opcode"
	//      0x58    "invalid opcode"
	//      0x59    "invalid opcode"
	//      0x5A    "invalid opcode"
	//      0x5B    "invalid opcode"
	//      0x5C    "invalid opcode"
	//      0x5D    "invalid opcode"

	//      0x5E    "bit	3, (ix/iy + offset)"
	void	do_bit_3_memat_idx_plus_offset(char wIndex, byte offset)
	{
		inline_BIT((char) 3, m_bus.MemReadByte((char)(((int)wIndex) + offset)) );
	}               

	//      0x5F    "invalid opcode"
	//      0x60    "invalid opcode"
	//      0x61    "invalid opcode"
	//      0x62    "invalid opcode"
	//      0x63    "invalid opcode"
	//      0x64    "invalid opcode"
	//      0x65    "invalid opcode"

	//      0x66    "bit	4, (ix/iy + offset)
	void	do_bit_4_memat_idx_plus_offset(char wIndex, byte offset)
	{
		inline_BIT((char) 4, m_bus.MemReadByte((char)(((int)wIndex) + offset)) );
	}       

	//      0x67    "invalid opcode"
	//      0x68    "invalid opcode"
	//      0x69    "invalid opcode"
	//      0x6A    "invalid opcode"
	//      0x6B    "invalid opcode"
	//      0x6C    "invalid opcode"
	//      0x6D    "invalid opcode"

	//      0x6E    "bit	5, (ix/iy + offset)"
	void	do_bit_5_memat_idx_plus_offset(char wIndex, byte offset)
	{
		inline_BIT((char) 5, m_bus.MemReadByte((char)(((int)wIndex) + offset)) );
	}       

	//      0x6F    "invalid opcode"
	//      0x70    "invalid opcode"
	//      0x71    "invalid opcode"
	//      0x72    "invalid opcode"
	//      0x73    "invalid opcode"
	//      0x74    "invalid opcode"
	//      0x75    "invalid opcode"

	//      0x76    "bit	6, (ix/iy + offset)"
	void	do_bit_6_memat_idx_plus_offset(char wIndex, byte offset)
	{
		inline_BIT((char) 6, m_bus.MemReadByte((char)(((int)wIndex) + offset)) );
	}       

	//      0x77    "invalid opcode"
	//      0x78    "invalid opcode"
	//      0x79    "invalid opcode"
	//      0x7A    "invalid opcode"
	//      0x7B    "invalid opcode"
	//      0x7C    "invalid opcode"
	//      0x7D    "invalid opcode"

	//      0x7E    "bit	7, (ix/iy + offset)"
	void	do_bit_7_memat_idx_plus_offset(char wIndex, byte offset)
	{
		inline_BIT((char) 7, m_bus.MemReadByte((char)(((int)wIndex) + offset)) );
	}       

	//      0x7F    "invalid opcode"
	//      0x80    "invalid opcode"
	//      0x81    "invalid opcode"
	//      0x82    "invalid opcode"
	//      0x83    "invalid opcode"
	//      0x84    "invalid opcode"
	//      0x85    "invalid opcode"

	//      0x86    "res	0, (ix/iy + offset)"
	void	do_res_0_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);

		bByte = inline_RES((char) 0, bByte );
		m_bus.MemWriteByte(wAddr, bByte);
	}       

	//      0x87    "invalid opcode"
	//      0x88    "invalid opcode"
	//      0x89    "invalid opcode"
	//      0x8A    "invalid opcode"
	//      0x8B    "invalid opcode"
	//      0x8C    "invalid opcode"
	//      0x8D    "invalid opcode"

	//      0x8E    "res	1, (ix/iy + offset)"
	void	do_res_1_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);

		bByte = inline_RES((char) 1, bByte );
		m_bus.MemWriteByte(wAddr, bByte);
	}       

	//      0x8F    "invalid opcode"
	//      0x90    "invalid opcode"
	//      0x91    "invalid opcode"
	//      0x92    "invalid opcode"
	//      0x93    "invalid opcode"
	//      0x94    "invalid opcode"
	//      0x95    "invalid opcode"

	//      0x96    "res	2, (ix/iy + offset)"
	void	do_res_2_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);

		bByte = inline_RES((char)(char) 2, bByte );
		m_bus.MemWriteByte(wAddr, bByte);
	}       

	//      0x97    "invalid opcode"
	//      0x98    "invalid opcode"
	//      0x99    "invalid opcode"
	//      0x9A    "invalid opcode"
	//      0x9B    "invalid opcode"
	//      0x9C    "invalid opcode"
	//      0x9D    "invalid opcode"

	//      0x9E    "res	3, (ix/iy + offset)"
	void	do_res_3_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);

		bByte = inline_RES((char)(char) 3, bByte );
		m_bus.MemWriteByte(wAddr, bByte);
	}       

	//      0x9F    "invalid opcode"
	//      0xA0    "invalid opcode"
	//      0xA1    "invalid opcode"
	//      0xA2    "invalid opcode"
	//      0xA3    "invalid opcode"
	//      0xA4    "invalid opcode"
	//      0xA5    "invalid opcode"

	//      0xA6    "res	4, (ix/iy + offset)"
	void	do_res_4_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);

		bByte = inline_RES((char)(char) 4, bByte );
		m_bus.MemWriteByte(wAddr, bByte);
	}       

	//      0xA7    "invalid opcode"
	//      0xA8    "invalid opcode"
	//      0xA9    "invalid opcode"
	//      0xAA    "invalid opcode"
	//      0xAB    "invalid opcode"
	//      0xAC    "invalid opcode"
	//      0xAD    "invalid opcode"

	//      0xAE    "res	5, (ix/iy + offset)"
	void	do_res_5_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);

		bByte = inline_RES((char)(char) 5, bByte );
		m_bus.MemWriteByte(wAddr, bByte);
	}       

	//      0xAF    "invalid opcode"
	//      0xB0    "invalid opcode"
	//      0xB1    "invalid opcode"
	//      0xB2    "invalid opcode"
	//      0xB3    "invalid opcode"
	//      0xB4    "invalid opcode"
	//      0xB5    "invalid opcode"

	//      0xB6    "res	6, (ix/iy + offset)"
	void	do_res_6_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);

		bByte = inline_RES((char)(char) 6, bByte );
		m_bus.MemWriteByte(wAddr, bByte);
	}       

	//      0xB7    "invalid opcode"
	//      0xB8    "invalid opcode"
	//      0xB9    "invalid opcode"
	//      0xBA    "invalid opcode"
	//      0xBB    "invalid opcode"
	//      0xBC    "invalid opcode"
	//      0xBD    "invalid opcode"

	//      0xBE    "res	7, (ix/iy + offset)"
	void	do_res_7_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);

		bByte = inline_RES((char)(char) 7, bByte );
		m_bus.MemWriteByte(wAddr, bByte);
	}

	//      0xBF    "invalid opcode"
	//      0xC0    "invalid opcode"
	//      0xC1    "invalid opcode"
	//      0xC2    "invalid opcode"
	//      0xC3    "invalid opcode"
	//      0xC4    "invalid opcode"
	//      0xC5    "invalid opcode"

	//      0xC6    "set	0, (ix/iy + offset)"
	void	do_set_0_memat_idx_plus_offset(char wIndex, byte offset)
	{       
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);

		bByte = inline_SET((char) 0, bByte );
		m_bus.MemWriteByte(wAddr, bByte);
	}

	//      0xC7    "invalid opcode"
	//      0xC8    "invalid opcode"
	//      0xC9    "invalid opcode"
	//      0xCA    "invalid opcode"
	//      0xCB    "invalid opcode"
	//      0xCC    "invalid opcode"
	//      0xCD    "invalid opcode"

	//      0xCE    "set	1, (ix/iy + offset)"
	void	do_set_1_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);

		bByte = inline_SET((char) 1, bByte );
		m_bus.MemWriteByte(wAddr, bByte);
	}       

	//      0xCF    "invalid opcode"
	//      0xD0    "invalid opcode"
	//      0xD1    "invalid opcode"
	//      0xD2    "invalid opcode"
	//      0xD3    "invalid opcode"
	//      0xD4    "invalid opcode"
	//      0xD5    "invalid opcode"


	//      0xD6    "set	2, (ix/iy + offset)"
	void	do_set_2_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);

		bByte = inline_SET((char) 2, bByte );
		m_bus.MemWriteByte(wAddr, bByte);
	}

	//      0xD7    "invalid opcode"
	//      0xD8    "invalid opcode"
	//      0xD9    "invalid opcode"
	//      0xDA    "invalid opcode"
	//      0xDB    "invalid opcode"
	//      0xDC    "invalid opcode"
	//      0xDD    "invalid opcode"

	//      0xDE    "set	3, (ix/iy + offset)"
	void	do_set_3_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);

		bByte = inline_SET((char) 3, bByte );
		m_bus.MemWriteByte(wAddr, bByte);
	}       

	//      0xDF    "invalid opcode"
	//      0xE0    "invalid opcode"
	//      0xE1    "invalid opcode"
	//      0xE2    "invalid opcode"
	//      0xE3    "invalid opcode"
	//      0xE4    "invalid opcode"
	//      0xE5    "invalid opcode"

	//      0xE6    "set	4, (ix/iy + offset)"
	void	do_set_4_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);

		bByte = inline_SET((char) 4, bByte );
		m_bus.MemWriteByte(wAddr, bByte);
	}       

	//      0xE7    "invalid opcode"
	//      0xE8    "invalid opcode"
	//      0xE9    "invalid opcode"
	//      0xEA    "invalid opcode"
	//      0xEB    "invalid opcode"
	//      0xEC    "invalid opcode"
	//      0xED    "invalid opcode"

	//      0xEE    "set	5, (ix/iy + offset)"
	void	do_set_5_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);

		bByte = inline_SET((char) 5, bByte );
		m_bus.MemWriteByte(wAddr, bByte);
	}

	//      0xEF    "invalid opcode"
	//      0xF0    "invalid opcode"
	//      0xF1    "invalid opcode"
	//      0xF2    "invalid opcode"
	//      0xF3    "invalid opcode"
	//      0xF4    "invalid opcode"
	//      0xF5    "invalid opcode"

	//      0xF6    "set	6, (ix/iy + offset)"
	void	do_set_6_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);

		bByte = inline_SET((char) 6, bByte );
		m_bus.MemWriteByte(wAddr, bByte);
	}

	//      0xF7    "invalid opcode"
	//      0xF8    "invalid opcode"
	//      0xF9    "invalid opcode"
	//      0xFA    "invalid opcode"
	//      0xFB    "invalid opcode"
	//      0xFC    "invalid opcode"
	//      0xFD    "invalid opcode"

	//      0xFE    "set	7, (ix/iy + offset)"
	void	do_set_7_memat_idx_plus_offset(char wIndex, byte offset)
	{
		char wAddr = (char)(((int)wIndex) + offset);
		char bByte = m_bus.MemReadByte(wAddr);

		bByte = inline_SET((char) 7, bByte );
		m_bus.MemWriteByte(wAddr, bByte);
	}

	//      0xFF    "invalid opcode"


	// 
	//                                                                          
	//   End of 0xDD 0xCB 0x?? and 0xFD 0xCB 0x?? opcode instructions				
	//                                                                          
	// 

	// 
	//                                                                          
	//   Invalid opcode handler for 0xDD 0x?? and 0xFD 0x?? opcode sequences		
	//                                                                          
	// 

	void	do_invalid_0xDDXX_0xFDXX_opcode	(char wIndex)
	{
//		char	bOpcode = m_bus.MemReadByte((char)(m_PC-1));

//		if (lpfnOpcodeExceptionHandler)
//			(*lpfnOpcodeExceptionHandler)( REG_PC - 2, bOpcode, 2, m_dwOEH_UserData );
	}

	// 
	//                                                                          
	//   Invalid opcode handler for 0xDD 0xCB 0x?? and 0xFD 0xCB 0x?? opcodes	
	//                                                                          
	// 

	void	do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode	( char wIndex, byte offset )
	{
//		char	bOpcode = m_bus.MemReadByte((char)(m_PC-1));

//		if (lpfnOpcodeExceptionHandler)
//			(*lpfnOpcodeExceptionHandler)( REG_PC - 4, bOpcode, 4, m_dwOEH_UserData );
	}


	// 
	//                                                                          
	//   Invalid opcode handler for 0xED 0x?? opcode sequences                  
	//                                                                          
	// 

	void	do_invalid_0xEDXX_opcode()
	{
//		char	bOpcode = m_bus.MemReadByte((char)(m_PC-1));

//		if (lpfnOpcodeExceptionHandler)
//			(*lpfnOpcodeExceptionHandler)( REG_PC - 2, bOpcode, 2, m_dwOEH_UserData );
	}


	void Exec0xCBOpcode(char bOpcode)
	{
		//	Do it...
		switch(bOpcode)
		{
			case 0x00:
				do_rlc_b();											//	00
				break;
			case 0x01:
				do_rlc_c();     // 01
				break;
			case 0x02:
				do_rlc_d();     // 02
				break;
			case 0x03:
				do_rlc_e();     // 03
				break;
			case 0x04:
				do_rlc_h();     // 04
				break;
			case 0x05:
				do_rlc_l();     // 05
				break;
			case 0x06:
				do_rlc_memat_hl();     // 06
				break;
			case 0x07:
				do_rlc_a();     // 07
				break;
			case 0x08:
				do_rrc_b();     // 08
				break;
			case 0x09:
				do_rrc_c();     // 09
				break;
			case 0x0A:
				do_rrc_d();     // 0A
				break;
			case 0x0B:
				do_rrc_e();     // 0B
				break;
			case 0x0C:
				do_rrc_h();     // 0C
				break;
			case 0x0D:
				do_rrc_l();     // 0D
				break;
			case 0x0E:
				do_rrc_memat_hl();     // 0E
				break;
			case 0x0F:
				do_rrc_a();     // 0F
				break;
			case 0x10:
				do_rl_b();      // 10
				break;
			case 0x11:
				do_rl_c();      // 11
				break;
			case 0x12:
				do_rl_d();      // 12
				break;
			case 0x13:
				do_rl_e();      // 13
				break;
			case 0x14:
				do_rl_h();      // 14
				break;
			case 0x15:
				do_rl_l();      // 15
				break;
			case 0x16:
				do_rl_memat_hl();      // 16
				break;
			case 0x17:
				do_rl_a();      // 17
				break;
			case 0x18:
				do_rr_b();      // 18
				break;
			case 0x19:
				do_rr_c();      // 19
				break;
			case 0x1A:
				do_rr_d();      // 1A
				break;
			case 0x1B:
				do_rr_e();      // 1B
				break;
			case 0x1C:
				do_rr_h();      // 1C
				break;
			case 0x1D:
				do_rr_l();      // 1D
				break;
			case 0x1E:
				do_rr_memat_hl();      // 1E
				break;
			case 0x1F:
				do_rr_a();      // 1F
				break;
			case 0x20:
				do_sla_b();											//	20
				break;
			case 0x21:
				do_sla_c();											//	21
				break;
			case 0x22:
				do_sla_d();     // 22
				break;
			case 0x23:
				do_sla_e();     // 23
				break;
			case 0x24:
				do_sla_h();     // 24
				break;
			case 0x25:
				do_sla_l();     // 25
				break;
			case 0x26:
				do_sla_memat_hl();     // 26
				break;
			case 0x27:
				do_sla_a();     // 27
				break;
			case 0x28:
				do_sra_b();     // 28
				break;
			case 0x29:
				do_sra_c();     // 29
				break;
			case 0x2A:
				do_sra_d();     // 2A
				break;
			case 0x2B:
				do_sra_e();     // 2B
				break;
			case 0x2C:
				do_sra_h();     // 2C
				break;
			case 0x2D:
				do_sra_l();     // 2D
				break;
			case 0x2E:
				do_sra_memat_hl();     // 2E
				break;
			case 0x2F:
				do_sra_a();     // 2F
				break;
			case 0x30:
				do_sll_b();											//	30
				break;
			case 0x31:
				do_sll_c();											//	31
				break;
			case 0x32:
				do_sll_d();											//	32
				break;
			case 0x33:
				do_sll_e();											//	33
				break;
			case 0x34:
				do_sll_h();											//	34
				break;
			case 0x35:
				do_sll_l();											//	35
				break;
			case 0x36:
				do_sll_memat_hl();									//	36
				break;
			case 0x37:
				do_sll_a();											//	37
				break;
			case 0x38:
				do_srl_b();     // 38
				break;
			case 0x39:
				do_srl_c();     // 39
				break;
			case 0x3A:
				do_srl_d();     // 3A
				break;
			case 0x3B:
				do_srl_e();     // 3B
				break;
			case 0x3C:
				do_srl_h();     // 3C
				break;
			case 0x3D:
				do_srl_l();     // 3D
				break;
			case 0x3E:
				do_srl_memat_hl();     // 3E
				break;
			case 0x3F:
				do_srl_a();     // 3F
				break;
			case 0x40:
				do_bit_0_b();   // 40
				break;
			case 0x41:
				do_bit_0_c();   // 41
				break;
			case 0x42:
				do_bit_0_d();   // 42
				break;
			case 0x43:
				do_bit_0_e();   // 43
				break;
			case 0x44:
				do_bit_0_h();   // 44
				break;
			case 0x45:
				do_bit_0_l();   // 45
				break;
			case 0x46:
				do_bit_0_memat_hl();   // 46
				break;
			case 0x47:
				do_bit_0_a();   // 47
				break;
			case 0x48:
				do_bit_1_b();   // 48
				break;
			case 0x49:
				do_bit_1_c();   // 49
				break;
			case 0x4A:
				do_bit_1_d();   // 4A
				break;
			case 0x4B:
				do_bit_1_e();   // 4B
				break;
			case 0x4C:
				do_bit_1_h();   // 4C
				break;
			case 0x4D:
				do_bit_1_l();   // 4D
				break;
			case 0x4E:
				do_bit_1_memat_hl();   // 4E
				break;
			case 0x4F:
				do_bit_1_a();   // 4F
				break;
			case 0x50:
				do_bit_2_b();   // 50
				break;
			case 0x51:
				do_bit_2_c();   // 51
				break;
			case 0x52:
				do_bit_2_d();   // 52
				break;
			case 0x53:
				do_bit_2_e();   // 53
				break;
			case 0x54:
				do_bit_2_h();   // 54
				break;
			case 0x55:
				do_bit_2_l();   // 55
				break;
			case 0x56:
				do_bit_2_memat_hl();   // 56
				break;
			case 0x57:
				do_bit_2_a();   // 57
				break;
			case 0x58:
				do_bit_3_b();   // 58
				break;
			case 0x59:
				do_bit_3_c();   // 59
				break;
			case 0x5A:
				do_bit_3_d();   // 5A
				break;
			case 0x5B:
				do_bit_3_e();   // 5B
				break;
			case 0x5C:
				do_bit_3_h();   // 5C
				break;
			case 0x5D:
				do_bit_3_l();   // 5D
				break;
			case 0x5E:
				do_bit_3_memat_hl();   // 5E
				break;
			case 0x5F:
				do_bit_3_a();   // 5F
				break;
			case 0x60:
				do_bit_4_b();   // 60
				break;
			case 0x61:
				do_bit_4_c();   // 61
				break;
			case 0x62:
				do_bit_4_d();   // 62
				break;
			case 0x63:
				do_bit_4_e();   // 63
				break;
			case 0x64:
				do_bit_4_h();   // 64
				break;
			case 0x65:
				do_bit_4_l();   // 65
				break;
			case 0x66:
				do_bit_4_memat_hl();   // 66
				break;
			case 0x67:
				do_bit_4_a();   // 67
				break;
			case 0x68:
				do_bit_5_b();   // 68
				break;
			case 0x69:
				do_bit_5_c();   // 69
				break;
			case 0x6A:
				do_bit_5_d();   // 6A
				break;
			case 0x6B:
				do_bit_5_e();   // 6B
				break;
			case 0x6C:
				do_bit_5_h();   // 6C
				break;
			case 0x6D:
				do_bit_5_l();   // 6D
				break;
			case 0x6E:
				do_bit_5_memat_hl();   // 6E
				break;
			case 0x6F:
				do_bit_5_a();   // 6F
				break;
			case 0x70:
				do_bit_6_b();   // 70
				break;
			case 0x71:
				do_bit_6_c();   // 71
				break;
			case 0x72:
				do_bit_6_d();   // 72
				break;
			case 0x73:
				do_bit_6_e();   // 73
				break;
			case 0x74:
				do_bit_6_h();   // 74
				break;
			case 0x75:
				do_bit_6_l();   // 75
				break;
			case 0x76:
				do_bit_6_memat_hl();   // 76
				break;
			case 0x77:
				do_bit_6_a();   // 77
				break;
			case 0x78:
				do_bit_7_b();   // 78
				break;
			case 0x79:
				do_bit_7_c();   // 79
				break;
			case 0x7A:
				do_bit_7_d();   // 7A
				break;
			case 0x7B:
				do_bit_7_e();   // 7B
				break;
			case 0x7C:
				do_bit_7_h();   // 7C
				break;
			case 0x7D:
				do_bit_7_l();   // 7D
				break;
			case 0x7E:
				do_bit_7_memat_hl();   // 7E
				break;
			case 0x7F:
				do_bit_7_a();   // 7F
				break;
			case 0x80:
				do_res_0_b();   // 80
				break;
			case 0x81:
				do_res_0_c();   // 81
				break;
			case 0x82:
				do_res_0_d();   // 82
				break;
			case 0x83:
				do_res_0_e();   // 83
				break;
			case 0x84:
				do_res_0_h();   // 84
				break;
			case 0x85:
				do_res_0_l();   // 85
				break;
			case 0x86:
				do_res_0_memat_hl();   // 86
				break;
			case 0x87:
				do_res_0_a();   // 87
				break;
			case 0x88:
				do_res_1_b();   // 88
				break;
			case 0x89:
				do_res_1_c();   // 89
				break;
			case 0x8A:
				do_res_1_d();   // 8A
				break;
			case 0x8B:
				do_res_1_e();   // 8B
				break;
			case 0x8C:
				do_res_1_h();   // 8C
				break;
			case 0x8D:
				do_res_1_l();   // 8D
				break;
			case 0x8E:
				do_res_1_memat_hl();   // 8E
				break;
			case 0x8F:
				do_res_1_a();   // 8F
				break;
			case 0x90:
				do_res_2_b();   // 90
				break;
			case 0x91:
				do_res_2_c();   // 91
				break;
			case 0x92:
				do_res_2_d();   // 92
				break;
			case 0x93:
				do_res_2_e();   // 93
				break;
			case 0x94:
				do_res_2_h();   // 94
				break;
			case 0x95:
				do_res_2_l();   // 95
				break;
			case 0x96:
				do_res_2_memat_hl();   // 96
				break;
			case 0x97:
				do_res_2_a();   // 97
				break;
			case 0x98:
				do_res_3_b();   // 98
				break;
			case 0x99:
				do_res_3_c();   // 99
				break;
			case 0x9A:
				do_res_3_d();   // 9A
				break;
			case 0x9B:
				do_res_3_e();   // 9B
				break;
			case 0x9C:
				do_res_3_h();   // 9C
				break;
			case 0x9D:
				do_res_3_l();   // 9D
				break;
			case 0x9E:
				do_res_3_memat_hl();   // 9E
				break;
			case 0x9F:
				do_res_3_a();   // 9F
				break;
			case 0xA0:
				do_res_4_b();   // A0
				break;
			case 0xA1:
				do_res_4_c();   // A1
				break;
			case 0xA2:
				do_res_4_d();   // A2
				break;
			case 0xA3:
				do_res_4_e();   // A3
				break;
			case 0xA4:
				do_res_4_h();   // A4
				break;
			case 0xA5:
				do_res_4_l();   // A5
				break;
			case 0xA6:
				do_res_4_memat_hl();   // A6
				break;
			case 0xA7:
				do_res_4_a();   // A7
				break;
			case 0xA8:
				do_res_5_b();   // A8
				break;
			case 0xA9:
				do_res_5_c();   // A9
				break;
			case 0xAA:
				do_res_5_d();   // AA
				break;
			case 0xAB:
				do_res_5_e();   // AB
				break;
			case 0xAC:
				do_res_5_h();   // AC
				break;
			case 0xAD:
				do_res_5_l();   // AD
				break;
			case 0xAE:
				do_res_5_memat_hl();   // AE
				break;
			case 0xAF:
				do_res_5_a();   // AF
				break;
			case 0xB0:
				do_res_6_b();   // B0
				break;
			case 0xB1:
				do_res_6_c();   // B1
				break;
			case 0xB2:
				do_res_6_d();   // B2
				break;
			case 0xB3:
				do_res_6_e();   // B3
				break;
			case 0xB4:
				do_res_6_h();   // B4
				break;
			case 0xB5:
				do_res_6_l();   // B5
				break;
			case 0xB6:
				do_res_6_memat_hl();   // B6
				break;
			case 0xB7:
				do_res_6_a();   // B7
				break;
			case 0xB8:
				do_res_7_b();   // B8
				break;
			case 0xB9:
				do_res_7_c();   // B9
				break;
			case 0xBA:
				do_res_7_d();   // BA
				break;
			case 0xBB:
				do_res_7_e();   // BB
				break;
			case 0xBC:
				do_res_7_h();   // BC
				break;
			case 0xBD:
				do_res_7_l();   // BD
				break;
			case 0xBE:
				do_res_7_memat_hl();   // BE
				break;
			case 0xBF:
				do_res_7_a();   // BF
				break;
			case 0xC0:
				do_set_0_b();   // C0
				break;
			case 0xC1:
				do_set_0_c();   // C1
				break;
			case 0xC2:
				do_set_0_d();   // C2
				break;
			case 0xC3:
				do_set_0_e();   // C3
				break;
			case 0xC4:
				do_set_0_h();   // C4
				break;
			case 0xC5:
				do_set_0_l();   // C5
				break;
			case 0xC6:
				do_set_0_memat_hl();   // C6
				break;
			case 0xC7:
				do_set_0_a();   // C7
				break;
			case 0xC8:
				do_set_1_b();   // C8
				break;
			case 0xC9:
				do_set_1_c();   // C9
				break;
			case 0xCA:
				do_set_1_d();   // CA
				break;
			case 0xCB:
				do_set_1_e();   // CB
				break;
			case 0xCC:
				do_set_1_h();   // CC
				break;
			case 0xCD:
				do_set_1_l();   // CD
				break;
			case 0xCE:
				do_set_1_memat_hl();   // CE
				break;
			case 0xCF:
				do_set_1_a();   // CF
				break;
			case 0xD0:
				do_set_2_b();   // D0
				break;
			case 0xD1:
				do_set_2_c();   // D1
				break;
			case 0xD2:
				do_set_2_d();   // D2
				break;
			case 0xD3:
				do_set_2_e();   // D3
				break;
			case 0xD4:
				do_set_2_h();   // D4
				break;
			case 0xD5:
				do_set_2_l();   // D5
				break;
			case 0xD6:
				do_set_2_memat_hl();   // D6
				break;
			case 0xD7:
				do_set_2_a();   // D7
				break;
			case 0xD8:
				do_set_3_b();   // D8
				break;
			case 0xD9:
				do_set_3_c();   // D9
				break;
			case 0xDA:
				do_set_3_d();   // DA
				break;
			case 0xDB:
				do_set_3_e();   // DB
				break;
			case 0xDC:
				do_set_3_h();   // DC 
				break;
			case 0xDD:
				do_set_3_l();   // DD
				break;
			case 0xDE:
				do_set_3_memat_hl();   // DE
				break;
			case 0xDF:
				do_set_3_a();   // DF
				break;
			case 0xE0:
				do_set_4_b();   // E0
				break;
			case 0xE1:
				do_set_4_c();   // E1
				break;
			case 0xE2:
				do_set_4_d();   // E2
				break;
			case 0xE3:
				do_set_4_e();   // E3
				break;
			case 0xE4:
				do_set_4_h();   // E4
				break;
			case 0xE5:
				do_set_4_l();   // E5
				break;
			case 0xE6:
				do_set_4_memat_hl();   // E6
				break;
			case 0xE7:
				do_set_4_a();   // E7
				break;
			case 0xE8:
				do_set_5_b();   // E8
				break;
			case 0xE9:
				do_set_5_c();   // E9
				break;
			case 0xEA:
				do_set_5_d();   // EA
				break;
			case 0xEB:
				do_set_5_e();   // EB
				break;
			case 0xEC:
				do_set_5_h();   // EC
				break;
			case 0xED:
				do_set_5_l();   // ED
				break;
			case 0xEE:
				do_set_5_memat_hl();   // EE
				break;
			case 0xEF:
				do_set_5_a();   // EF
				break;
			case 0xF0:
				do_set_6_b();   // F0
				break;
			case 0xF1:
				do_set_6_c();   // F1
				break;
			case 0xF2:
				do_set_6_d();   // F2
				break;
			case 0xF3:
				do_set_6_e();   // F3
				break;
			case 0xF4:
				do_set_6_h();   // F4
				break;
			case 0xF5:
				do_set_6_l();   // F5
				break;
			case 0xF6:
				do_set_6_memat_hl();   // F6
				break;
			case 0xF7:
				do_set_6_a();   // F7
				break;
			case 0xF8:
				do_set_7_b();   // F8
				break;
			case 0xF9:
				do_set_7_c();   // F9
				break;
			case 0xFA:
				do_set_7_d();   // FA
				break;
			case 0xFB:
				do_set_7_e();   // FB
				break;
			case 0xFC:
				do_set_7_h();   // FC
				break;
			case 0xFD:
				do_set_7_l();   // FD
				break;
			case 0xFE:
				do_set_7_memat_hl();   // FE
				break;
			case 0xFF:
				do_set_7_a();    // FF
				break;
		}

		m_nCycle += m_rgb0xCBOpcodeCycles[ bOpcode ];
		if (m_nCycle < 0)
			m_nCycle = -m_nCycle;
	}

	char Exec0xDD_or_0xFDOpcode(char bOpcode, char wIndex)
	{
		//	Do it...
		switch(bOpcode)
		{
			case 0x00:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x01:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x02:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x03:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x04:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x05:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x06:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x07:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x08:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x09:
				wIndex = do_add_idx_bc(wIndex);
				break;
			case 0x0A:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x0B:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x0C:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x0D:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x0E:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x0F:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x10:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x11:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x12:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x13:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x14:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x15:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x16:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x17:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x18:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x19:
				wIndex = do_add_idx_de(wIndex);
				break;
			case 0x1A:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x1B:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x1C:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x1D:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x1E:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x1F:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x20:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x21:
				wIndex = do_ld_idx_WORD();
				break;
			case 0x22:
				do_ld_memataddr_idx(wIndex);
				break;
			case 0x23:
				wIndex = do_inc_idx(wIndex);
				break;
			case 0x24:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x25:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x26:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x27:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x28:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x29:
				wIndex = do_add_idx_idx(wIndex);
				break;
			case 0x2A:
				wIndex = do_ld_idx_memataddr();
				break;
			case 0x2B:
				wIndex = do_dec_idx(wIndex);
				break;
			case 0x2C:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x2D:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x2E:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x2F:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x30:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x31:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x32:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x33:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x34:
				do_inc_memat_idx_plus_offset(wIndex);
				break;
			case 0x35:
				do_dec_memat_idx_plus_offset(wIndex);
				break;
			case 0x36:
				do_ld_memat_idx_plus_offset_BYTE(wIndex);
				break;
			case 0x37:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x38:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x39:
				wIndex = do_add_idx_sp(wIndex);
				break;
			case 0x3A:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x3B:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x3C:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x3D:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x3E:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x3F:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x40:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x41:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x42:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x43:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x44:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x45:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x46:
				do_ld_b_memat_idx_plus_offset(wIndex);
				break;
			case 0x47:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x48:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x49:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x4A:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x4B:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x4C:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x4D:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x4E:
				do_ld_c_memat_idx_plus_offset(wIndex);
				break;
			case 0x4F:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x50:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x51:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x52:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x53:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x54:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x55:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x56:
				do_ld_d_memat_idx_plus_offset(wIndex);
				break;
			case 0x57:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x58:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x59:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x5A:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x5B:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x5C:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x5D:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x5E:
				do_ld_e_memat_idx_plus_offset(wIndex);
				break;
			case 0x5F:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x60:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x61:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x62:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x63:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x64:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x65:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x66:
				do_ld_h_memat_idx_plus_offset(wIndex);
				break;
			case 0x67:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x68:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x69:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x6A:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x6B:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x6C:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x6D:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x6E:
				do_ld_l_memat_idx_plus_offset(wIndex);
				break;
			case 0x6F:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x70:
				do_ld_memat_idx_plus_offset_b(wIndex);
				break;
			case 0x71:
				do_ld_memat_idx_plus_offset_c(wIndex);
				break;
			case 0x72:
				do_ld_memat_idx_plus_offset_d(wIndex);
				break;
			case 0x73:
				do_ld_memat_idx_plus_offset_e(wIndex);
				break;
			case 0x74:
				do_ld_memat_idx_plus_offset_h(wIndex);
				break;
			case 0x75:
				do_ld_memat_idx_plus_offset_l(wIndex);
				break;
			case 0x76:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x77:
				do_ld_memat_idx_plus_offset_a(wIndex);
				break;
			case 0x78:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x79:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x7A:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x7B:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x7C:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x7D:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x7E:
				do_ld_a_memat_idx_plus_offset(wIndex);
				break;
			case 0x7F:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x80:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x81:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x82:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x83:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x84:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x85:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x86:
				do_add_a_memat_idx_plus_offset(wIndex);
				break;
			case 0x87:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x88:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x89:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x8A:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x8B:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x8C:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x8D:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x8E:
				do_adc_a_memat_idx_plus_offset(wIndex);
				break;
			case 0x8F:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x90:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x91:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x92:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x93:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x94:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x95:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x96:
				do_sub_a_memat_idx_plus_offset(wIndex);
				break;
			case 0x97:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x98:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x99:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x9A:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x9B:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x9C:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x9D:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0x9E:
				do_sbc_a_memat_idx_plus_offset(wIndex);
				break;
			case 0x9F:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xA0:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xA1:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xA2:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xA3:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xA4:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xA5:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xA6:
				do_and_memat_idx_plus_offset(wIndex);
				break;
			case 0xA7:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xA8:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xA9:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xAA:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xAB:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xAC:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xAD:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xAE:
				do_xor_memat_idx_plus_offset(wIndex);
				break;
			case 0xAF:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xB0:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xB1:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xB2:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xB3:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xB4:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xB5:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xB6:
				do_or_memat_idx_plus_offset(wIndex);
				break;
			case 0xB7:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xB8:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xB9:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xBA:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xBB:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xBC:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xBD:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xBE:
				do_cp_memat_idx_plus_offset(wIndex);
				break;
			case 0xBF:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xC0:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xC1:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xC2:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xC3:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xC4:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xC5:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xC6:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xC7:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xC8:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xC9:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xCA:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xCB:
				do_0xDDCB_or_0xFDCB_instruction(wIndex);
				break;
			case 0xCC:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xCD:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xCE:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xCF:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xD0:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xD1:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xD2:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xD3:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xD4:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xD5:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xD6:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xD7:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xD8:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xD9:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xDA:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xDB:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xDC:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xDD:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xDE:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xDF:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xE0:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xE1:
				wIndex = do_pop_idx();
				break;
			case 0xE2:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xE3:
				do_ex_memat_sp_idx(wIndex);
				break;
			case 0xE4:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xE5:
				do_push_idx(wIndex);
				break;
			case 0xE6:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xE7:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xE8:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xE9:
				do_jp_memat_idx(wIndex);
				break;
			case 0xEA:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xEB:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xEC:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xED:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xEE:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xEF:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xF0:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xF1:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xF2:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xF3:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xF4:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xF5:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xF6:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xF7:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xF8:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xF9:
				do_ld_sp_idx(wIndex);
				break;
			case 0xFA:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xFB:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xFC:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xFD:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xFE:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
			case 0xFF:
				do_invalid_0xDDXX_0xFDXX_opcode(wIndex);
				break;
		}

		m_nCycle += m_rgb_0xDD_and_0xFD_OpcodeCycles[ bOpcode ];
		if (m_nCycle < 0)
			m_nCycle = -m_nCycle;

		return	wIndex;
	}

	void Exec0xEDOpcode(char bOpcode)
	{
		//	Do it...
		switch(bOpcode)
		{
			case 0x00:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x01:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x02:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x03:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x04:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x05:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x06:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x07:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x08:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x09:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x0A:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x0B:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x0C:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x0D:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x0E:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x0F:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x10:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x11:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x12:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x13:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x14:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x15:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x16:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x17:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x18:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x19:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x1A:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x1B:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x1C:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x1D:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x1E:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x1F:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x20:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x21:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x22:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x23:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x24:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x25:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x26:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x27:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x28:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x29:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x2A:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x2B:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x2C:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x2D:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x2E:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x2F:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x30:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x31:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x32:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x33:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x34:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x35:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x36:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x37:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x38:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x39:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x3A:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x3B:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x3C:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x3D:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x3E:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x3F:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x40:
				do_in_b_portin_c();
				break;
			case 0x41:
				do_out_portin_c_b();
				break;
			case 0x42:
				do_sbc_hl_bc();
				break;
			case 0x43:
				do_ld_memataddr_bc();
				break;
			case 0x44:
				do_negate();
				break;
			case 0x45:
				do_retn();
				break;
			case 0x46:
				do_im_0();
				break;
			case 0x47:
				do_ld_i_a();
				break;
			case 0x48:
				do_in_c_portin_c();
				break;
			case 0x49:
				do_out_portin_c_c();
				break;
			case 0x4A:
				do_adc_hl_bc();
				break;
			case 0x4B:
				do_ld_bc_memataddr();
				break;
			case 0x4C:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x4D:
				do_reti();
				break;
			case 0x4E:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x4F:
				do_ld_r_a();
				break;
			case 0x50:
				do_in_d_portin_c();
				break;
			case 0x51:
				do_out_portin_c_d();
				break;
			case 0x52:
				do_sbc_hl_de();
				break;
			case 0x53:
				do_ld_memataddr_de();
				break;
			case 0x54:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x55:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x56:
				do_im_1();
				break;
			case 0x57:
				do_ld_a_i();
				break;
			case 0x58:
				do_in_e_portin_c();
				break;
			case 0x59:
				do_out_portin_c_e();
				break;
			case 0x5A:
				do_adc_hl_de();
				break;
			case 0x5B:
				do_ld_de_memataddr();
				break;
			case 0x5C:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x5D:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x5E:
				do_im_2();
				break;
			case 0x5F:
				do_ld_a_r();
				break;
			case 0x60:
				do_in_h_portin_c();
				break;
			case 0x61:
				do_out_portin_c_h();
				break;
			case 0x62:
				do_sbc_hl_hl();
				break;
			case 0x63:
				do_ld_memataddr_hl();
				break;
			case 0x64:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x65:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x66:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x67:
				do_rrd();
				break;
			case 0x68:
				do_in_l_portin_c();
				break;
			case 0x69:
				do_out_portin_c_l();
				break;
			case 0x6A:
				do_adc_hl_hl();
				break;
			case 0x6B:
				do_ld_hl_memataddr();
				break;
			case 0x6C:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x6D:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x6E:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x6F:
				do_rld();
				break;
			case 0x70:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x71:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x72:
				do_sbc_hl_sp();
				break;
			case 0x73:
				do_ld_memataddr_sp();
				break;
			case 0x74:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x75:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x76:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x77:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x78:
				do_in_a_portin_c();
				break;
			case 0x79:
				do_out_portin_c_a();
				break;
			case 0x7A:
				do_adc_hl_sp();
				break;
			case 0x7B:
				do_ld_sp_memataddr();
				break;
			case 0x7C:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x7D:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x7E:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x7F:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x80:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x81:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x82:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x83:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x84:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x85:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x86:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x87:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x88:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x89:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x8A:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x8B:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x8C:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x8D:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x8E:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x8F:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x90:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x91:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x92:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x93:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x94:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x95:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x96:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x97:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x98:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x99:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x9A:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x9B:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x9C:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x9D:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x9E:
				do_invalid_0xEDXX_opcode();
				break;
			case 0x9F:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xA0:
				do_ldi();
				break;
			case 0xA1:
				do_cpi();
				break;
			case 0xA2:
				do_ini();
				break;
			case 0xA3:
				do_outi();
				break;
			case 0xA4:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xA5:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xA6:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xA7:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xA8:
				do_ldd();
				break;
			case 0xA9:
				do_cpd();
				break;
			case 0xAA:
				do_ind();
				break;
			case 0xAB:
				do_outd();
				break;
			case 0xAC:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xAD:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xAE:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xAF:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xB0:
				do_ldir();
				break;
			case 0xB1:
				do_cpir();
				break;
			case 0xB2:
				do_inir();
				break;
			case 0xB3:
				do_otir();
				break;
			case 0xB4:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xB5:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xB6:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xB7:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xB8:
				do_lddr();
				break;
			case 0xB9:
				do_cpdr();
				break;
			case 0xBA:
				do_indr();
				break;
			case 0xBB:
				do_otdr();
				break;
			case 0xBC:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xBD:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xBE:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xBF:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xC0:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xC1:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xC2:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xC3:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xC4:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xC5:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xC6:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xC7:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xC8:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xC9:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xCA:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xCB:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xCC:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xCD:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xCE:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xCF:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xD0:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xD1:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xD2:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xD3:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xD4:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xD5:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xD6:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xD7:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xD8:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xD9:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xDA:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xDB:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xDC:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xDD:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xDE:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xDF:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xE0:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xE1:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xE2:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xE3:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xE4:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xE5:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xE6:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xE7:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xE8:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xE9:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xEA:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xEB:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xEC:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xED:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xEE:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xEF:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xF0:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xF1:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xF2:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xF3:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xF4:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xF5:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xF6:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xF7:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xF8:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xF9:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xFA:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xFB:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xFC:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xFD:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xFE:
				do_invalid_0xEDXX_opcode();
				break;
			case 0xFF:
				do_invalid_0xEDXX_opcode();
				break;
		}

		m_nCycle += m_rgb0xEDOpcodeCycles[ bOpcode ];
		if (m_nCycle < 0)
			m_nCycle = -m_nCycle;
	}


	void Exec0xDDCB_or_0xFDCBOpcode(char bOpcode, char wIndex, byte offset)
	{
		switch(bOpcode)
		{
			case 0x00:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x01:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x02:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x03:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x04:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x05:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x06:
				do_rlc_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x07:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x08:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x09:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x0A:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x0B:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x0C:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x0D:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x0E:
				do_rrc_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x0F:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x10:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x11:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x12:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x13:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x14:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x15:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x16:
				do_rl_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x17:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x18:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x19:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x1A:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x1B:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x1C:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x1D:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x1E:
				do_rr_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x1F:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x20:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x21:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x22:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x23:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x24:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x25:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x26:
				do_sla_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x27:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x28:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x29:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x2A:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x2B:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x2C:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x2D:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x2E:
				do_sra_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x2F:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x30:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x31:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x32:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x33:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x34:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x35:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x36:
				do_sll_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x37:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x38:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x39:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x3A:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x3B:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x3C:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x3D:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x3E:
				do_srl_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x3F:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x40:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x41:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x42:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x43:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x44:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x45:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x46:
				do_bit_0_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x47:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x48:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x49:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x4A:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x4B:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x4C:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x4D:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x4E:
				do_bit_1_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x4F:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x50:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x51:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x52:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x53:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x54:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x55:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x56:
				do_bit_2_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x57:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x58:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x59:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x5A:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x5B:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x5C:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x5D:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x5E:
				do_bit_3_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x5F:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x60:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x61:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x62:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x63:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x64:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x65:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x66:
				do_bit_4_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x67:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x68:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x69:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x6A:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x6B:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x6C:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x6D:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x6E:
				do_bit_5_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x6F:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x70:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x71:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x72:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x73:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x74:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x75:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x76:
				do_bit_6_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x77:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x78:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x79:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x7A:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x7B:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x7C:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x7D:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x7E:
				do_bit_7_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x7F:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x80:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x81:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x82:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x83:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x84:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x85:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x86:
				do_res_0_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x87:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x88:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x89:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x8A:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x8B:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x8C:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x8D:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x8E:
				do_res_1_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x8F:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x90:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x91:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x92:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x93:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x94:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x95:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x96:
				do_res_2_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x97:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x98:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x99:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x9A:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x9B:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x9C:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x9D:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0x9E:
				do_res_3_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0x9F:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xA0:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xA1:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xA2:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xA3:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xA4:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xA5:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xA6:
				do_res_4_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0xA7:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xA8:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xA9:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xAA:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xAB:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xAC:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xAD:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xAE:
				do_res_5_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0xAF:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xB0:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xB1:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xB2:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xB3:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xB4:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xB5:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xB6:
				do_res_6_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0xB7:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xB8:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xB9:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xBA:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xBB:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xBC:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xBD:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xBE:
				do_res_7_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0xBF:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xC0:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xC1:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xC2:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xC3:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xC4:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xC5:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xC6:
				do_set_0_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0xC7:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xC8:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xC9:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xCA:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xCB:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xCC:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xCD:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xCE:
				do_set_1_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0xCF:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xD0:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xD1:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xD2:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xD3:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xD4:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xD5:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xD6:
				do_set_2_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0xD7:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xD8:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xD9:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xDA:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xDB:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xDC:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xDD:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xDE:
				do_set_3_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0xDF:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xE0:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xE1:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xE2:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xE3:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xE4:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xE5:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xE6:
				do_set_4_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0xE7:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xE8:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xE9:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xEA:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xEB:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xEC:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xED:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xEE:
				do_set_5_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0xEF:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xF0:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xF1:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xF2:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xF3:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xF4:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xF5:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xF6:
				do_set_6_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0xF7:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xF8:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xF9:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xFA:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xFB:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xFC:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xFD:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
			case 0xFE:
				do_set_7_memat_idx_plus_offset(wIndex, offset);
				break;
			case 0xFF:
				do_invalid_0xDDCBXXXX_0xFDCBXXXX_opcode(wIndex, offset);
				break;
		}
		
		m_nCycle += m_rgb_0xDDCB_and_0xFDCB_OpcodeCycles[ bOpcode ];
		if (m_nCycle < 0)
			m_nCycle = -m_nCycle;
	}

	void ExecSingleOpcode(char bOpcode)
	{
		switch(bOpcode)
		{
			case 0x00:
				do_nop();												// 00
				break;
			case 0x01:
				do_ld_bc_word();      // 01
				break;
			case 0x02:
				do_ld_memat_bc_a();    // 02
				break;
			case 0x03:
				do_inc_bc();    // 03
				break;
			case 0x04:
				do_inc_b();     // 04
				break;
			case 0x05:
				do_dec_b();     // 05
				break;
			case 0x06:
				do_ld_b_byte();// 06
				break;
			case 0x07:
				do_rlca();      // 07
				break;
			case 0x08:
				do_ex_af_afprime();    // 08
				break;
			case 0x09:
				do_add_hl_bc(); // 09
				break;
			case 0x0A:
				do_ld_a_memat_bc();    // 0A
				break;
			case 0x0B:
				do_dec_bc();    // 0B
				break;
			case 0x0C:
				do_inc_c();     // 0C
				break;
			case 0x0D:
				do_dec_c();     // 0D
				break;
			case 0x0E:
				do_ld_c_BYTE();// 0E
				break;
			case 0x0F:
				do_rrca();      // 0F
				break;
			case 0x10:
				do_djnz_offset();      // 10
				break;
			case 0x11:
				do_ld_de_WORD();      // 11
				break;
			case 0x12:
				do_ld_memat_de_a();    // 12
				break;
			case 0x13:
				do_inc_de();    // 13
				break;
			case 0x14:
				do_inc_d();     // 14
				break;
			case 0x15:
				do_dec_d();     // 15
				break;
			case 0x16:
				do_ld_d_BYTE();// 16
				break;
			case 0x17:
				do_rla();// 17
				break;
			case 0x18:
				do_jr_offset(); // 18
				break;
			case 0x19:
				do_add_hl_de(); // 19
				break;
			case 0x1A:
				do_ld_a_memat_de();    // 1A
				break;
			case 0x1B:
				do_dec_de();    // 1B
				break;
			case 0x1C:
				do_inc_e();     // 1C
				break;
			case 0x1D:
				do_dec_e();     // 1D
				break;
			case 0x1E:
				do_ld_e_BYTE();// 1E
				break;
			case 0x1F:
				do_rra();// 1F
				break;
			case 0x20:
				do_jr_nz_offset();     // 20
				break;
			case 0x21:
				do_ld_hl_WORD();      // 21
				break;
			case 0x22:
				do_ld_memataddr_hl();  // 22
				break;
			case 0x23:
				do_inc_hl();    // 23
				break;
			case 0x24:
				do_inc_h();     // 24
				break;
			case 0x25:
				do_dec_h();     // 25
				break;
			case 0x26:
				do_ld_h_BYTE();// 26
				break;
			case 0x27:
				do_daa();// 27
				break;
			case 0x28:
				do_jr_z_offset();      // 28
				break;
			case 0x29:
				do_add_hl_hl(); // 29
				break;
			case 0x2A:
				do_ld_hl_memataddr();  // 2A
				break;
			case 0x2B:
				do_dec_hl();    // 2B
				break;
			case 0x2C:
				do_inc_l();     // 2C
				break;
			case 0x2D:
				do_dec_l();     // 2D
				break;
			case 0x2E:
				do_ld_l_BYTE();// 2E
				break;
			case 0x2F:
				do_cpl();// 2F
				break;
			case 0x30:
				do_jr_nc_offset();     // 30
				break;
			case 0x31:
				do_ld_sp_WORD();      // 31
				break;
			case 0x32:
				do_ld_memataddr_a();   // 32
				break;
			case 0x33:
				do_inc_sp();    // 33
				break;
			case 0x34:
				do_inc_memat_hl();     // 34
				break;
			case 0x35:
				do_dec_memat_hl();     // 35
				break;
			case 0x36:
				do_ld_memat_hl_BYTE();// 36
				break;
			case 0x37:
				do_scf();// 37
				break;
			case 0x38:
				do_jr_c_offset();      // 38
				break;
			case 0x39:
				do_add_hl_sp(); // 39
				break;
			case 0x3A:
				do_ld_a_memataddr();   // 3A
				break;
			case 0x3B:
				do_dec_sp();    // 3B
				break;
			case 0x3C:
				do_inc_a();     // 3C
				break;
			case 0x3D:
				do_dec_a();     // 3D
				break;
			case 0x3E:
				do_ld_a_BYTE();// 3E
				break;
			case 0x3F:
				do_ccf();// 3F
				break;
			case 0x40:
				do_ld_b_b();    // 40
				break;
			case 0x41:
				do_ld_b_c();    // 41
				break;
			case 0x42:
				do_ld_b_d();    // 42
				break;
			case 0x43:
				do_ld_b_e();    // 43
				break;
			case 0x44:
				do_ld_b_h();    // 44
				break;
			case 0x45:
				do_ld_b_l();    // 45
				break;
			case 0x46:
				do_ld_b_memat_hl();    // 46
				break;
			case 0x47:
				do_ld_b_a();    // 47
				break;
			case 0x48:
				do_ld_c_b();    // 48
				break;
			case 0x49:
				do_ld_c_c();    // 49
				break;
			case 0x4A:
				do_ld_c_d();    // 4A
				break;
			case 0x4B:
				do_ld_c_e();    // 4B
				break;
			case 0x4C:
				do_ld_c_h();    // 4C
				break;
			case 0x4D:
				do_ld_c_l();    // 4D
				break;
			case 0x4E:
				do_ld_c_memat_hl();    // 4E
				break;
			case 0x4F:
				do_ld_c_a();    // 4F
				break;
			case 0x50:
				do_ld_d_b();    // 50
				break;
			case 0x51:
				do_ld_d_c();    // 51
				break;
			case 0x52:
				do_ld_d_d();    // 52
				break;
			case 0x53:
				do_ld_d_e();    // 53
				break;
			case 0x54:
				do_ld_d_h();    // 54
				break;
			case 0x55:
				do_ld_d_l();    // 55
				break;
			case 0x56:
				do_ld_d_memat_hl();    // 56
				break;
			case 0x57:
				do_ld_d_a();    // 57
				break;
			case 0x58:
				do_ld_e_b();    // 58
				break;
			case 0x59:
				do_ld_e_c();    // 59
				break;
			case 0x5A:
				do_ld_e_d();    // 5A
				break;
			case 0x5B:
				do_ld_e_e();    // 5B
				break;
			case 0x5C:
				do_ld_e_h();    // 5C
				break;
			case 0x5D:
				do_ld_e_l();    // 5D
				break;
			case 0x5E:
				do_ld_e_memat_hl();    // 5E
				break;
			case 0x5F:
				do_ld_e_a();    // 5F
				break;
			case 0x60:
				do_ld_h_b();    // 60
				break;
			case 0x61:
				do_ld_h_c();    // 61
				break;
			case 0x62:
				do_ld_h_d();    // 62
				break;
			case 0x63:
				do_ld_h_e();    // 63
				break;
			case 0x64:
				do_ld_h_h();    // 64
				break;
			case 0x65:
				do_ld_h_l();    // 65
				break;
			case 0x66:
				do_ld_h_memat_hl();    // 66
				break;
			case 0x67:
				do_ld_h_a();    // 67
				break;
			case 0x68:
				do_ld_l_b();    // 68
				break;
			case 0x69:
				do_ld_l_c();    // 69
				break;
			case 0x6A:
				do_ld_l_d();    // 6A
				break;
			case 0x6B:
				do_ld_l_e();    // 6B
				break;
			case 0x6C:
				do_ld_l_h();    // 6C
				break;
			case 0x6D:
				do_ld_l_l();    // 6D
				break;
			case 0x6E:
				do_ld_l_memat_hl();    // 6E
				break;
			case 0x6F:
				do_ld_l_a();    // 6F
				break;
			case 0x70:
				do_ld_memat_hl_b();    // 70
				break;
			case 0x71:
				do_ld_memat_hl_c();    // 71
				break;
			case 0x72:
				do_ld_memat_hl_d();    // 72
				break;
			case 0x73:
				do_ld_memat_hl_e();    // 73
				break;
			case 0x74:
				do_ld_memat_hl_h();    // 74
				break;
			case 0x75:
				do_ld_memat_hl_l();    // 75
				break;
			case 0x76:
				do_halt();      // 76
				break;
			case 0x77:
				do_ld_memat_hl_a();    // 77
				break;
			case 0x78:
				do_ld_a_b();    // 78
				break;
			case 0x79:
				do_ld_a_c();    // 79
				break;
			case 0x7A:
				do_ld_a_d();    // 7A
				break;
			case 0x7B:
				do_ld_a_e();    // 7B
				break;
			case 0x7C:
				do_ld_a_h();    // 7C
				break;
			case 0x7D:
				do_ld_a_l();    // 7D
				break;
			case 0x7E:
				do_ld_a_memat_hl();    // 7E
				break;
			case 0x7F:
				do_ld_a_a();    // 7F
				break;
			case 0x80:
				do_add_a_b();   // 80
				break;
			case 0x81:
				do_add_a_c();   // 81
				break;
			case 0x82:
				do_add_a_d();   // 82
				break;
			case 0x83:
				do_add_a_e();   // 83
				break;
			case 0x84:
				do_add_a_h();   // 84
				break;
			case 0x85:
				do_add_a_l();   // 85
				break;
			case 0x86:
				do_add_a_memat_hl();   // 86
				break;
			case 0x87:
				do_add_a_a();   // 87
				break;
			case 0x88:
				do_adc_a_b();   // 88
				break;
			case 0x89:
				do_adc_a_c();   // 89
				break;
			case 0x8A:
				do_adc_a_d();   // 8A
				break;
			case 0x8B:
				do_adc_a_e();   // 8B
				break;
			case 0x8C:
				do_adc_a_h();   // 8C
				break;
			case 0x8D:
				do_adc_a_l();   // 8D
				break;
			case 0x8E:
				do_adc_a_memat_hl();   // 8E
				break;
			case 0x8F:
				do_adc_a_a();   // 8F
				break;
			case 0x90:
				do_sub_a_b();   // 90
				break;
			case 0x91:
				do_sub_a_c();   // 91
				break;
			case 0x92:
				do_sub_a_d();   // 92
				break;
			case 0x93:
				do_sub_a_e();   // 93
				break;
			case 0x94:
				do_sub_a_h();   // 94
				break;
			case 0x95:
				do_sub_a_l();   // 95
				break;
			case 0x96:
				do_sub_a_memat_hl();   // 96
				break;
			case 0x97:
				do_sub_a_a();   // 97
				break;
			case 0x98:
				do_sbc_a_b();   // 98
				break;
			case 0x99:
				do_sbc_a_c();   // 99
				break;
			case 0x9A:
				do_sbc_a_d();   // 9A
				break;
			case 0x9B:
				do_sbc_a_e();   // 9B
				break;
			case 0x9C:
				do_sbc_a_h();   // 9C
				break;
			case 0x9D:
				do_sbc_a_l();   // 9D
				break;
			case 0x9E:
				do_sbc_a_memat_hl();   // 9E
				break;
			case 0x9F:
				do_sbc_a_a();   // 9F
				break;
			case 0xA0:
				do_and_b();     // A0
				break;
			case 0xA1:
				do_and_c();     // A1
				break;
			case 0xA2:
				do_and_d();     // A2
				break;
			case 0xA3:
				do_and_e();     // A3
				break;
			case 0xA4:
				do_and_h();     // A4
				break;
			case 0xA5:
				do_and_l();     // A5
				break;
			case 0xA6:
				do_and_memat_hl();     // A6
				break;
			case 0xA7:
				do_and_a();     // A7
				break;
			case 0xA8:
				do_xor_b();     // A8
				break;
			case 0xA9:
				do_xor_c();     // A9
				break;
			case 0xAA:
				do_xor_d();     // AA
				break;
			case 0xAB:
				do_xor_e();     // AB
				break;
			case 0xAC:
				do_xor_h();     // AC
				break;
			case 0xAD:
				do_xor_l();     // AD
				break;
			case 0xAE:
				do_xor_memat_hl();     // AE
				break;
			case 0xAF:
				do_xor_a();     // AF
				break;
			case 0xB0:
				do_or_b();      // B0
				break;
			case 0xB1:
				do_or_c();      // B1
				break;
			case 0xB2:
				do_or_d();      // B2
				break;
			case 0xB3:
				do_or_e();      // B3
				break;
			case 0xB4:
				do_or_h();      // B4
				break;
			case 0xB5:
				do_or_l();      // B5
				break;
			case 0xB6:
				do_or_memat_hl();      // B6
				break;
			case 0xB7:
				do_or_a();      // B7
				break;
			case 0xB8:
				do_cp_b();      // B8
				break;
			case 0xB9:
				do_cp_c();      // B9
				break;
			case 0xBA:
				do_cp_d();      // BA
				break;
			case 0xBB:
				do_cp_e();      // BB
				break;
			case 0xBC:
				do_cp_h();      // BC
				break;
			case 0xBD:
				do_cp_l();      // BD
				break;
			case 0xBE:
				do_cp_memat_hl();      // BE
				break;
			case 0xBF:
				do_cp_a();      // BF
				break;
			case 0xC0:
				do_ret_nz();    // C0
				break;
			case 0xC1:
				do_pop_bc();    // C1
				break;
			case 0xC2:
				do_jp_nz_addr();// C2
				break;
			case 0xC3:
				do_jp_addr();   // C3
				break;
			case 0xC4:
				do_call_nz_addr();     // C4
				break;
			case 0xC5:
				do_push_bc();   // C5
				break;
			case 0xC6:
				do_add_a_BYTE();      // C6
				break;
			case 0xC7:
				do_rst_0x00();  // C7
				break;
			case 0xC8:
				do_ret_z();     // C8
				break;
			case 0xC9:
				do_ret();// C9
				break;
			case 0xCA:
				do_jp_z_address();     // CA
				break;
			case 0xCB:
				do_0xCB_instruction(); // CB
				break;
			case 0xCC:
				do_call_z_addr();      // CC
				break;
			case 0xCD:
				do_call_addr(); // CD
				break;
			case 0xCE:
				do_adc_a_BYTE();      // CE
				break;
			case 0xCF:
				do_rst_0x08();  // CF
				break;
			case 0xD0:
				do_ret_nc();    // D0
				break;
			case 0xD1:
				do_pop_de();    // D1
				break;
			case 0xD2:
				do_jp_nc_addr();// D2
				break;
			case 0xD3:
				do_out_port_a();// D3
				break;
			case 0xD4:
				do_call_nc_addr();     // D4
				break;
			case 0xD5:
				do_push_de();   // D5
				break;
			case 0xD6:
				do_sub_a_BYTE();      // D6
				break;
			case 0xD7:
				do_rst_0x10();  // D7
				break;
			case 0xD8:
				do_ret_c();     // D8
				break;
			case 0xD9:
				do_exx();// D9
				break;
			case 0xDA:
				do_jp_c_addr(); // DA
				break;
			case 0xDB:
				do_in_a_port(); // DB
				break;
			case 0xDC:
				do_call_c_addr();      // DC
				break;
			case 0xDD:
				do_0xDD_instruction(); // DD
				break;
			case 0xDE:
				do_sbc_a_BYTE();      // DE
				break;
			case 0xDF:
				do_rst_0x18();  // DF
				break;
			case 0xE0:
				do_ret_po();    // E0
				break;
			case 0xE1:
				do_pop_hl();    // E1
				break;
			case 0xE2:
				do_jp_po_addr();// E2
				break;
			case 0xE3:
				do_ex_memat_sp_hl();   // E3
				break;
			case 0xE4:
				do_call_po_addr();     // E4
				break;
			case 0xE5:
				do_push_hl();   // E5
				break;
			case 0xE6:
				do_and_BYTE(); // E6
				break;
			case 0xE7:
				do_rst_0x20();  // E7
				break;
			case 0xE8:
				do_ret_pe();    // E8
				break;
			case 0xE9:
				do_jp_memat_hl();      // E9
				break;
			case 0xEA:
				do_jp_pe_addr();// EA
				break;
			case 0xEB:
				do_ex_de_hl();  // EB
				break;
			case 0xEC:
				do_call_pe_addr();     // EC
				break;
			case 0xED:
				do_0xED_instruction(); // ED
				break;
			case 0xEE:
				do_xor_BYTE(); // EE
				break;
			case 0xEF:
				do_rst_0x28();  // EF
				break;
			case 0xF0:
				do_ret_p();     // F0
				break;
			case 0xF1:
				do_pop_af();    // F1
				break;
			case 0xF2:
				do_jp_p_addr(); // F2
				break;
			case 0xF3:
				do_di(); // F3
				break;
			case 0xF4:
				do_call_p_addr();      // F4
				break;
			case 0xF5:
				do_push_af();   // F5
				break;
			case 0xF6:
				do_or_BYTE();  // F6
				break;
			case 0xF7:
				do_rst_0x30();  // F7
				break;
			case 0xF8:
				do_ret_m();     // F8
				break;
			case 0xF9:
				do_ld_sp_hl();  // F9
				break;
			case 0xFA:
				do_jp_m_addr(); // FA
				break;
			case 0xFB:
				do_ei(); // FB
				break;
			case 0xFC:
				do_call_m_addr();      // FC
				break;
			case 0xFD:
				do_0xFD_instruction(); // FD
				break;
			case 0xFE:
				do_cp_BYTE();  // FE
				break;
			case 0xFF:
				do_rst_0x38();   // FF
				break;
		}

		// Update the cycle counter
		m_nCycle += m_rgbSingleOpcodeCycles[ bOpcode ];

		if (m_nCycle < 0)
			m_nCycle = -m_nCycle;
	}

	public char timedCycle(int nCycles)
	{
		char	wPCAtExit = m_PC;
		int	nCycleAtStart = m_nCycle;
		int	nCycleToStop = m_nCycle + nCycles;
		char	bOpcode = 0;

		do
		{
//			if (fDebug)
//				System.out.println(	"AF: 0x" + Integer.toHexString((int)m_PC) + " " +
//											"BC: 0x" + Integer.toHexString((int)m_BC) + " " +
//											"DE: 0x" + Integer.toHexString((int)m_DE) + " " +
//											"HL: 0x" + Integer.toHexString((int)m_HL) + " " +
//											"SP: 0x" + Integer.toHexString((int)m_SP)
//										);
			bOpcode = m_bus.MemReadByte( m_PC++ );

			ExecSingleOpcode(bOpcode);

			// Increment refresh register
			m_R++;

			// NMI triggered?
			if (m_fNMIFired)
			{
				// Reset NMI interrupt flag
				m_fNMIFired = false;

				// Take care of halted state
				if (m_fHalted)
				{
					m_PC++;				// Inc past HALT instruction
					m_fHalted = false;	// Reset flag
				}

				// handle a non-maskable interrupt
				m_SP -= 2;
				m_bus.MemWriteWord( m_SP, m_PC );
				m_IFF2 = m_IFF1;
				m_IFF1 = false;
				m_PC = (char)0x0066;
				continue;
			}

			// If an interrupt has occurred and interrupts are
			//		enabled, then handle it appropriately.
			if (m_fInterrupt && m_IFF1)
			{
				// Disable interrupts
				m_IFF1 = m_IFF2 = false;

				// Reset interrupt state...
				m_fInterrupt = false;
				
				// Take care of halted state.
				if (m_fHalted)
				{
					m_PC++;				// Inc past HALT instruction
					m_fHalted = false;	// Reset flag
				}
				
				switch( m_nInterruptMode )
				{
					// handle interrupt mode 0
					case	0:
						{
							char cExec;
							
							for (cExec=0; cExec<6; cExec++)
								ExecSingleOpcode(m_bInterruptData);
						}
						break;

					// handle interrupt mode 1
					case	1:
						m_SP -= 2;
						m_bus.MemWriteWord( m_SP, m_PC );
						m_PC = (char)0x0038;
						break;

					// handle interrupt mode 2
					case	2:
						m_SP -= 2;
						m_bus.MemWriteWord( m_SP, m_PC );
						m_PC = (char)((m_I << 8) | (m_bInterruptData & 0xFF));
						m_bInterruptData = 0xFF;
						break;
				}
			}

			wPCAtExit = m_PC;

		} while(((m_nCycle <= nCycleToStop) &&
					(m_nCycle >= nCycleAtStart)));

		return	wPCAtExit;

	}

	public boolean reset()
	{
		// Reset registers...
		m_AF = 0x0000;
		m_BC = 0x0000;
		m_DE = 0x0000;
		m_HL = 0x0000;
		m_IX = 0x0000;
		m_IY = 0x0000;
		m_SP = 0xF000;
		m_PC = 0x0000;
		m_AF_PRIME = 0x0000;
		m_BC_PRIME = 0x0000;
		m_DE_PRIME = 0x0000;
		m_HL_PRIME = 0x0000;
		m_I = 0x00;
		m_R = 0x00;
		m_IFF1 = false;
		m_IFF2 = false;

		// Reset states...
		m_nInterruptMode = 0;
		m_fInterrupt = false;
		m_fNMIFired = false;
		m_fHalted = false;
		m_nCycle = 0;
		return	true;
	}

	public void generateNMI()
	{
		m_fNMIFired = true;
	}

	public int getCycleCount()
	{
		return	m_nCycle;
	}

	public char getPC()
	{
		return	m_PC;
	}
}
